%Bayesian inference surface plot
% Plots z = P(H|T) vs x = P(H) and y = P(T|H)
%
% Assumes symmetric condition P(T|H') = 1 - P(T|H) = 1 - y
%
% LAST UPDATED by Andy French 17/02/2014

function bayes_surface

N = 500;
fsize = 14;

%x = P(H)
x = linspace(0,1,N);

%y = P(T|H)
y = linspace(0,1,N);

%Compute z = P(H|T)
[x,y] = meshgrid(x,y);
z = 1./( (1-x).*(1-y)./(x.*y) + 1 );

%Plot surface
figure;
pcolor(x,y,z);
shading interp
interp_colormap(N);
colorbar
xlabel('P(H)','fontsize',fsize)
ylabel('P(T|H)','fontsize',fsize)
title('Bayesian Inference P(H|T)','fontsize',fsize)
set(gca,'fontsize',fsize)
print(gcf,'bayes_surface','-dpng','-r300')
close(gcf);

%%

%interp_colormap
% Function which interpolates current colourmap to yield better graduated
% shading. N is number of possible colours.
%
% LAST UPDATED by Andy French 14 September 2008

function interp_colormap(N)

%Get current colourmap
map = colormap;

%Initialise new colormap
new_map = ones(N,3);

%Get size of current colormap and initalise red,green,blue vectors
dim = size(map);
R = ones(1,dim(1));
G = ones(1,dim(1));
B = ones(1,dim(1));
RR = ones(1,N);
GG = ones(1,N);
BB = ones(1,N);

%Populate these with current colormap
R(:) = map(:,1);
G(:) = map(:,2);
B(:) = map(:,3);

%Interpolate to yield new colour map
x = linspace( 1, dim(1), N );
RR = interp1( 1:dim(1), R, x );
GG = interp1( 1:dim(1), G, x );
BB = interp1( 1:dim(1), B, x );
new_map(:,1) = RR(:);
new_map(:,2) = GG(:);
new_map(:,3) = BB(:);

%Set colormap to be new map
colormap( new_map );

%End of code