%find_p_from_P_and_n_and_x
% Find Binomial distribuition trial 'success' probability p given Binomial
% probability P(x) and number of trials n.
% LAST UPDATED by Andy French 25/Feb/2025
function find_p_from_P_and_n_and_x
P = 0.07;  N = 12; x = 7;

%Step through p and work out Binomial probabilities
p = linspace(0,1,10000); P_Bin = nchoosek(N,x).*(p.^x).*(1-p).^(N-x) ;

%Plot p vs P_Bin and overlay P
plot( p, P_Bin,'r-' ); hold on; plot( [0,1],P*[1,1],'b-'); 
xlabel('Success probability p'); ylabel('Binomial probability P'); grid on;

%Step through P_Bin and work out pairs of p values which straddle the
%solution to P_Bin = P
for n=2:length(p)
    if ( P_Bin(n)>P ) && ( P_Bin(n-1) <= P )
        p1 = p(n-1);  p2 = p(n);
    elseif ( P_Bin(n)<=P ) && ( P_Bin(n-1) > P )
        p3 = p(n-1);  p4 = p(n);
    end
end

%Return approximate solutions 
p_solutions = [ 0.5*(p1+p2), 0.5*(p3+p4) ];

%Set graph title and plot a PNG file
title( {['Binomial probability P( ',num2str(x),'|',num2str(N),',p )'],...
    ['p = ',num2str(p_solutions(1)),', ',num2str(p_solutions(2))] });
print(gcf,'P vs p.png','-dpng','-r300'); close(gcf);

%%

%Binomial probability calculator
function P_Bin = B(n,x,p)
P_Bin = nchoosek(n,x).*(p.^x).*(1-p).^(n-x) ;
%End of code