% cipher
% Substitution cipher generation and solver assistant.
%
% Andy French December 2018.

%%%% GUI INITIALIZATION %%%%

%Top level GUI initialization
function varargout = cipher(varargin)

% Generic MATLAB GUI initialization. Edit at your peril!
gui_Singleton = 1;
gui_State = struct('gui_Name',       mfilename, ...
    'gui_Singleton',  gui_Singleton, ...
    'gui_OpeningFcn', @cipher_OpeningFcn, ...
    'gui_OutputFcn',  @cipher_OutputFcn, ...
    'gui_LayoutFcn',  [] , ...
    'gui_Callback',   []);
if nargin && ischar(varargin{1})
    gui_State.gui_Callback = str2func(varargin{1});
end

if nargout
    [varargout{1:nargout}] = gui_mainfcn(gui_State, varargin{:});
else
    gui_mainfcn(gui_State, varargin{:});
end

%%

% Outputs from this function are returned to the command line.
function varargout = cipher_OutputFcn(hObject, eventdata, handles)
varargout{1} = handles.output;

%%

% Code which executes just before cipher is made visible.
function cipher_OpeningFcn(hObject, eventdata, handles, varargin)

% Choose default command line output for cipher
handles.output = hObject;
guidata(hObject, handles);

%Add all subdirectories within current directory to MATLAB path
addpath( genpath(pwd), '-begin');

%Store GUI object handles data in a global array d which can be seen (and
%modified) by all functions. This will be the main GUI data array which
%enables all objects to communicate with one another.
global d
d.h = handles; clear handles

%Centre figure
figcentre(d.h.FIGUREcipher);

%Define default random cipherbet
d.alphabet = 'ABCDEFGHIJKLMNOPQRSTUVWXYZ';
d.N = length(d.alphabet);
d.cipherbet = d.alphabet( randperm(d.N) );
d.loaded_cipherbet = d.cipherbet;

%Set default mode to decrypt
d.encrypt = 0;

%Initialize empty plaintext and ciphertext
d.plaintext = '';
d.ciphertext = '';

%Set up default GUI data
set( d.h.EDITnumcharacters,'string',num2str( length(d.plaintext) ) );
set( d.h.EDITnumwords,'string',num2str( wordcount(d.plaintext) ) );
set( d.h.LISTplaintext, 'string', string_vector2cell(d.plaintext) );
set( d.h.LISTciphertext, 'string', string_vector2cell(d.ciphertext) );
update_gui_cipherbet;

%%

%Load new plaintext pushbutton
function PUSHloadplaintext_Callback(hObject, eventdata, handles)
global d
[filename,pathname] = uigetfile( '*.txt','Load plaintext' );
if filename~=0
    f = fopen([pathname,filename],'r');
    d.plaintext = upper( fscanf(f, '%c') );
    d.plaintext( double(d.plaintext)==13 ) = [];
    fclose(f);
else
    d.plaintext = '';
end
set( d.h.LISTplaintext, 'string', string_vector2cell(d.plaintext) );
d.ciphertext = encrypt( d.plaintext, d.alphabet, d.cipherbet );
set( d.h.LISTciphertext, 'string', string_vector2cell(d.ciphertext) );
set( d.h.EDITnumcharacters,'string',num2str( length(d.plaintext) ) );
set( d.h.EDITnumwords,'string',num2str( wordcount(d.plaintext) ) );

%%

%Load new ciphertext pushbutton
function PUSHloadciphertext_Callback(hObject, eventdata, handles)
global d
[filename,pathname] = uigetfile( '*.txt','Load ciphertext' );
if filename~=0
    f = fopen([pathname,filename],'r');
    d.ciphertext = upper( fscanf(f, '%c') );
    d.ciphertext( double(d.ciphertext)==13 ) = [];
    fclose(f);
else
    d.ciphertext = '';
end
set( d.h.LISTciphertext, 'string', string_vector2cell(d.ciphertext) );
d.plaintext = decrypt( d.ciphertext, d.alphabet, d.cipherbet );
set( d.h.LISTplaintext, 'string', string_vector2cell(d.plaintext) );
set( d.h.EDITnumcharacters,'string',num2str( length(d.plaintext) ) );
set( d.h.EDITnumwords,'string',num2str( wordcount(d.plaintext) ) );

%%

%Generate new random cipherbet
function PUSHnewcipherbet_Callback(hObject, eventdata, handles)
global d
d.cipherbet = d.alphabet( randperm(d.N) );
%If in encrypt mode, update ciphertext
if d.encrypt==1;
    d.ciphertext = encrypt( d.plaintext, d.alphabet, d.cipherbet );
    set( d.h.LISTciphertext, 'string', string_vector2cell(d.ciphertext) );
else
    %Otherwise if in decrypt mode, upate plaintext
    d.plaintext = decrypt( d.ciphertext, d.alphabet, d.cipherbet );
    set( d.h.LISTplaintext, 'string', string_vector2cell(d.plaintext) );
end
update_gui_cipherbet;

%%

%Clear cipherbet - i.e. set all as '*'
function PUSHclearcipherbet_Callback(hObject, eventdata, handles)
global d
d.cipherbet = '';
for n=1:d.N;
    d.cipherbet = [d.cipherbet,'*'];
end
%If in encrypt mode mode, update ciphertext
if d.encrypt==1;
    d.ciphertext = encrypt( d.plaintext, d.alphabet, d.cipherbet );
    set( d.h.LISTciphertext, 'string', string_vector2cell(d.ciphertext) );
else
    %Otherwise if in decrypt mode edit mode, upate plaintext
    d.plaintext = decrypt( d.ciphertext, d.alphabet, d.cipherbet );
    set( d.h.LISTplaintext, 'string', string_vector2cell(d.plaintext) );
end
update_gui_cipherbet;

%%

%Update cipherbet edit boxes
function EDITA_Callback(hObject, eventdata, handles)
global d; update_cipherbet_editbox(d.h.EDITA,1);
function EDITB_Callback(hObject, eventdata, handles)
global d; update_cipherbet_editbox(d.h.EDITB,2);
function EDITC_Callback(hObject, eventdata, handles)
global d; update_cipherbet_editbox(d.h.EDITC,3);
function EDITD_Callback(hObject, eventdata, handles)
global d; update_cipherbet_editbox(d.h.EDITD,4);
function EDITE_Callback(hObject, eventdata, handles)
global d; update_cipherbet_editbox(d.h.EDITE,5);
function EDITF_Callback(hObject, eventdata, handles)
global d; update_cipherbet_editbox(d.h.EDITF,6);
function EDITG_Callback(hObject, eventdata, handles)
global d; update_cipherbet_editbox(d.h.EDITG,7);
function EDITH_Callback(hObject, eventdata, handles)
global d; update_cipherbet_editbox(d.h.EDITH,8);
function EDITI_Callback(hObject, eventdata, handles)
global d; update_cipherbet_editbox(d.h.EDITI,9);
function EDITJ_Callback(hObject, eventdata, handles)
global d; update_cipherbet_editbox(d.h.EDITJ,10);
function EDITK_Callback(hObject, eventdata, handles)
global d; update_cipherbet_editbox(d.h.EDITK,11);
function EDITL_Callback(hObject, eventdata, handles)
global d; update_cipherbet_editbox(d.h.EDITL,12);
function EDITM_Callback(hObject, eventdata, handles)
global d; update_cipherbet_editbox(d.h.EDITM,13);
function EDITN_Callback(hObject, eventdata, handles)
global d; update_cipherbet_editbox(d.h.EDITN,14);
function EDITO_Callback(hObject, eventdata, handles)
global d; update_cipherbet_editbox(d.h.EDITO,15);
function EDITP_Callback(hObject, eventdata, handles)
global d; update_cipherbet_editbox(d.h.EDITP,16);
function EDITQ_Callback(hObject, eventdata, handles)
global d; update_cipherbet_editbox(d.h.EDITQ,17);
function EDITR_Callback(hObject, eventdata, handles)
global d; update_cipherbet_editbox(d.h.EDITR,18);
function EDITS_Callback(hObject, eventdata, handles)
global d; update_cipherbet_editbox(d.h.EDITS,19);
function EDITT_Callback(hObject, eventdata, handles)
global d; update_cipherbet_editbox(d.h.EDITT,20);
function EDITU_Callback(hObject, eventdata, handles)
global d; update_cipherbet_editbox(d.h.EDITU,21);
function EDITV_Callback(hObject, eventdata, handles)
global d; update_cipherbet_editbox(d.h.EDITV,22);
function EDITW_Callback(hObject, eventdata, handles)
global d; update_cipherbet_editbox(d.h.EDITW,23);
function EDITX_Callback(hObject, eventdata, handles)
global d; update_cipherbet_editbox(d.h.EDITX,24);
function EDITY_Callback(hObject, eventdata, handles)
global d; update_cipherbet_editbox(d.h.EDITY,25);
function EDITZ_Callback(hObject, eventdata, handles)
global d; update_cipherbet_editbox(d.h.EDITZ,26);

%%

%ENCRYPT or DECRYPT mode toggle
function TOGGLEmode_Callback(hObject, eventdata, handles)
global d
if strcmp( get(d.h.TOGGLEmode,'string'),'ENCRYPT' );
    d.encrypt = 0;
    set( d.h.TOGGLEmode,'string','DECRYPT','backgroundcolor',[1,0,0] );
else
    d.encrypt = 1;
    set( d.h.TOGGLEmode,'string','ENCRYPT','backgroundcolor',[0,1,0] );
end

%%

%Pushbutton to show loaded cipherbet
function PUSHshowcipherbet_Callback(hObject, eventdata, handles)
global d
d.cipherbet = d.loaded_cipherbet;
update_gui_cipherbet
d.plaintext = decrypt( d.ciphertext, d.alphabet, d.cipherbet );
    set( d.h.LISTplaintext, 'string', string_vector2cell(d.plaintext) );

%%

%Load settings from .mat file push button
function PUSHload_Callback(hObject, eventdata, handles)
global d
[filename,pathname] = uigetfile( '*.mat','Select cipher settings' );
if filename~=0
    
    %Load data from .mat file
    load( [pathname,filename] );
    d.cipherbet = cipherbet;
    d.loaded_cipherbet = d.cipherbet;
    d.plaintext = plaintext;
    d.ciphertext = ciphertext;
    
    %Update plaintext and ciphertext list boxes
    set( d.h.LISTplaintext,'string',string_vector2cell(d.plaintext) );
    set( d.h.LISTciphertext,'string',string_vector2cell(d.ciphertext) );
    set( d.h.EDITnumcharacters,'string',num2str( length(d.plaintext) ) );
    set( d.h.EDITnumwords,'string',num2str( wordcount(d.plaintext) ) );
    
    %Update cipherbet
    update_gui_cipherbet;
    
    %Save plaintext.txt and ciphertext.txt files
    f = fopen('plaintext.txt','w+');
    fwrite( f, d.plaintext );
    fclose(f);
    f = fopen('ciphertext.txt','w+');
    fwrite( f, d.ciphertext );
    fclose(f);
    
    %Start with a cleared cipherbet
    PUSHclearcipherbet_Callback
end

%%

%Save settings to .mat file push button
function PUSHsave_Callback(hObject, eventdata, handles)
global d
filename = ['cipher ',strrep( datestr(now),':','-' )];
plaintext = d.plaintext;
ciphertext = d.ciphertext;
alphabet = d.alphabet;
cipherbet = d.cipherbet;
save( filename, 'plaintext','ciphertext','alphabet','cipherbet' );

%Save plaintext.txt and ciphertext.txt files
f = fopen('plaintext.txt','w+');
fwrite( f, d.plaintext );
fclose(f);
f = fopen('ciphertext.txt','w+');
fwrite( f, d.ciphertext );
fclose(f);

%%

%Open plaintext in text editor, after firstly saving it to a file
function PUSHplaintextopen_Callback(hObject, eventdata, handles)
global d
f = fopen('plaintext.txt','w+');
fwrite( f, d.plaintext );
fclose(f);
open('plaintext.txt');

%%

%Open ciphertext in text editor, after firstly saving it to a file
function PUSHciphertextopen_Callback(hObject, eventdata, handles)
global d
f = fopen('ciphertext.txt','w+');
fwrite( f, d.ciphertext );
fclose(f);
open('ciphertext.txt');

%%

%Callback for cipherbet character n edit box with handle h
function update_cipherbet_editbox(h,n)
global d

%Get letter and check that it is (i) in the alphabet and (ii) not already
%used in the cipherbet
c = get(h, 'string');
in_alphabet = strfind( d.alphabet, c );
in_cipherbet = strfind( d.cipherbet, c );
if isempty(in_alphabet) || (length( in_cipherbet ) == 1 )
    %Not a valid character of cipherbet
    set(h, 'string','*');
    c = '*';
end
%Update cipherbet
d.cipherbet(n) = c;

%If in enrypt mode, update ciphertext
if d.encrypt==1
    d.ciphertext = encrypt( d.plaintext, d.alphabet, d.cipherbet );
    set( d.h.LISTciphertext, 'string', string_vector2cell(d.ciphertext) );
else
    %Otherwise if decrypt mode, upate plaintext
    d.plaintext = decrypt( d.ciphertext, d.alphabet, d.cipherbet );
    set( d.h.LISTplaintext, 'string', string_vector2cell(d.plaintext) );
end

%%

%Update cipherbet text boxes in GUI
function update_gui_cipherbet
global d
set( d.h.EDITA,'string',d.cipherbet(1) );
set( d.h.EDITB,'string',d.cipherbet(2) );
set( d.h.EDITC,'string',d.cipherbet(3) );
set( d.h.EDITD,'string',d.cipherbet(4) );
set( d.h.EDITE,'string',d.cipherbet(5) );
set( d.h.EDITF,'string',d.cipherbet(6) );
set( d.h.EDITG,'string',d.cipherbet(7) );
set( d.h.EDITH,'string',d.cipherbet(8) );
set( d.h.EDITI,'string',d.cipherbet(9) );
set( d.h.EDITJ,'string',d.cipherbet(10) );
set( d.h.EDITK,'string',d.cipherbet(11) );
set( d.h.EDITL,'string',d.cipherbet(12) );
set( d.h.EDITM,'string',d.cipherbet(13) );
set( d.h.EDITN,'string',d.cipherbet(14) );
set( d.h.EDITO,'string',d.cipherbet(15) );
set( d.h.EDITP,'string',d.cipherbet(16) );
set( d.h.EDITQ,'string',d.cipherbet(17) );
set( d.h.EDITR,'string',d.cipherbet(18) );
set( d.h.EDITS,'string',d.cipherbet(19) );
set( d.h.EDITT,'string',d.cipherbet(20) );
set( d.h.EDITU,'string',d.cipherbet(21) );
set( d.h.EDITV,'string',d.cipherbet(22) );
set( d.h.EDITW,'string',d.cipherbet(23) );
set( d.h.EDITX,'string',d.cipherbet(24) );
set( d.h.EDITY,'string',d.cipherbet(25) );
set( d.h.EDITZ,'string',d.cipherbet(26) );

%%

%encrypt
% Function which encrypts a text string s using a substitution cipher
% and supplied alphabet and cipherbet (which must be of the same length)
function x = encrypt( s, alphabet, cipherbet )
x = s;
for n=1:length(alphabet)
    x( strfind( upper(s), alphabet(n) ) ) = cipherbet(n);
end

%%

%decrypt
% Function which decrypts a text string s using a substitution cipher
% and supplied alphabet and cipherbet (which must be of the same length)
function x = decrypt( s, alphabet, cipherbet )

%Initialize decrypted string as the ciphertext
s = upper(s);
x = s;

%Replace characters that are in the alphabet, but not on the cipherbet
%by an '*'
alphabet_only = '';
for n=1:length(alphabet)
    if isempty( strfind( cipherbet,alphabet(n) ) )
        alphabet_only = [alphabet_only, alphabet(n)];
    end
end
for n=1:length(alphabet_only);
    x( strfind( s, alphabet_only(n) ) )  = '*';
end

%Decrypt rest of x
for n=1:length(cipherbet)
    %Replace strings in s that match nth cipherbet character with
    %equivalent alphabet character
    if strcmp( cipherbet(n), '*' )==0
        x( strfind( s, cipherbet(n) ) ) = alphabet(n);
    end
end

%%

%single_str
% Converts a multi-line string ss to a single string with line return
% character char(10)
function s = single_str(ss)
s = ss(:)';
s(2,:) = {char(10)};
s = s(:)';
s(end) = [];
s = cat(2, s{:});

%%

%wordcount
% Crudely estimates number of words in a text string s, based upon the number of
% spaces. No doubt this algorithm could be improved!
function w = wordcount(s)
w = length( strfind( strtrim(s),' ') ) + 1;

%%

% Function which converts a vector of ASCII text s to a cell array c, with each
% element being a different line of the ASCII text
function c = string_vector2cell(s)

%Initialize output cell array and element number i
c{1} = '';
i = 1;

%Step through text file and add to appropriate elements of c
for n=1:length(s)
    c{i} = [c{i},s(n)];
    if ( strcmp( s(n), char(10) ) == 1) ||...
            ( strcmp( s(n), char(13) ) == 1) ||...
            ( strcmp( s(n), char(15) ) == 1)
        i = i + 1;
        c{i} = '';
    end
end

%%

%figcenter
% Centres a figure with handle fig in the current display.

function figcentre(fig)

%Get screen dimensions in pixels.
set(0,'units','pixels') ;
screen_size=get(0,'Screensize');
display_size=get(0,'MonitorPositions');

%Get original figure units
orig_units = get(fig,'units');

%Get position vector for figure in pixels
set(fig,'units','pixels');
fig_position=get(gcf,'position') ;

%Define coordinates or figure bottom hand corner from screen bottom hand corner
%that will place the figure window in the center of the screen.
centre_x = 0.5*( screen_size(3)-fig_position(3) ) - screen_size(1);
centre_y = 0.5*( screen_size(4)-fig_position(4) ) - screen_size(2);

%Modify figure position to centre
set(fig,'position',[centre_x,centre_y,fig_position(3),fig_position(4)] );

%Reset figure units
set(fig,'units',orig_units);

%End of code
