%Generic graph plotter
%
% By Andy French February 2019

function graph_plotter

%Blank axis?
blank_axis = 1;

%Number of data points (N for smooth curve, P for discrete points)
N = 1000; P = 14;

%x,y limits
xlimits = [0,100]; ylimits = [0,550];

%Define x and y tickmarks and grid
xtick = 0 : 10 : 100;  ytick = 0 : 50 : 550;
standard_grid = 1;

%Define x,y
x = linspace( xlimits(1),xlimits(2),N );
xx = linspace( xlimits(1),xlimits(2),P );
y = 500*(2.^(-x/17));
yy = 500*(2.^(-xx/17));

%Fontsize, markersize & linewidth
fsize = 18; linewidth = 1; msize = 14;

%Equal axis scale?
equal_scale = 0;

%Axis labels and title
xlabel_str = 'Time /days';
ylabel_str = 'Activity /Bq';
title_str = 'Radioactive activity of source vs time';

%

%Plot graph
fig = figure('color',[1 1 1],'name',title_str,...
    'units','pixels','PaperPositionMode','auto');
if blank_axis==1
    x = [];y = []; xx=[]; yy=[];
end
plot(x,y,'b-','linewidth',linewidth); hold on;
if equal_scale == 1
    axis equal
end
xlim(xlimits); ylim(ylimits);
set(gca,'fontsize',fsize,'Box','on',...
    'xtick',xtick,'ytick',ytick,'linewidth',linewidth);
if standard_grid==1
    set(gca,'Xgrid','on','Ygrid','on',...
    'Xminorgrid','on','Yminorgrid','on','GridLineStyle','-');
else
    %Draw lines based upon tick marks
    grid off
    for n=1:length(xtick)
        plot( [xtick(n),xtick(n)],ylimits,'k-','linewidth',linewidth );
    end
    for n=1:length(ytick)
        plot( xlimits, [ytick(n),ytick(n)],'k-','linewidth',linewidth );
    end
end
plot(xx,yy,'r+','markersize',msize);
xlabel( xlabel_str,'fontsize',fsize );
ylabel( ylabel_str,'fontsize',fsize );
title( title_str,'fontsize',fsize );
print(gcf,[title_str,'.png'],'-dpng','-r300');
close(fig);

%End of code