%forces_gui
% GUI to assist with the teaching about vector addition of forces.
% Free body and vector sum diagrams are compared side by side. Up to five
% 2D forces can be added, in x,y components. The resultant force is
% calculated and diplayed graphically, in x,y component form and in
% magnitude and polar angle.
%
% LAST UPDATED by Andy French January 2017

function varargout = forces(varargin)

%% Initialization code - DO NOT EDIT
gui_Singleton = 1;
gui_State = struct('gui_Name',       mfilename, ...
    'gui_Singleton',  gui_Singleton, ...
    'gui_OpeningFcn', @forces_OpeningFcn, ...
    'gui_OutputFcn',  @forces_OutputFcn, ...
    'gui_LayoutFcn',  [] , ...
    'gui_Callback',   []);
if nargin && ischar(varargin{1})
    gui_State.gui_Callback = str2func(varargin{1});
end

if nargout
    [varargout{1:nargout}] = gui_mainfcn(gui_State, varargin{:});
else
    gui_mainfcn(gui_State, varargin{:});
end

%%

%% forces opening function. This executes just before forces is made visible.
function forces_OpeningFcn(hObject, eventdata, handles, varargin)

% Choose default command line output for forces
handles.output = hObject;

%Turn off warnings (e.g. 'divide by zero')
warning off

%Set up global variable d which can be accessed by all functions
global d
d.h = handles;
d.legend = [];

%Center figure
figcentre( d.h.FIGUREforces );

%Set defaults
forces_defaults

%Update forces
forces_update;

%%

% Default parameters
function forces_defaults
global d

%Plot resultant?
d.plot_resultant = 0;

%Font size for graphs
d.fsize = 18;

%Arrow size for vector diagrams
d.arrowsize = 4;
d.arrowheadsize = 0.2;

%Arrow RGB colours
d.arrow_colours{1} = [1,0,0];
d.arrow_colours{2} = [0,1,0];
d.arrow_colours{3} = [0,0,1];
d.arrow_colours{4} = [0,1,1];
d.arrow_colours{5} = [1,0,1];
d.arrow_colours{6} = [0,0,0];

%Force x,y component defaults
d.F1x = 2;
d.F1y = 3;
d.F2x = 0;
d.F2y = 1;
d.F3x = -3;
d.F3y = -1;
d.F4x = 2;
d.F4y = 4;
d.F5x = 3;
d.F5y = -4;

%Store defaults
d.F1x_default = d.F1x;
d.F1y_default = d.F1y;
d.F2x_default = d.F2x;
d.F2y_default = d.F2y;
d.F3x_default = d.F3x;
d.F3y_default = d.F3y;
d.F4x_default = d.F4x;
d.F4y_default = d.F4y;
d.F5x_default = d.F5x;
d.F5y_default = d.F5y;

%Set GUI with defaults
set( d.h.EDITF1x,'string',num2str(d.F1x) );
set( d.h.EDITF1y,'string',num2str(d.F1y) );
set( d.h.EDITF2x,'string',num2str(d.F2x) );
set( d.h.EDITF2y,'string',num2str(d.F2y) );
set( d.h.EDITF3x,'string',num2str(d.F3x) );
set( d.h.EDITF3y,'string',num2str(d.F3y) );
set( d.h.EDITF4x,'string',num2str(d.F4x) );
set( d.h.EDITF4y,'string',num2str(d.F4y) );
set( d.h.EDITF5x,'string',num2str(d.F5x) );
set( d.h.EDITF5y,'string',num2str(d.F5y) );
set( d.h.AXESfreebody,'fontsize',d.fsize );
set( d.h.AXESvectoradd,'fontsize',d.fsize );

%Initialize various GUI flags
%(don't change these, as they need to start as being zero)
d.plot_and_save_PNG = 0;
d.fix_scale = 0;

%%

%% Update GUI
function forces_update
global d

%Determine resultant force
d.Rx = d.F1x + d.F2x + d.F3x + d.F4x + d.F5x;
d.Ry = d.F1y + d.F2y + d.F3y + d.F4y + d.F5y;
d.Rmag = sqrt( d.Rx^2 + d.Ry^2 );
d.Rdir = (180/pi)*atan2( d.Ry, d.Rx );
if d.plot_resultant == 1
    set( d.h.TEXTRmag, 'string',['MAGNITUDE = ',num2str( d.Rmag,3 )]);
    set( d.h.TEXTRdir, 'string',['DIRECTION = ',num2str( d.Rdir,3 ),' deg']);
    set( d.h.EDITRx, 'string',num2str( d.Rx,3 ));
    set( d.h.EDITRy, 'string',num2str( d.Ry,3 ));
else
    set( d.h.TEXTRmag, 'string','MAGNITUDE = ');
    set( d.h.TEXTRdir, 'string','DIRECTION = ');
    set( d.h.EDITRx, 'string',' ');
    set( d.h.EDITRy, 'string',' ');
end

%Only plot vectors that are not [0;0]
c1 = notzerovec( [0,d.F1x ; 0,d.F1y] );
c2 = notzerovec( [0,d.F2x ; 0,d.F2y] );
c3 = notzerovec( [0,d.F3x ; 0,d.F3y] );
c4 = notzerovec( [0,d.F4x ; 0,d.F4y] );
c5 = notzerovec( [0,d.F5x ; 0,d.F5y] );
c6 = notzerovec( [0,d.Rx ; 0,d.Ry] );

%Plot free body diagram
if d.plot_and_save_PNG == 0;
    axes( d.h.AXESfreebody );
    cla;
else
    fig = figure('name','forces');
    axes;
end
hold on;
plot( [0,d.F1x],[0,d.F1y],'linewidth',d.arrowsize,'color',d.arrow_colours{1} );
plot( [0,d.F2x],[0,d.F2y],'linewidth',d.arrowsize,'color',d.arrow_colours{2} );
plot( [0,d.F3x],[0,d.F3y],'linewidth',d.arrowsize,'color',d.arrow_colours{3} );
plot( [0,d.F4x],[0,d.F4y],'linewidth',d.arrowsize,'color',d.arrow_colours{4} );
plot( [0,d.F5x],[0,d.F5y],'linewidth',d.arrowsize,'color',d.arrow_colours{5} );

%Plot arrows
if c1==1; drawarrow( 0, d.F1x, 0, d.F1y, d.arrowheadsize, d.arrow_colours{1} ); end;
if c2==1; drawarrow( 0, d.F2x, 0, d.F2y, d.arrowheadsize, d.arrow_colours{2} ); end;
if c3==1; drawarrow( 0, d.F3x, 0, d.F3y, d.arrowheadsize, d.arrow_colours{3} ); end;
if c4==1; drawarrow( 0, d.F4x, 0, d.F4y, d.arrowheadsize, d.arrow_colours{4} ); end;
if c5==1; drawarrow( 0, d.F5x, 0, d.F5y, d.arrowheadsize, d.arrow_colours{5} ); end;

%Plot resultant force
if d.plot_resultant == 1
    plot( [0,d.Rx],[0,d.Ry],'linewidth',d.arrowsize,'color',d.arrow_colours{6} );
    if c6==1; drawarrow( 0, d.Rx, 0, d.Ry, d.arrowheadsize, d.arrow_colours{6} ); end
end

%Axis scale
axis equal
if d.fix_scale == 0
    axis tight
    d.xlimits_freebody = get(d.h.AXESfreebody,'xlim');
    d.ylimits_freebody = get(d.h.AXESfreebody,'ylim');
else
    set( d.h.AXESfreebody,'xlim',d.xlimits_freebody );
    set( d.h.AXESfreebody,'ylim',d.ylimits_freebody );
end
grid on
box on

if d.plot_and_save_PNG == 1;
    %Save .PNG file
    filename = strrep( datestr(now),':','-' );
    filename = ['free body ',filename,'.png'];
    print( fig, filename, '-dpng','-r300' );
end

%Plot vector addition diagram
if d.plot_and_save_PNG == 0;
    axes( d.h.AXESvectoradd );
    cla;
else
    cla;
end
hold on;
x1 = d.F1x;
x2 = d.F1x+d.F2x;
x3 = d.F1x+d.F2x+d.F3x;
x4 = d.F1x+d.F2x+d.F3x+d.F4x;
x5 = d.F1x+d.F2x+d.F3x+d.F4x+d.F5x;
y1 = d.F1y;
y2 = d.F1y+d.F2y;
y3 = d.F1y+d.F2y+d.F3y;
y4 = d.F1y+d.F2y+d.F3y+d.F4y;
y5 = d.F1y+d.F2y+d.F3y+d.F4y+d.F5y;
plot( [0,x1],[0,y1],'linewidth',d.arrowsize,'color',d.arrow_colours{1} );
plot( [x1,x2],[y1,y2],...
    'linewidth',d.arrowsize,'color',d.arrow_colours{2} );
plot( [x2,x3],[y2,y3],...
    'linewidth',d.arrowsize,'color',d.arrow_colours{3} );
plot( [x3,x4],[y3,y4],...
    'linewidth',d.arrowsize,'color',d.arrow_colours{4} );
plot( [x4,x5],[y4,y5],...
    'linewidth',d.arrowsize,'color',d.arrow_colours{5} );

%Plot arrows
if c1==1; drawarrow( 0, x1, 0, y1, d.arrowheadsize, d.arrow_colours{1} ); end
if c2==1; drawarrow( x1, x2, y1, y2, d.arrowheadsize, d.arrow_colours{2} ); end
if c3==1; drawarrow( x2, x3, y2, y3, d.arrowheadsize, d.arrow_colours{3} ); end
if c4==1; drawarrow( x3, x4, y3, y4, d.arrowheadsize, d.arrow_colours{4} ); end
if c5==1; drawarrow( x4, x5, y4, y5, d.arrowheadsize, d.arrow_colours{5} ); end

%Plot resultant force
if d.plot_resultant == 1
    plot( [0,d.Rx],[0,d.Ry],'linewidth',d.arrowsize,'color',d.arrow_colours{6} );
    if c6==1; drawarrow( 0, d.Rx, 0, d.Ry, d.arrowheadsize, d.arrow_colours{6} ); end
end

%Axis scale
axis equal
if d.fix_scale == 0
    axis tight
    d.xlimits_vectoradd = get(d.h.AXESvectoradd,'xlim');
    d.ylimits_vectoradd = get(d.h.AXESvectoradd,'ylim');
else
    set( d.h.AXESvectoradd,'xlim',d.xlimits_vectoradd );
    set( d.h.AXESvectoradd,'ylim',d.ylimits_vectoradd );
end
grid on
box on

if d.plot_and_save_PNG == 1;
    %Save .PNG file
    filename = strrep( datestr(now),':','-' );
    filename = ['vector add ',filename,'.png'];
    print( fig, filename, '-dpng','-r300' );
    close(fig);
    figure(d.h.FIGUREforces);
end

%%

%% Diffractor output function. Outputs from this function are returned to the command line.
function varargout = forces_OutputFcn(hObject, eventdata, handles)
global d

% Get default command line output from handles structure
varargout{1} = d.h.output;

%%

%% Toggle fix axes scale for plot
function TOGGLEfixaxesscale_Callback(hObject, eventdata, handles)
global d
if d.fix_scale==1
    d.fix_scale = 0;
else
    d.fix_scale = 1;
end
forces_update

%%

%% Save PNG of graph
function PUSHsavepng_Callback(hObject, eventdata, handles)
global d

%Force forces_update to generate figures in a new window
d.plot_and_save_PNG = 1;
forces_update;
d.plot_and_save_PNG = 0;

%%

%% Hide vector addition graph
function CHECKhide_Callback(hObject, eventdata, handles)
global d
if get(d.h.CHECKhide,'value') == 1
    set( d.h.TEXTbiggraybox,'visible','on' );
else
    set( d.h.TEXTbiggraybox,'visible','off' );
end

%%

%% Show resultant
function TOGGLEshowresultant_Callback(hObject, eventdata, handles)
global d
if d.plot_resultant==1
    d.plot_resultant = 0;
else
    d.plot_resultant = 1;
end
forces_update

%%

%% Force 1 x component edit box
function EDITF1x_Callback(hObject, eventdata, handles)
global d
d.F1x = str2num( get( d.h.EDITF1x,'string' ));
forces_update;

%%

%% Force 1 y component edit box
function EDITF1y_Callback(hObject, eventdata, handles)
global d
d.F1y = str2num( get( d.h.EDITF1y,'string' ));
forces_update;

%%

%% Force 1 zero pushbutton
function PUSHF10_Callback(hObject, eventdata, handles)
global d
d.F1x = 0;
d.F1y = 0;
set( d.h.EDITF1x,'string','0');
set( d.h.EDITF1y,'string','0');
forces_update;

%%

%% Force 1 default pushbutton
function PUSHF1D_Callback(hObject, eventdata, handles)
global d
d.F1x = d.F1x_default;
d.F1y = d.F1y_default;
set( d.h.EDITF1x,'string',num2str( d.F1x ));
set( d.h.EDITF1y,'string',num2str( d.F1y ));
forces_update;

%%

%% Force 2 x component edit box
function EDITF2x_Callback(hObject, eventdata, handles)
global d
d.F2x = str2num( get( d.h.EDITF2x,'string' ));
forces_update;

%%

%% Force 2 y component edit box
function EDITF2y_Callback(hObject, eventdata, handles)
global d
d.F2y = str2num( get(  d.h.EDITF2y,'string' ));
forces_update;

%%

%% Force 2 zero pushbutton
function PUSHF20_Callback(hObject, eventdata, handles)
global d
d.F2x = 0;
d.F2y = 0;
set( d.h.EDITF2x,'string','0');
set( d.h.EDITF2y,'string','0');
forces_update;

%%

%% Force 2 default pushbutton
function PUSHF2D_Callback(hObject, eventdata, handles)
global d
d.F2x = d.F2x_default;
d.F2y = d.F2y_default;
set( d.h.EDITF2x,'string',num2str( d.F2x ));
set( d.h.EDITF2y,'string',num2str( d.F2y ));
forces_update;

%%

%% Force 3 x component edit box
function EDITF3x_Callback(hObject, eventdata, handles)
global d
d.F3x = str2num( get( d.h.EDITF3x,'string' ));
forces_update;

%%

%% Force 3 y component edit box
function EDITF3y_Callback(hObject, eventdata, handles)
global d
d.F3y = str2num( get( d.h.EDITF3y,'string' ));
forces_update;

%%

%% Force 3 zero pushbutton
function PUSHF30_Callback(hObject, eventdata, handles)
global d
d.F3x = 0;
d.F3y = 0;
set( d.h.EDITF3x,'string','0');
set( d.h.EDITF3y,'string','0');
forces_update;

%%

%% Force 3 default pushbutton
function PUSHF3D_Callback(hObject, eventdata, handles)
global d
d.F3x = d.F3x_default;
d.F3y = d.F3y_default;
set( d.h.EDITF3x,'string',num2str( d.F3x ));
set( d.h.EDITF3y,'string',num2str( d.F3y ));
forces_update;

%%

%% Force 4 x component edit box
function EDITF4x_Callback(hObject, eventdata, handles)
global d
d.F4x = str2num( get( d.h.EDITF4x,'string' ));
forces_update;

%%

%% Force 4 y component edit box
function EDITF4y_Callback(hObject, eventdata, handles)
global d
d.F4y = str2num( get( d.h.EDITF4y,'string' ));
forces_update;

%%

%% Force 4 zero pushbutton
function PUSHF40_Callback(hObject, eventdata, handles)
global d
d.F4x = 0;
d.F4y = 0;
set( d.h.EDITF4x,'string','0');
set( d.h.EDITF4y,'string','0');
forces_update;

%%

%% Force 4 default pushbutton
function PUSHF4D_Callback(hObject, eventdata, handles)
global d
d.F4x = d.F4x_default;
d.F4y = d.F4y_default;
set( d.h.EDITF4x,'string',num2str( d.F4x ));
set( d.h.EDITF4y,'string',num2str( d.F4y ));
forces_update;

%%

%% Force 5 x component edit box
function EDITF5x_Callback(hObject, eventdata, handles)
global d
d.F5x = str2num( get( d.h.EDITF5x,'string' ));
forces_update;

%%

%% Force 1 y component edit box
function EDITF5y_Callback(hObject, eventdata, handles)
global d
d.F5y = str2num( get( d.h.EDITF5y,'string' ));
forces_update;

%%

%% Force 5 zero pushbutton
function PUSHF50_Callback(hObject, eventdata, handles)
global d
d.F5x = 0;
d.F5y = 0;
set( d.h.EDITF5x,'string','0');
set( d.h.EDITF5y,'string','0');
forces_update;

%%

%% Force 5 default pushbutton
function PUSHF5D_Callback(hObject, eventdata, handles)
global d
d.F5x = d.F5x_default;
d.F5y = d.F5y_default;
set( d.h.EDITF5x,'string',num2str( d.F5x ));
set( d.h.EDITF5y,'string',num2str( d.F5y ));
forces_update;

%%

%figcenter
% Centres a figure with handle fig in the current display.
function figcentre(fig)

%Get screen dimensions in pixels.
set(0,'units','pixels') ;
screen_size=get(0,'Screensize');

%Get original figure units
orig_units = get(fig,'units');

%Get position vector for figure in pixels
set(fig,'units','pixels');
fig_position=get(gcf,'position') ;

%Define coordinates or figure bottom hand corner from screen bottom hand corner
%that will place the figure window in the center of the screen.
centre_x = 0.5*screen_size(3) - 0.5*fig_position(3);
centre_y = 0.5*screen_size(4) - 0.5*fig_position(4);

%Modify figure position to centre
set(fig,'position',[centre_x,centre_y,fig_position(3),fig_position(4)] );

%Reset figure units
set(fig,'units',orig_units);

%%

%drawarrow
% Function which draws an arrow in the middle of a line defined by start
% and end coordinates [x0,x1] and [y0,y1]
function a = drawarrow( x0, x1, y0, y1, arrowsize, arrow_RGB )

%Define centre of arrowhead
xc = (x0+x1)/2;
yc = (y0+y1)/2;

%Rotation angle of arrow from horizontal in order for arrow to point along
%line
t = atan2( y1-y0,x1-x0 );

%Transform arrow coordinates
a = arrowsize;
r = [ 0, 2*a, a; 0, 0, 2*a];
r = r - [a,a,a;a,a,a];
r = [ sin(t), cos(t); -cos(t), sin(t)] * r ;
r = r + [xc,xc,xc; yc,yc,yc];
a = patch(r(1,:),r(2,:),arrow_RGB);
set(a,'Edgecolor',arrow_RGB );

%%

%notzerovec
% Function which returns a 1 if the vector x is not [0,0]
function y = notzerovec(x)
if sum(abs(x))==0
    y = 0;
else
    y = 1;
end

%End of code






