%julia_ini
% Default input parameters for GUI julia.m. These will be stored in a
% structure h. Julia parameters are stored within h.J. 
% NOTE: h must be the handles array which is associated with the figure.

function h = julia_INI(h)

%%DEFAULTS%%

%Define size of bitmap image in pixels
h.J.width=800;
h.J.height=600;

%Define X width (Y width is computed using bitmap image aspect ratio)
h.J.xwidth=3.14;

%Define coordinates of plot centre
h.J.xc=-0.6;
h.J.yc=0;

%Define number of iterations
h.J.N=50;

%Define julia function of z,z0 and index
% z       - Matrix of complex numbers
% z0      - Matrix of complex numbers, same size as z
%           z0 is typically the coordinates describing the argand diagram
%           upon which z is plotted.
h.J.funcstr='z^2 +z0';

%Define julia map function
%Options: 'exp_decay' 'sin' 'cos' 'arg'  otherwise 'abs' is used
h.J.map_functions={'abs',...
    'exp_decay',...
    'sin',...
    'cos',...
    'arg'};
h.J.map_function = 'abs';

%Define julia map creation rule
%Options: 'plot z' 'abs diverge' 're and im diverge' 're or im diverge'
%Default is 'abs_diverge'
h.J.map_creation_rules={'plot z',...
    'abs diverge',...
    'abs diverge & mod(2)=0',...
    're and im diverge',...
    're or im diverge'};
h.J.map_creation_rule='abs diverge';

%Define z divergence radius
h.J.C=4;

%Define [R,G,B] colour of NaN regions of the map
h.J.NaN_colour=[1,1,1];

%Define colour range vector
h.J.clim = [0,1];

%Define colourmap
h.J.cmaps={'hsv',...     %Hue-saturation-value color map.
    'hot',...       %Black-red-yellow-white color map.
    'gray',...      %Linear gray-scale color map.
    'bone',...      %Gray-scale with tinge of blue color map.
    'copper',...    %Linear copper-tone color map.
    'pink',...      %Pastel shades of pink color map.
    'white',...     %All white color map. Use with a non-white NaN colour
    'flag',...      %Alternating red, white, blue, and black color map.
    'lines',...     %Color map with the line colors.
    'colorcube',... %Enhanced color-cube color map.
    'vga',...       %Windows colormap for 16 colors.
    'jet',...       %Variant of HSV.
    'black & white',... %Alternating black and white - similar to 'prism'                   
    'prism',...     %Prism color map.
    'cool',...      %Shades of cyan and magenta color map.
    'autumn',...    %Shades of red and yellow color map.
    'spring',...    %Shades of magenta and yellow color map.
    'winter',...    %Shades of blue and green color map.
    'summer'};      %Shades of green and yellow color map.
h.J.cmap = 'prism';

%Define maximum number of pixel in the X or Y dimensions. If an image has a
%dimension greater than this, several Julia maps will be formed and then
%stitched together. If this is not done then large images (> 2000 x 2000)
%may not be possible to form due to the large amount of memory required.
h.J.max_pixels=800;

%Binary flag which defines whether composite images are deleted
h.J.delete_composites=1;

%Binary flag to cause plots for iterations 1:N to be plotted
h.J.plot_all_iter = 0;

%Binary flag to set whether 3D plots are created in addition to the 2D julia map
h.J.threeD_surf = 0;

%Binary flag to cause combined 3D and 2D plots to be created
h.J.threeD_and_twoD_surf = 0;

%Output bitmap resolution /DPI
h.J.output_bitmap_DPI = 600;

%Output image type
h.J.output_image_type = 'png';

%

%Update GUI objects
h = set_gui_data(h);

%Initialize julia function STOP flag
h.J.STOP=0;

%Initialise Julia map axis limits bounding box parameters
aspect = h.J.height/h.J.width;
h.J.bbox_flag=0;
h.J.bbox.x1=h.J.xc-0.2*h.J.xwidth;
h.J.bbox.y1=h.J.yc-0.2*h.J.xwidth*aspect;
h.J.bbox.x2=h.J.xc+0.2*h.J.xwidth;
h.J.bbox.y2=h.J.yc+0.2*h.J.xwidth*aspect;
h.J.bbox_handle=[];

%Plot julia logo
julia_logo=imread('Functions\julia_logo.png','png');
axes(h.AXESlogo);
image(julia_logo);
axis equal;
axis tight;
axis off;

%Plot default julia map.
julia_image=imread('Functions\mandlebrot.png','png');
axes(h.AXESjulia);
x=linspace(h.J.xc-h.J.xwidth/2,h.J.xc+h.J.xwidth/2,h.J.width);
y=linspace(h.J.yc-0.5*h.J.height*h.J.xwidth/h.J.width,...
    h.J.yc+0.5*h.J.height*h.J.xwidth/h.J.width,h.J.height);
image(x,y,julia_image);
axis equal;
axis tight;
axis off

%End of code

