%julia
% Top level GUI code for julia.m This drives all the callback functions.
% Non trivial ones are the functions which run the dynamic image bounding
% box facility and the Julia map update. Note in the latter there are some
% user definable parameters : maximum numbers of pixels per Argand diagram
% dimension (images with more pixels are split into patches and then joined
% after processing) ; and a binary flag which allows composite images (the
% patches) to be deleted or saved.

function varargout = julia(varargin)

%Add Functions directory to path
addpath 'Functions' -begin

%Begin initialization code - DO NOT EDIT
gui_Singleton = 1;
gui_State = struct('gui_Name',       mfilename, ...
    'gui_Singleton',  gui_Singleton, ...
    'gui_OpeningFcn', @julia_OpeningFcn, ...
    'gui_OutputFcn',  @julia_OutputFcn, ...
    'gui_LayoutFcn',  [] , ...
    'gui_Callback',   []);
if nargin && ischar(varargin{1})
    gui_State.gui_Callback = str2func(varargin{1});
end

if nargout
    [varargout{1:nargout}] = gui_mainfcn(gui_State, varargin{:});
else
    gui_mainfcn(gui_State, varargin{:});
end
%End initialization code - DO NOT EDIT

%%

%Opening function
function julia_OpeningFcn(hObject, eventdata, handles, varargin)

%Turn off warnings
warning off;

%Get GUI information
handles = guidata(hObject);

%Set GUI window in middle of screen
figcentre(handles.FIGUREjulia);

%Set GUI defaults
handles = julia_INI(handles);

%Update handles structure
guidata(hObject, handles);

%%

%Outputs from this function are returned to the command line.
function varargout = julia_OutputFcn(hObject, eventdata, handles)
varargout{1} = [];

%%

%Run this function when julia is closed
function varargout = CloseRequestFcn(hObject, eventdata, handles)
rmpath 'Functions'
closereq;

%%

%Window Button Down Function
function varargout = WindowButtonDownFcn(hObject, eventdata, handles)

%Get GUI data
handles = guidata(hObject);

%Get current coordinates of mouse pointer in axis units measured in [x y]
%from bottom hand corner of axis window
axes(handles.AXESjulia);
cp=get(gca,'currentpoint');
x = cp(1,1);
y = cp(1,2);

%Get axis limits
xlim=get(gca,'xlim');
ylim=get(gca,'ylim');

%Delete any previous bounding boxes
delete( handles.J.bbox_handle );
handles.J.bbox_handle = [];

%Change 'draw bounding box flag' from 0 to 1 if mouse click is within axis
%and update x1,y1 fields of handles.bbox structure
if (x <= xlim(2)) && (x >= xlim(1)) && ...
        (y <= ylim(2)) & (y >= ylim(1))
    handles.J.bbox_flag=1;
    handles.J.bbox.x1=x;
    handles.J.bbox.y1=y;
else
    handles.J.bbox_flag=0;
    handles.J.bbox.x1=NaN;
    handles.J.bbox.y1=NaN;
    handles.J.bbox.x2=NaN;
    handles.J.bbox.y2=NaN;
end

%Update handles structure
guidata(hObject, handles);

%%

%Window Button Up Function
function varargout = WindowButtonUpFcn(hObject, eventdata, handles)

%Get GUI data
handles = guidata(hObject);

%Determine boundaries of box and use this to set xc, yc and x width
%parameters.
x1=handles.J.bbox.x1;
x2=handles.J.bbox.x2;
y1=handles.J.bbox.y1;
y2=handles.J.bbox.y2;
xc=0.5*(x1+x2);
yc=0.5*(y1+y2);
xwidth = abs(x2-x1);

%Update GUI based upon coordinates of bounding box
set(handles.EDITxc,'string',num2str(xc));
set(handles.EDITyc,'string',num2str(yc));
set(handles.EDITxwidth,'string',num2str( xwidth ));

%Set 'draw bounding box flag' to be 0
handles.J.bbox_flag=0;

%Change Update button to amber
set(handles.PUSHupdate,'backgroundcolor',[1,0.502,0]);

%Update handles structure
guidata(hObject, handles);

%%

%Window Button Motion Function
function varargout = WindowButtonMotionFcn(hObject, eventdata, handles)

%Get GUI data
handles = guidata(hObject);

%Check initialization has run ... Sometimes this function is called before
%the handles array has been fully populated!
try
    %If 'draw bounding box flag' is 1 then determine current point within axis
    %and draw bounding box. Fixed aspect ratio of bounding box to equal the aspect ratio
    %of the julia map image.
    image_width_pixels = str2num(get(handles.EDITwidth,'string'));
    image_height_pixels = str2num(get(handles.EDITheight,'string'));
    if handles.J.bbox_flag==1
        
        %Point to julia axes and delete previous bounding box
        axes(handles.AXESjulia);
        delete(handles.J.bbox_handle);
        handles.J.bbox_handle = [];
        
        %Get axis limits
        xlim=get(gca,'xlim');
        ylim=get(gca,'ylim');
        
        %Determine top right hand corner of modified bounding box. Make
        %sure this doesn't extend beyond the axes.
        cp=get(gca,'currentpoint');
        x = cp(1,1);
        y = cp(1,2);
        
        %Check x extent first
        if x < xlim(1)
            x = xlim(1);
        elseif x > xlim(2)
            x = xlim(2);
        end
        
        %Force aspect to be correct based upon xwidth
        xwidth = x - handles.J.bbox.x1 ;
        ywidth = y - handles.J.bbox.y1 ;
        desired_aspect = image_height_pixels / image_width_pixels;
        y = handles.J.bbox.y1 + sign(ywidth)*abs(xwidth)*desired_aspect;
        
        %Check if y is now within axis. If not fix y by the
        %axis boundary and modify x
        if y < ylim(1)
            y = ylim(1);
            ywidth = y - handles.J.bbox.y1 ;
            x = handles.J.bbox.x1 + sign(xwidth)*abs(ywidth)/desired_aspect;
        elseif y > ylim(2)
            y = ylim(2);
            ywidth = y - handles.J.bbox.y1 ;
            x = handles.J.bbox.x1 + sign(xwidth)*abs(ywidth)/desired_aspect;
        end
        
        %Plot bounding box
        handles.J.bbox.x2 = x;
        handles.J.bbox.y2 = y;
        hold on;
        handles.J.bbox_handle = plot( [ handles.J.bbox.x1,...
            handles.J.bbox.x2,...
            handles.J.bbox.x2,...
            handles.J.bbox.x1,...
            handles.J.bbox.x1 ],...
            [ handles.J.bbox.y1,...
            handles.J.bbox.y1,...
            handles.J.bbox.y2,...
            handles.J.bbox.y2,...
            handles.J.bbox.y1 ],'k--' );
        hold off;
    end
    
    %Update handles structure
    guidata(hObject, handles);
end

%%

%Reset button callabck
function PUSHreset_Callback(hObject, eventdata, handles)

%Get GUI data
handles = guidata(hObject);

%Set GUI defaults
handles=julia_INI(handles);

%Change Update button to green
set(handles.PUSHupdate,'backgroundcolor',[0,1,0.251]);

%Update handles structure
guidata(hObject, handles);

%%

%Update button callback
function PUSHupdate_Callback(hObject, eventdata, handles)

%Get data from GUI
handles = guidata(hObject);
handles = get_gui_data(handles);
J = handles.J;

%Remove bounding box
delete( J.bbox_handle );
handles.J.bbox_handle = [];

%Change Update button to green
set(handles.PUSHupdate,'backgroundcolor',[0,1,0.251]);

%Change handles.STOP flag to enable update to proceed
handles.J.STOP=0;
guidata(hObject,handles);
J = handles.J;

%If 'plot all iterations' button is pressed then loop through iterations
if J.plot_all_iter == 1
    N = 1:J.N;
else
    N = J.N;
end
for n=1:length(N)
    
    %Create 200 x 150 pixel julia map to determine normalization
    isnorm = 1;
    dont_patch = 1;
    no_bitmap = 1;
    [julia_map,x,y]=make_julia_map(200,150,J.max_pixels,J.xwidth,J.xc,J.yc,N(n),J.funcstr,...
        J.map_function,J.map_creation_rule,J.C,J.NaN_colour,J.cmap,[],[],isnorm,J.clim,dont_patch,no_bitmap);
    if isempty(julia_map)
        set(handles.PUSHupdate,'backgroundcolor',[1,0.502,0]);
        return
    end
    
    %Determine maximum and minimum values of this smaller julia map
    min_julia = min(min(real(julia_map)));
    max_julia = max(max(real(julia_map)));
    
    %Compute Julia map and save PNG bitmap to current directory
    isnorm=0;
    dont_patch = 0;
    no_bitmap = 0;
    [julia_map,x,y]=make_julia_map(J.width,J.height,J.max_pixels,J.xwidth,J.xc,J.yc,N(n),J.funcstr,...
        J.map_function,J.map_creation_rule,J.C,J.NaN_colour,J.cmap,min_julia,max_julia,isnorm,J.clim,dont_patch,no_bitmap);
    if isempty(julia_map)
        set(handles.PUSHupdate,'backgroundcolor',[1,0.502,0]);
        return
    end
    
    %Split up width
    num_x = ceil(J.width / J.max_pixels);
    
    %Split up the height
    num_y = ceil(J.height / J.max_pixels);
    
    %Stitch the images back together
    num=0;
    A=[];
    julia_image =[];
    for nx = 1:num_x
        B = [];
        for ny = 1:num_y
            num = num + 1;
            A = imread(['julia map iter=',leadingzero(N(n),3),' ',num2str(num),'.png'],'png');
            B = [B; A];
        end
        julia_image = [julia_image, B];
    end
    
    %Write stitched bitmap and delete composite images, if option set
    imwrite(julia_image,['julia map iter=',leadingzero(N(n),3),'.png'],'png');
    if J.delete_composites==1
        for k=1:num
            delete(['julia map iter=',leadingzero(N(n),3),' ',num2str(k),'.png']);
        end
    end
    
    %Update GUI image with this new image
    axes(handles.AXESjulia);
    x=linspace(J.xc-J.xwidth/2,J.xc+J.xwidth/2,J.width);
    y=linspace(J.yc-0.5*J.height*J.xwidth/J.width,J.yc+0.5*J.height*J.xwidth/J.width,J.height);
    image(x,y,julia_image);
    axis equal;
    axis tight;
    axis off;
    
    %Compute Julia surface if option is set. Use the maximum number of
    %pixels to define the surface, then zoom montage to achieve the desired
    %resolution.
    if J.threeD_surf == 1
        
        %Create julia map
        isnorm=0;
        dont_patch = 1;
        no_bitmap = 1;
        [julia_map,x,y]=make_julia_map(J.max_pixels,ceil(J.height*J.max_pixels/J.width),...
            J.max_pixels,J.xwidth,J.xc,J.yc,N(n),J.funcstr,...
            J.map_function,J.map_creation_rule,J.C,J.NaN_colour,J.cmap,min_julia,max_julia,isnorm,J.clim,dont_patch,no_bitmap);
        
        %Plot surface
        fig = figure('color',[1 1 1],'name',['julia map 3D iter=',leadingzero(N(n),3)],'renderer','opengl');
        figcentre(fig);
        ax = axes;
        surf(x,y,julia_map);
        colormap(J.cmap),
        interp_colormap(J.max_pixels);
        zlabel(J.funcstr)
        caxis(J.clim)
        shading interp;
        axis vis3d
        axis equal
        axis tight
        view([-35,64]);
        alpha(0.9);
        lighting phong
        camlight
        zoom(1.2);
        
        %Write bitmap image
        cropfig(fig,ax,1);
        print(fig,['julia map 3D iter=',leadingzero(N(n),3)],...
            ['-d',J.output_image_type], ['-r',num2str(J.output_bitmap_DPI)])
        croppic(['julia map 3D iter=',leadingzero(N(n),3),'.png'],[255,255,255]);
        close(gcf);
    end
    
    %Compute Julia surface & 2D view if option is set. Use the maximum
    %number of pixels per patch to define both surface and 2D view.
    if J.threeD_and_twoD_surf == 1
        %Create julia map
        isnorm=0;
        dont_patch = 1;
        no_bitmap = 1;
        [julia_map,x,y]=make_julia_map(J.max_pixels,ceil(J.height*J.max_pixels/J.width),...
            J.max_pixels,J.xwidth,J.xc,J.yc,N(n),J.funcstr,...
            J.map_function,J.map_creation_rule,J.C,J.NaN_colour,J.cmap,min_julia,max_julia,isnorm,J.clim,dont_patch,no_bitmap);
        
        %Create 3D surface
        fig = figure('color',[1 1 1],'name',['julia map 3D&2D iter=',leadingzero(N(n),3)],'renderer','opengl');
        figcentre(fig);
        ax = axes;
        subplot(1,2,1);
        surf(x,y,julia_map);
        colormap(J.cmap),
        interp_colormap(J.max_pixels);
        zlabel(J.funcstr)
        caxis(J.clim)
        shading interp;
        axis vis3d
        axis equal
        view([-35,64]);
        alpha(0.9);
        lighting phong
        camlight
        zoom(1.2);
          
        %Create 2D view
        subplot(1,2,2);
        pcolor(x,y,julia_map);
        caxis(J.clim)
        
        %Annotate figure, set colouring, lighting etc
        xlabel('real numbers x')
        ylabel('complex numbers i*y')
        zlabel(J.funcstr);
        caxis(J.clim)
        title( ['julia map: ',J.funcstr,' iteration = ',leadingzero(N(n),3)] );
        shading interp;
        axis equal
        axis tight
        box on
        view(2);
       
        %Write bitmap image
        print(fig,['julia map 2D&3D iter=',leadingzero(N(n),3)],...
            ['-d',J.output_image_type], ['-r',num2str(J.output_bitmap_DPI)])
        croppic(['julia map 2D&3D iter=',leadingzero(N(n),3),'.png'],[255,255,255]);
        close(gcf);   
    end
end

%Change Update button to green
set(handles.PUSHupdate,'backgroundcolor',[0,1,0.251]);

%Update handles structure
guidata(hObject, handles);

%%

%z convergence radius
function EDITc_Callback(hObject, eventdata, handles)

%Get GUI data
handles = guidata(hObject);

%Change Update button to amber
set(handles.PUSHupdate,'backgroundcolor',[1,0.502,0]);

%Update handles structure
guidata(hObject, handles);

%%

%Bitmap width in pixels
function EDITwidth_Callback(hObject, eventdata, handles)

%Get GUI data
handles = guidata(hObject);

%Change Update button to amber
set(handles.PUSHupdate,'backgroundcolor',[1,0.502,0]);

%Update handles structure
guidata(hObject, handles);

%%

%Bitmap height in pixels
function EDITheight_Callback(hObject, eventdata, handles)

%Get GUI data
handles = guidata(hObject);

%Change Update button to amber
set(handles.PUSHupdate,'backgroundcolor',[1,0.502,0]);

%Update handles structure
guidata(hObject, handles);

%%

%Argand diagram x width
function EDITxwidth_Callback(hObject, eventdata, handles)

%Get GUI data
handles = guidata(hObject);

%Change Update button to amber
set(handles.PUSHupdate,'backgroundcolor',[1,0.502,0]);

%Update handles structure
guidata(hObject, handles);

%%

%Argand diagram x centre
function EDITxc_Callback(hObject, eventdata, handles)

%Get GUI data
handles = guidata(hObject);

%Change Update button to amber
set(handles.PUSHupdate,'backgroundcolor',[1,0.502,0]);

%Update handles structure
guidata(hObject, handles);

%%

%Argand diagram y centre
function EDITyc_Callback(hObject, eventdata, handles)

%Get GUI data
handles = guidata(hObject);

%Change Update button to amber
set(handles.PUSHupdate,'backgroundcolor',[1,0.502,0]);

%Update handles structure
guidata(hObject, handles);

%%

%Iterations editbox
function EDITiter_Callback(hObject, eventdata, handles)

%Get GUI data
handles = guidata(hObject);

%Change Update button to amber
set(handles.PUSHupdate,'backgroundcolor',[1,0.502,0]);

%Update handles structure
guidata(hObject, handles);

%%

%Julia function editbox
function EDITjuliafunction_Callback(hObject, eventdata, handles)

%Get GUI data
handles = guidata(hObject);

%Change Update button to amber
set(handles.PUSHupdate,'backgroundcolor',[1,0.502,0]);

%Update handles structure
guidata(hObject, handles);

%%

%NaN colour editbox
function EDITnancolor_Callback(hObject, eventdata, handles)

%Get GUI data
handles = guidata(hObject);

%Change Update button to amber
set(handles.PUSHupdate,'backgroundcolor',[1,0.502,0]);

%Update handles structure
guidata(hObject, handles);

%%

%Julia map creation rule popupmenu
function POPUPMENUmapcreationrule_Callback(hObject, eventdata, handles)

%Get GUI data
handles = guidata(hObject);

%Change Update button to amber
set(handles.PUSHupdate,'backgroundcolor',[1,0.502,0]);

%If map creation rule is 'plot z' then de-activate convergence radius edit
%box. (i.e. grey it out so it can't be used and confuse the user!)
map_creation_rules = get(handles.POPUPMENUmapcreationrule, 'string');
map_creation_rule = map_creation_rules{ get(handles.POPUPMENUmapcreationrule, 'value') };
if strcmp( map_creation_rule, 'plot z' )
    set( handles.EDITc,'Enable','off' );
else
    set( handles.EDITc,'Enable','on' );
end

%Update handles structure
guidata(hObject, handles);

%%

%Julia map function popupmenu
function POPUPMENUmapfunction_Callback(hObject, eventdata, handles)

%Get GUI data
handles = guidata(hObject);

%Change Update button to amber
set(handles.PUSHupdate,'backgroundcolor',[1,0.502,0]);

%Update handles structure
guidata(hObject, handles);

%%

%Set colormap popupmenu
function POPUPMENUcolormap_Callback(hObject, eventdata, handles)

%Get GUI data
handles = guidata(hObject);

%Change Update button to amber
set(handles.PUSHupdate,'backgroundcolor',[1,0.502,0]);

%Update handles structure
guidata(hObject, handles);

%%

%Set colormap range mapping to normalized julia map
function EDITclim_Callback(hObject, eventdata, handles)

%Get GUI data
handles = guidata(hObject);

%Change Update button to amber
set(handles.PUSHupdate,'backgroundcolor',[1,0.502,0]);

%Update handles structure
guidata(hObject, handles);

%%

%Max pixels in julia map edit box
function EDITmaxpixels_Callback(hObject, eventdata, handles)

%Get GUI data
handles = guidata(hObject);

%Change Update button to amber
set(handles.PUSHupdate,'backgroundcolor',[1,0.502,0]);

%Update handles structure
guidata(hObject, handles);

%DPI of output bitmap for 2D and 3D images
function EDITDPI_Callback(hObject, eventdata, handles)

%Get GUI data
handles = guidata(hObject);

%Change Update button to amber
set(handles.PUSHupdate,'backgroundcolor',[1,0.502,0]);

%Update handles structure
guidata(hObject, handles);

%%

%Stop button
function PUSHstop_Callback(hObject, eventdata, handles)

%Get GUI data
handles = guidata(hObject);

%Change Update button to amber and return control to the GUI, terminating
%the current run. This is done from make_julia_map.m, which has a check of
%the status of handles.STOP
set(handles.PUSHupdate,'backgroundcolor',[1,0.502,0]);
handles.J.STOP=1;

%Update handles structure
guidata(hObject, handles);

%%

%Set delete composites checkbox
function CHECKBOXdeletecomposites_Callback(hObject, eventdata, handles)

%Get GUI data
handles = guidata(hObject);

%Change Update button to amber
set(handles.PUSHupdate,'backgroundcolor',[1,0.502,0]);

%Update handles structure
guidata(hObject, handles);

%%

%Set plot all iterations checkbox
function CHECKplotalliter_Callback(hObject, eventdata, handles)

%Get GUI data
handles = guidata(hObject);

%Change Update button to amber
set(handles.PUSHupdate,'backgroundcolor',[1,0.502,0]);

%Update handles structure
guidata(hObject, handles);

%%

%Set plot 3D (as well as 2D) checkbox
function CHECK3dsurf_Callback(hObject, eventdata, handles)

%Get GUI data
handles = guidata(hObject);

%Change Update button to amber
set(handles.PUSHupdate,'backgroundcolor',[1,0.502,0]);

%Update handles structure
guidata(hObject, handles);

%%

%Set plot combined 3D and 2D checkbox (as well as 2D)
function CHECK3and2dsurf_Callback(hObject, eventdata, handles)

%Get GUI data
handles = guidata(hObject);

%Change Update button to amber
set(handles.PUSHupdate,'backgroundcolor',[1,0.502,0]);

%Update handles structure
guidata(hObject, handles);

%%

%Load GUI settings pushbutton
function PUSHloadsettings_Callback(hObject, eventdata, handles)

%Get GUI data
handles = guidata(hObject);

%Change Update button to amber
set(handles.PUSHupdate,'backgroundcolor',[1,0.502,0]);

%Return path to valid .mat file obtained via Explorer navigation by user
%which contains julia setup information.
[filename, pathname, filterindex] = uigetfile('*.mat', 'Choose a .mat file containing julia GUI settings');

%Load .mat file and update handles array of GUI with information contained
%within the .mat file. If this doesn't work flag up an error dialogue box
try
    handles.J = load( [pathname, '\', filename] );
    handles = set_gui_data(handles);
catch
    errordlg('.mat file selected does not contain the correct inputs for julia.m','julia error!')
    return
end

%Update handles structure
guidata(hObject, handles);

%%

%Save GUI settings pushbutton
function PUSHsavesettings_Callback(hObject, eventdata, handles)

%Get GUI data
handles = guidata(hObject);
handles = get_gui_data(handles);

%Return path to valid .mat file obtained via Explorer navigation by user
%which will contains julia setup information.
[filename, pathname, filterindex] = uiputfile('*.mat', 'Save julia GUI settings in a .mat file');

%Save .mat file containing GUI settings
J = handles.J;
if ~isequal(filename,0) && ~isequal(pathname,0)
    save( [pathname,'\',filename], '-struct','J' );
end

%Update handles structure
guidata(hObject, handles);

%%

%figcenter
% Centres a figure with handle fig in the current display.
%
% LAST UPDATED by Andy French Feb 2015

function figcentre(fig)

%Get screen dimensions in pixels.
set(0,'units','pixels') ;
screen_size=get(0,'Screensize');
display_size=get(0,'MonitorPositions');

%Get original figure units
orig_units = get(fig,'units');

%Get position vector for figure in pixels
set(fig,'units','pixels');
fig_position=get(gcf,'position') ;

%Define coordinates or figure bottom hand corner from screen bottom hand corner
%that will place the figure window in the center of the screen.
centre_x = 0.5*( screen_size(3)-fig_position(3) ) - screen_size(1);
centre_y = 0.5*( screen_size(4)-fig_position(4) ) - screen_size(2);

%Modify figure position to centre
set(fig,'position',[centre_x,centre_y,fig_position(3),fig_position(4)] );

%Reset figure units
set(fig,'units',orig_units);

%End of code.


