%mtrans3d
% 3D geometric transformations using 3x3 matrix multiplication.
% Dr Andrew French. January 2025.


%%%% GUI INITIALIZATION %%%%

%Top level GUI initialization
function varargout = mtrans3d(varargin)

% Generic MATLAB GUI initialization. Edit at your peril!
gui_Singleton = 1;
gui_State = struct('gui_Name',       mfilename, ...
    'gui_Singleton',  gui_Singleton, ...
    'gui_OpeningFcn', @mtrans3d_OpeningFcn, ...
    'gui_OutputFcn',  @mtrans3d_OutputFcn, ...
    'gui_LayoutFcn',  [] , ...
    'gui_Callback',   []);
if nargin && ischar(varargin{1})
    gui_State.gui_Callback = str2func(varargin{1});
end

if nargout
    [varargout{1:nargout}] = gui_mainfcn(gui_State, varargin{:});
else
    gui_mainfcn(gui_State, varargin{:});
end

%%

% Outputs from this function are returned to the command line.
function varargout = mtrans3d_OutputFcn(hObject, eventdata, handles)
varargout{1} = handles.output;

%%

% Code which executes just before mtrans3d is made visible.
function mtrans3d_OpeningFcn(hObject, eventdata, handles, varargin)

% Choose default command line output for mtrans3d
handles.output = hObject;
guidata(hObject, handles);

%Store GUI object handles data in a global array d which can be seen (and
%modified) by all functions. This will be the main GUI data array which
%enables all objects to communicate with one another.
global d
d.h = handles; clear handles

%Initialize 3x3 transformation matrix
d.A = [ 1,0,0 ; 0,1,0; 0,0,1 ];

%Initialize x,y,z shifts
set( d.h.EDITxshift,'string','0');
set( d.h.EDITyshift,'string','0');
set( d.h.EDITzshift,'string','0');

%Initialize rotation angle
d.theta = 0;
set( d.h.EDITangle,'string',num2str( 180*d.theta/pi ) );

%Point to plot axes
axes( d.h.AXES ); hold on; 

%Generate coloured surface
[x,y,z] = gen3dsurface; r = z.*sqrt( x.^2 + y.^2 + z.^2 ); colormap(jet); interp_colormap(500);
d.S = surf(x,y,z,r); shading interp; axis vis3d; view([120,36]); zoom(0.7); caxis([-1,1]);
d.Scopy = []; %Initialize copy of surface S.
d.C = get(d.S,'CData'); %Store colour data matrix
set( d.S,'Cdatamode','manual' ); %Fix surface colour
set( d.S, 'xdata', get(d.S,'xdata') + 2 ); %Shift position

%Set axes bounds and labels
xlim([-3,4]); ylim([-3,3]); zlim([-3,3]);
xlabel('x'); ylabel('y'); zlabel('z'); box on; grid on; axis manual;

%Add x,y,z axis arrows
head_frac = 0.91; arrow_radius = 0.02; head_radius = 0.05;
arrow3d([0,4],[0,0],[0,0],head_frac,arrow_radius,head_radius,[1,0,0] );
arrow3d([0,0],[0,3],[0,0],head_frac,arrow_radius,head_radius,[0,1,0] );
arrow3d([0,0],[0,0],[0,3],head_frac,arrow_radius,head_radius,[0,0,1] );

%Set transform menu
transforms = {'Identity','Reflect in y,z plane','Reflect in x,z plane','Reflect in x,y plane',...
    'Rotate by theta about x','Rotate by theta about y','Rotate by theta about z' };
set(d.h.MENU,'string',transforms);

%Update GUI
update_gui

%%

%Callback function for RESET pushbutton
function PUSHreset_Callback(hObject, eventdata, handles); 
global d; 

%Generate coloured surface x,y,z coordinates
[x,y,z] = gen3dsurface; x = x + 2;

%Update surface
set( d.S,'xdata',x,'ydata',y,'zdata',z,'Cdatamapping','scaled','Cdata',d.C );

%Reset transformation matrix and associated GUI edit boxes
d.A = [ 1,0,0 ; 0,1,0; 0,0,1 ];
set( d.h.MENU,'value', 1 );
set( d.h.EDITxshift,'string','0');
set( d.h.EDITyshift,'string','0');
set( d.h.EDITzshift,'string','0');

%Delete any copies following transforms
for n=1:length(d.Scopy)
    delete(d.Scopy(n));
end
d.Scopy = [];

%Update GUI
update_gui

%%

%Callback function for EARTH pushbutton
function PUSHearth_Callback(hObject, eventdata, handles); 
global d; 

%Create unit sphere with centre [2,0,0] x,y,z arrays of the same dimensions as Earth bitmap
long = linspace(-pi,pi,1000); lat = linspace(-pi/2,pi/2,500);
[long,lat] = meshgrid(long,lat);
x = cos(lat).*cos(long) + 2; y = cos(lat).*sin(long); z = sin(lat);

%Load Earth image
[I,cmap] = imread('earth.jpg');
    
%Convert image to double precision on [0,1]
I = double(I)/255 ; I = flipdim(I,1);

%Update surface
set( d.S,'xdata',x,'ydata',y,'zdata',z, 'CDatamapping', 'Direct','CData', I );

%Reset transformation matrix and associated GUI edit boxes
d.A = [ 1,0,0 ; 0,1,0; 0,0,1 ];
set( d.h.MENU,'value', 1 );
set( d.h.EDITxshift,'string','0');
set( d.h.EDITyshift,'string','0');
set( d.h.EDITzshift,'string','0');

%Delete any copies following transforms
for n=1:length(d.Scopy)
    delete(d.Scopy(n));
end
d.Scopy = [];

%Update GUI
update_gui

%%

%Callback function for TRANSFORM! pushbutton
function PUSHtransform_Callback(hObject, eventdata, handles)
global d; 
d.A(1,1) = str2num( get(d.h.A11,'string') ); 
d.A(1,2) = str2num( get(d.h.A12,'string') );
d.A(1,3) = str2num( get(d.h.A13,'string') ); 
d.A(2,1) = str2num( get(d.h.A21,'string') ); 
d.A(2,2) = str2num( get(d.h.A22,'string') ); 
d.A(2,3) = str2num( get(d.h.A23,'string') );
d.A(3,1) = str2num( get(d.h.A31,'string') ); 
d.A(3,2) = str2num( get(d.h.A32,'string') ); 
d.A(3,3) = str2num( get(d.h.A33,'string') ); 
update_gui

%%

%Callback function for x,y,z,shift
function PUSHshift_Callback(hObject, eventdata, handles)
global d;

%Get shifts from GUI
dx = str2num( get(d.h.EDITxshift,'string') );
dy = str2num( get(d.h.EDITyshift,'string') );
dz = str2num( get(d.h.EDITzshift,'string') );

%Shift surface
set( d.S,'xdata',get(d.S,'xdata') + dx );
set( d.S,'ydata',get(d.S,'ydata') + dy );
set( d.S,'zdata',get(d.S,'zdata') + dz );

%Reset edit boxes of shift
set( d.h.EDITxshift,'string','0');
set( d.h.EDITyshift,'string','0');
set( d.h.EDITzshift,'string','0');

%%

%%Calback function for transforms menu
function MENU_Callback(hObject, eventdata, handles)
global d;
if get(d.h.MENU,'value')==1
    %Identity
    d.A = [1,0,0 ; 0,1,0 ; 0,0,1 ];
elseif get(d.h.MENU,'value')==2
    %Reflect in y,z plane
    d.A = [-1,0,0 ; 0,1,0 ; 0,0,1 ];
elseif get(d.h.MENU,'value')==3
    %Reflect in x,z plane
    d.A = [1,0,0 ; 0,-1,0 ; 0,0,1 ];
elseif get(d.h.MENU,'value')==4
    %Reflect in x,y plane
    d.A = [1,0,0 ; 0,1,0 ; 0,0,-1 ];
elseif get(d.h.MENU,'value')==5
    %Rotate anticlockwise about x axis by theta
    d.A = [1,0,0 ; 0,cos(d.theta),-sin(d.theta) ; 0,sin(d.theta),cos(d.theta) ];
elseif get(d.h.MENU,'value')==6
    %Rotate anticlockwise about y axis by theta
    d.A = [cos(d.theta),0,sin(d.theta) ; 0,1,0 ; -sin(d.theta),0,cos(d.theta) ];
elseif get(d.h.MENU,'value')==7
    %Rotate anticlockwise about z axis by theta
    d.A = [cos(d.theta),-sin(d.theta),0 ; sin(d.theta),cos(d.theta),0 ; 0,0,1 ];
end
update_gui

%%

%Callback function for rotation angle edit box
function EDITangle_Callback(hObject, eventdata, handles);
global d; d.theta = (pi/180)*str2num( get(d.h.EDITangle,'string') );
if get(d.h.MENU,'value')==5
    %Rotate anticlockwise about x axis by theta
    d.A = [1,0,0 ; 0,cos(d.theta),-sin(d.theta) ; 0,sin(d.theta),cos(d.theta) ];
elseif get(d.h.MENU,'value')==6
    %Rotate anticlockwise about y axis by theta
    d.A = [cos(d.theta),0,sin(d.theta) ; 0,1,0 ; -sin(d.theta),0,cos(d.theta) ];
elseif get(d.h.MENU,'value')==7
    %Rotate anticlockwise about z axis by theta
    d.A = [cos(d.theta),-sin(d.theta),0 ; sin(d.theta),cos(d.theta),0 ; 0,0,1 ];
end
update_gui

%%

%Update GUI
function update_gui
global d

%Update GUI elements
set(d.h.A11,'string',num2str( d.A(1,1),3 ) );
set(d.h.A12,'string',num2str( d.A(1,2),3 ) );
set(d.h.A13,'string',num2str( d.A(1,3),3 ) );
set(d.h.A21,'string',num2str( d.A(2,1),3 ) );
set(d.h.A22,'string',num2str( d.A(2,2),3 ) );
set(d.h.A23,'string',num2str( d.A(2,3),3 ) );
set(d.h.A31,'string',num2str( d.A(3,1),3 ) );
set(d.h.A32,'string',num2str( d.A(3,2),3 ) );
set(d.h.A33,'string',num2str( d.A(3,3),3 ) );
set(d.h.EDITdetA,'string',num2str(det(d.A)));

%Get x,y,z data from AXES
x = get(d.S,'xdata'); y = get(d.S,'ydata'); z = get(d.S,'zdata');

%Transform x,y,z data by matrix multipliation
for n=1:numel(x)
    xyz = d.A * [ x(n) ; y(n) ; z(n) ];
    x(n) = xyz(1); y(n) = xyz(2); z(n) = xyz(3);
end

%Update plot
set(d.S,'xdata',x); set(d.S,'ydata',y); set(d.S,'zdata',z); %Update plot x,y,z coordinates
n = length(d.Scopy);
if get(d.h.CHECKcopy,'value')==1
    d.Scopy(n+1) = copyobj(d.S,d.h.AXES);
end

%%

%3D surface generator - based on Spherium
function [x,y,z] = gen3dsurface

%Generate azimuth and elevation angles /rad
azi = linspace(0,2*pi,500); elev = linspace(-pi/2,pi/2,500);
[azi,elev] = meshgrid(azi,elev);

%Create range array, and hence x,y,z
r = sin(azi.*elev*3);
x = r.*cos(azi).*cos(elev) + 0;
y = r.*sin(azi).*cos(elev);
z = r.*sin(elev);

%%

function interp_colormap(N)

%Get current colourmap
map = colormap;

%Initialise new colormap
new_map = ones(N,3);

%Get size of current colormap and initalise red,green,blue vectors
dim = size(map);
R = ones(1,dim(1));
G = ones(1,dim(1));
B = ones(1,dim(1));
RR = ones(1,N);
GG = ones(1,N);
BB = ones(1,N);

%Populate these with current colormap
R(:) = map(:,1);
G(:) = map(:,2);
B(:) = map(:,3);

%Interpolate to yield new colour map
x = linspace( 1, dim(1), N );
RR = interp1( 1:dim(1), R, x );
GG = interp1( 1:dim(1), G, x );
BB = interp1( 1:dim(1), B, x );
new_map(:,1) = RR(:);
new_map(:,2) = GG(:);
new_map(:,3) = BB(:);

%Set colormap to be new map
colormap( new_map );

%%

%3D arrow plot function
function [h]=arrow3d(x,y,z,head_frac,radii,radii2,colr)
%
% The inputs are:
%       x,y,z =  vectors of the starting point and the ending point of the
%           arrow, e.g.:  x=[x_start, x_end]; y=[y_start, y_end];z=[z_start,z_end];
%       head_frac = fraction of the arrow length where the head should  start
%       radii = radius of the arrow
%       radii2 = radius of the arrow head (defult = radii*2)
%       colr =   color of the arrow, can be string of the color name, or RGB vector  (default='blue')
%
% The output is the handle of the surfaceplot graphics object.
% The settings of the plot can changed using: set(h, 'PropertyName', PropertyValue)
%
% example #1:
%        arrow3d([0 0],[0 0],[0 6],.5,3,4,[1 0 .5]);
% example #2:
%        arrow3d([2 0],[5 0],[0 -6],.2,3,5,'r');
% example #3:
%        h = arrow3d([1 0],[0 1],[-2 3],.8,3);
%        set(h,'facecolor',[1 0 0])
% 
% Written by Moshe Lindner , Bar-Ilan University, Israel. July 2010 (C)
if nargin==5
    radii2=radii*2;
    colr='blue';
elseif nargin==6
    colr='blue';
end
if size(x,1)==2
    x=x';
    y=y';
    z=z';
end
x(3)=x(2);
x(2)=x(1)+head_frac*(x(3)-x(1));
y(3)=y(2);
y(2)=y(1)+head_frac*(y(3)-y(1));
z(3)=z(2);
z(2)=z(1)+head_frac*(z(3)-z(1));
r=[x(1:2)',y(1:2)',z(1:2)'];
N=50;
dr=diff(r);
dr(end+1,:)=dr(end,:);
origin_shift=(ones(size(r))*(1+max(abs(r(:))))+[dr(:,1) 2*dr(:,2) -dr(:,3)]);
r=r+origin_shift;
normdr=(sqrt((dr(:,1).^2)+(dr(:,2).^2)+(dr(:,3).^2)));
normdr=[normdr,normdr,normdr];
dr=dr./normdr;
Pc=r;
n1=cross(dr,Pc);
normn1=(sqrt((n1(:,1).^2)+(n1(:,2).^2)+(n1(:,3).^2)));
normn1=[normn1,normn1,normn1];
n1=n1./normn1;
P1=n1+Pc;
X1=[];Y1=[];Z1=[];
j=1;
for theta=([0:N])*2*pi./(N);
    R1=Pc+radii*cos(theta).*(P1-Pc) + radii*sin(theta).*cross(dr,(P1-Pc)) -origin_shift;
    X1(2:3,j)=R1(:,1);
    Y1(2:3,j)=R1(:,2);
    Z1(2:3,j)=R1(:,3);
    j=j+1;
end
r=[x(2:3)',y(2:3)',z(2:3)'];
dr=diff(r);
dr(end+1,:)=dr(end,:);
origin_shift=(ones(size(r))*(1+max(abs(r(:))))+[dr(:,1) 2*dr(:,2) -dr(:,3)]);
r=r+origin_shift;
normdr=(sqrt((dr(:,1).^2)+(dr(:,2).^2)+(dr(:,3).^2)));
normdr=[normdr,normdr,normdr];
dr=dr./normdr;
Pc=r;
n1=cross(dr,Pc);
normn1=(sqrt((n1(:,1).^2)+(n1(:,2).^2)+(n1(:,3).^2)));
normn1=[normn1,normn1,normn1];
n1=n1./normn1;
P1=n1+Pc;
j=1;
for theta=([0:N])*2*pi./(N);
    R1=Pc+radii2*cos(theta).*(P1-Pc) + radii2*sin(theta).*cross(dr,(P1-Pc)) -origin_shift;
    X1(4:5,j)=R1(:,1);
    Y1(4:5,j)=R1(:,2);
    Z1(4:5,j)=R1(:,3);
    j=j+1;
end
X1(1,:)=X1(1,:)*0 + x(1);
Y1(1,:)=Y1(1,:)*0 + y(1);
Z1(1,:)=Z1(1,:)*0 + z(1);
X1(5,:)=X1(5,:)*0 + x(3);
Y1(5,:)=Y1(5,:)*0 + y(3);
Z1(5,:)=Z1(5,:)*0 + z(3);
h=surf(X1,Y1,Z1,'facecolor',colr,'edgecolor','none');

%End of code
