%ORBITS
% Plots the orbits of two planets around a star of mass M solar masses
% from the perspective of a third planet.
%
% LAST UPDATED by Andy French. June 2023.

function varargout = orbits(varargin)

% Begin initialization code - DO NOT EDIT
gui_Singleton = 1;
gui_State = struct('gui_Name',       mfilename, ...
                   'gui_Singleton',  gui_Singleton, ...
                   'gui_OpeningFcn', @orbits_OpeningFcn, ...
                   'gui_OutputFcn',  @orbits_OutputFcn, ...
                   'gui_LayoutFcn',  [] , ...
                   'gui_Callback',   []);
if nargin && ischar(varargin{1})
    gui_State.gui_Callback = str2func(varargin{1});
end

if nargout
    [varargout{1:nargout}] = gui_mainfcn(gui_State, varargin{:});
else
    gui_mainfcn(gui_State, varargin{:});
end
% End initialization code - DO NOT EDIT

%%

% --- Executes just before orbits is made visible.
function orbits_OpeningFcn(hObject, eventdata, handles, varargin)

% Choose default command line output for orbits
handles.output = hObject;

%Set up global variable d which can be accessed by all functions
global d
d.h = handles;

%Set default parameters
orbits_defaults

%%

function varargout = orbits_OutputFcn(hObject, eventdata, handles) 
global d

% Get default command line output from handles structure
varargout{1} = d.h.output;

%%

function SLIDERplanet1_Callback(hObject, eventdata, handles)
global d
v = get(d.h.SLIDERplanet1,'value'); d.r1 = v*d.rmax;
set(d.h.EDITplanet1,'string',num2str( d.r1 ) );
orbits_update

%%

function EDITplanet1_Callback(hObject, eventdata, handles)
global d
d.r1 = str2num( get( d.h.EDITplanet1,'string' ) );
if d.r1 > d.rmax
    d.r1 = d.rmax;
elseif d.r1<0
    d.r1 = 0;
end
set( d.h.SLIDERplanet1,'value',d.r1/d.rmax );
orbits_update

%%

function SLIDERplanet2_Callback(hObject, eventdata, handles)
global d
v = get(d.h.SLIDERplanet2,'value'); d.r2 = v*d.rmax;
set(d.h.EDITplanet2,'string',num2str( d.r2 ) );
orbits_update

%%

function EDITplanet2_Callback(hObject, eventdata, handles)
global d
d.r2 = str2num( get( d.h.EDITplanet2,'string' ) );
if d.r2 > d.rmax
    d.r2 = d.rmax;
elseif d.r2<0
    d.r2 = 0;
end
set( d.h.SLIDERplanet2,'value',d.r2/d.rmax );
orbits_update

%%

function EDIToriginplanet_Callback(hObject, eventdata, handles)
global d
d.r3 = str2num( get( d.h.EDIToriginplanet,'string' ) );
if d.r3 > d.rmax
    d.r3 = d.rmax;
elseif d.r3<0
    d.r3 = 0;
end
set( d.h.SLIDERoriginplanet,'value',d.r3/d.rmax );
orbits_update

%%

function SLIDERoriginplanet_Callback(hObject, eventdata, handles)
global d
v = get(d.h.SLIDERoriginplanet,'value'); d.r3 = v*d.rmax;
set(d.h.EDIToriginplanet,'string',num2str( d.r3 ) );
orbits_update

%%

function EDITM_Callback(hObject, eventdata, handles)
global d
d.M = str2num( get(d.h.EDITM,'string') ); 
orbits_update

%%
function EDITrmax_Callback(hObject, eventdata, handles)
global d
d.rmax = str2num( get( d.h.EDITrmax,'string' ) );
if d.rmax < 2
    d.rmax = 2;
end
set(d.h.AXESplot, 'xlim',[-d.rmax,d.rmax],'ylim',[-d.rmax,d.rmax] );
set( d.h.SLIDERplanet1,'value',d.r1/d.rmax );
set( d.h.SLIDERplanet2,'value',d.r2/d.rmax );
set( d.h.SLIDERoriginplanet,'value',d.r3/d.rmax );

%%

function PUSHpng_Callback(hObject, eventdata, handles)
global d

%Calculate orbits
[x1,y1,x2,y2,x3,y3,t] = orbits_calculator( d.r1, d.r2, d.r3, d.M );

%Make initial plot of planets 1 and 2 (and star) orbits, 
%relative to planet 3
fig = figure('name','solar system','color',[0 0 0],'InvertHardCopy', 'off');
axes('nextplot','add','color', [0 0 0],'xcolor',[1 1 1],'ycolor',[1 1 1],'zcolor',[1 1 1]); 
plot( x1-x3, y1-y3, 'r-' );
plot( x2-x3, y2-y3, 'b-' );
plot( zeros(size(x3)) - x3, zeros(size(x3)) - y3, 'y-' );
axis equal; grid on; set(gca, 'xlim',[-d.rmax,d.rmax],'ylim',[-d.rmax,d.rmax] );
xlabel('x (AU)'); ylabel('y (AU)'); box on;
title(['M=',num2str(d.M),', r_1=',num2str(d.r1),...
    ', r_2=',num2str(d.r2),', r_3=',num2str(d.r3)],'color',[1 1 1]);
legend({'Planet 1','Planet 2','Star'},'textcolor',[1 1 1],'edgecolor',[1 1 1] )
print( gcf,'orbits.png','-dpng','-r300');
close(fig);
axes(d.h.AXESplot);

%%

%GUI defaults
function orbits_defaults
global d
d.M = 1; %Star mass in solar masses
d.rmax = 30; %Max scale of plot area in AU, and indeed sliders
d.r1 = 5;  %Planet 1 orbital radius in AU
d.r2 = 15;  %Planet 1 orbital radius in AU
d.r3 = 7;  %Coordinate origin planet orbital radius in AU

%Set GUI with defaults
set( d.h.EDITrmax,'string',num2str(d.rmax) );
set( d.h.EDITM,'string',num2str(d.M) );
set( d.h.EDITplanet1,'string',num2str(d.r1) );
set( d.h.EDITplanet2,'string',num2str(d.r2) );
set( d.h.EDIToriginplanet,'string',num2str(d.r3) );
set( d.h.SLIDERplanet1,'value',d.r1/d.rmax );
set( d.h.SLIDERplanet2,'value',d.r2/d.rmax );
set( d.h.SLIDERoriginplanet,'value',d.r3/d.rmax );

%Calculate orbits
[x1,y1,x2,y2,x3,y3,t] = orbits_calculator( d.r1, d.r2, d.r3, d.M );

%Make initial plot of planets 1 and 2 (and star) orbits, 
%relative to planet 3
axes(d.h.AXESplot); set(gca,'nextplot','add','color',[0 0 0],...
    'xcolor',[1 1 1],'ycolor',[1 1 1],'zcolor',[1 1 1]); 
d.p1 = plot( x1-x3, y1-y3, 'r-' );
d.p2 = plot( x2-x3, y2-y3, 'b-' );
d.star = plot( zeros(size(x3)) - x3, zeros(size(x3)) - y3, 'y-' );
axis equal; grid on; set(gca, 'xlim',[-d.rmax,d.rmax],'ylim',[-d.rmax,d.rmax] );

%%

%GUI update
function orbits_update
global d

%Calculate orbits
[x1,y1,x2,y2,x3,y3,t] = orbits_calculator( d.r1, d.r2, d.r3, d.M );
set( d.p1,'xdata',x1-x3, 'ydata', y1-y3 );
set( d.p2,'xdata',x2-x3, 'ydata', y2-y3 );
set( d.star,'xdata',zeros(size(x3)) - x3, 'ydata', zeros(size(x3)) - y3 );

%%

%Orbit calculator
function [x1,y1,x2,y2,x3,y3,t] = orbits_calculator( r1, r2, r3, M )

%Calculate orbital periods /years based on Kepler's Third Law. r1,r2,r3 are
%(circular) orbit radii in AU, M is the star mass in solar masses.
T1 = (r1^(3/2))/sqrt(M); T2 = (r2^(3/2))/sqrt(M); T3 = (r3^(3/2))/sqrt(M);

%Define time array in years
tmax = 20*max( [T1,T2,T3] ); t = linspace(0,tmax,5000);

%Compute circular orbits. Star is at the origin.
x1 = r1*cos( 2*pi*t/T1 ); y1 = r1*sin( 2*pi*t/T1 );
x2 = r2*cos( 2*pi*t/T2 ); y2 = r2*sin( 2*pi*t/T2 );
x3 = r3*cos( 2*pi*t/T3 ); y3 = r3*sin( 2*pi*t/T3 );

%%

%Solar system planet
% Outputs a string if planet has the approximate orbital radius r of a solar system
% planet.
function planet = solar_system_planet(r)
r = '';

%%

%figcenter
% Centres a figure with handle fig in the current display.
function figcentre(fig)

%Get screen dimensions in pixels.
set(0,'units','pixels') ;
screen_size=get(0,'Screensize');

%Get original figure units
orig_units = get(fig,'units');

%Get position vector for figure in pixels
set(fig,'units','pixels');
fig_position=get(gcf,'position') ;

%Define coordinates or figure bottom hand corner from screen bottom hand corner
%that will place the figure window in the center of the screen.
centre_x = 0.5*screen_size(3) - 0.5*fig_position(3);
centre_y = 0.5*screen_size(4) - 0.5*fig_position(4);

%Modify figure position to centre
set(fig,'position',[centre_x,centre_y,fig_position(3),fig_position(4)] );

%Reset figure units
set(fig,'units',orig_units);

%End of code
