%quadratica
% Annotates a quadratic with roots, vertex and y intercept. Use this
% function to generate examples.
%
% Inputs can be:
% (i)   A root, root separtion, x^2 coefficient
% (ii)  x and y vertex coordinate, x^2 coefficient
% (iii) All three quadratic coefficients a,b,c
%
% Note vertex, roots and y intercept will all be
% integer if the following recipe is followed:
%
% The coefficient of x^2 , a is integer
% The first root is integer
% The root spacing is an even integer
%
% LAST UPDATED by Andy French Jan 2011

function quadratica

%% TOLERANCE FOR IMAGINARY COMPONENT OF ROOT %%

%If abs(imag( root )) is less than tol we shall assume it to be zero!
tol = 1e-10;

%

%Ask user for coefficient of x^2, and another two inputs
disp(' ');
disp('Welcome to QUADRATICA by A. French')
disp(' ')
disp(' Choose input options ....')
disp(' ')
disp(' ... Type 1 to input a root, root separtion and x^2 coefficient')
disp(' ... Type 2 to input x and y vertex coordinates and x^2 coefficient')
disp(' ... Any other key will ask for all three quadratic coefficients a,b,c')
disp(' ')
option = input('');
if option==1
    %Coefficient of x^2, first root and root spacing
    a = str2num( input( ' ... Input coefficient of x^2 (a) \n','s') );
    first_root = str2num( input(' ... Input first root i.e. x when y=0 \n','s') );
    root_spacing = str2num( input(' ... Input root spacing \n','s' ));
    
    %Derive other quadratic parameters
    [second_root,vertex_x,vertex_y,b,c] = find_quad_points( a, first_root, root_spacing );
elseif option==2
    %Coefficient of x^2 and vertex coordinates
    a = str2num( input( ' ... Input coefficient of x^2 (a) \n','s') );
    vertex_x = str2num( input(' ... Input vertex x coordinate \n','s') );
    vertex_y = str2num( input(' ... Input vertex y coordinate \n','s' ));
    
    %Derive other quadratic parameters
    [first_root,second_root,b,c] = find_quad_points2( a, vertex_x, vertex_y );
else
    a = str2num( input( ' ... Input coefficient of x^2 (a) \n','s') );
    b = str2num( input(' ... Input coefficient of x^1 (b) \n','s') );
    c = str2num( input(' ... Input coefficient of x^0 (c) \n','s' ));
    
    %Derive other quadratic parameters
    [first_root,second_root,vertex_x,vertex_y] = find_quad_points3( a, b, c );
end

%Construct vectors of x,y points which define the quadratic
N=100;
x0 = min( real(first_root), real(second_root) );
x1 = max( real(first_root), real(second_root) );
d = abs(x1-x0);
xmin = min([0,x0])-d;
xmax = 2*vertex_x - xmin;
x = linspace( xmin,xmax, N );
y = a*x.^2 + b*x + c;

%Plot and annotate quadratic
figure('color',[1 1 1],'name','quadratica')
plot(x,y,'b');
xlabel('x')
ylabel('y')
quadratic_str = ['Quadratic y = ',num2str(a),...
    'x^2 + ',num2str(b),'x + ',num2str(c) ];
title(quadratic_str);
hold on
if abs(imag(first_root))<tol
    plot( x0,0,'r*'); annotate_coordinate(x0,0)
    plot( x1,0,'m*'); annotate_coordinate(x1,0)
else
    plot( 0,c,'r*')
    plot( 0,c,'m*')
end
plot( 0,c,'g*'); annotate_coordinate(0,c)
plot( vertex_x, vertex_y, 'k*' ); annotate_coordinate(vertex_x,vertex_y)
ylimit = get(gca,'ylim');
plot([xmin,xmax],[0,0],'r');
plot([0,0],ylimit,'r');
axis tight
grid on
box on
legend('Quadratic','Root 1','Root 2', 'y intercept','Vertex',...
    'location','eastoutside')

%Save a PNG file of the quadratic
saveas( gcf, quadratic_str, 'png' );

%%

%Compute parameters of the quadratic, root inputs
function [second_root,vertex_x,vertex_y,b,c] = find_quad_points( a, first_root, root_spacing )
second_root = first_root + root_spacing ;
vertex_x = first_root + root_spacing/2;
vertex_y =  - (a/4)*root_spacing^2 ;
c = first_root*a*(second_root);
b = -a*(first_root + second_root);

%%

%Compute other parameters of the quadratic, vertex inputs
function [first_root,second_root,b,c] = find_quad_points2( a, vertex_x, vertex_y )
b = -vertex_x*2*a;
c = vertex_y + (1/(4*a))*b^2;
first_root = (-b - sqrt( b^2 - 4*a*c))/(2*a);
second_root = (-b + sqrt( b^2 - 4*a*c))/(2*a);

%%

%Compute other parameters of the quadratic, a,b,c inputs
function [first_root,second_root,vertex_x,vertex_y] = find_quad_points3( a, b, c )
first_root = (-b - sqrt( b^2 - 4*a*c))/(2*a);
second_root = (-b + sqrt( b^2 - 4*a*c))/(2*a);
vertex_x = -b/(2*a);
vertex_y = c - (1/(4*a))*b^2;

%%

%Annotate coordinates with (x,y) text
function annotate_coordinate(x,y)
axes(gca)
hold on
d = 0;
text(x+d,y,['(',num2str(x),',',num2str(y),')' ]);

%End of code