%quadratic_gui
% GUI which displays all the parameters of a quadratic equation.
%
% LAST UPDATED by Andy French March 2013

function varargout = quadratica_gui(varargin)

%GUI initialization code
gui_Singleton = 1;
gui_State = struct('gui_Name',       mfilename, ...
    'gui_Singleton',  gui_Singleton, ...
    'gui_OpeningFcn', @quadratica_gui_OpeningFcn, ...
    'gui_OutputFcn',  @quadratica_gui_OutputFcn, ...
    'gui_LayoutFcn',  [] , ...
    'gui_Callback',   []);
if nargin && ischar(varargin{1})
    gui_State.gui_Callback = str2func(varargin{1});
end

if nargout
    [varargout{1:nargout}] = gui_mainfcn(gui_State, varargin{:});
else
    gui_mainfcn(gui_State, varargin{:});
end

%%

% GUI opening function
function quadratica_gui_OpeningFcn(hObject, eventdata, handles, varargin)

%%%%%%%%% SET MAXIMUM SIZE OF RANDOMLY GENERATED QUADRATIC FACTORIZED FORM INTEGER PARAMETERS %%%%%%%%%%%%%
q.N = 5;

%Randomly generate a quadratic
[q.A,q.B,q.C,q.D,q.a,q.b,q.c,expanded_quad_str,fact_quad_str] = gen_quad(q.N);
q.alpha = q.a;
q.beta = q.b/(2*q.a);
q.gamma = q.c - (q.b^2)/(4*q.a);

%Compute vertex coordinates
q.vertex_x = -q.b/(2*q.a);
q.vertex_y = q.c - (q.b^2)/(4*q.a);

%Compute roots
q.xroot1 = (-q.b-sqrt( q.b^2 -4*q.a*q.c ))/(2*q.a);
q.xroot2 = (-q.b+sqrt( q.b^2 -4*q.a*q.c ))/(2*q.a);

%Initially allow axis not to be fixed, i.e. automatically resized to
%include all the interesting features
q.fix_axis = 0;

%Show title in plot
q.show_title = 1;

%Update GUI
update_gui_from_quad(q);

%%

% GUI output function
function varargout = quadratica_gui_OutputFcn(hObject, eventdata, handles)

%%

%Coefficient of x^2 in expanded form
function EDITa_Callback(hObject, eventdata, handles)
q = update_q_from_gui('expanded');
update_gui_from_quad(q);

%%

%Coefficient of x in expanded form
function EDITb_Callback(hObject, eventdata, handles)
q = update_q_from_gui('expanded');
update_gui_from_quad(q);

%%

%Constant in expanded form
function EDITc_Callback(hObject, eventdata, handles)
q = update_q_from_gui('expanded');
update_gui_from_quad(q);

%%

%Square multiplier in completed square form
function EDITaa_Callback(hObject, eventdata, handles)
q = update_q_from_gui('completed square');
update_gui_from_quad(q);

%%

%Minus x vertex in completed square form
function EDITbb_Callback(hObject, eventdata, handles)
q = update_q_from_gui('completed square');
update_gui_from_quad(q);

%%

%y vertex coordinate in completed square form
function EDITcc_Callback(hObject, eventdata, handles)
q = update_q_from_gui('completed square');
update_gui_from_quad(q);

%%

%A parameter in (Ax+B)(Bx+C) factorized form
function EDITU_Callback(hObject, eventdata, handles)
q = update_q_from_gui('factorized');
update_gui_from_quad(q);

%%

%A parameter in (Ax+B)(Bx+C) factorized form
function EDITV_Callback(hObject, eventdata, handles)
q = update_q_from_gui('factorized');
update_gui_from_quad(q);

%%

%A parameter in (Ax+B)(Bx+C) factorized form
function EDITQ_Callback(hObject, eventdata, handles)
q = update_q_from_gui('factorized');
update_gui_from_quad(q);

%%

%A parameter in (Ax+B)(Bx+C) factorized form
function EDITW_Callback(hObject, eventdata, handles)
q = update_q_from_gui('factorized');
update_gui_from_quad(q);

%%

%Save graph as a .png image push-button
function PUSHPNG_Callback(hObject, eventdata, handles)
q = update_q_from_gui('factorized');
update_gui_from_quad(q);

%Create new plot
fig = figure('color',[1 1 1]);
ax = axes;
q = plot_quad(ax,q);

%Save as a .PNG file to the current directory
print( fig, '-dpng','-r300',['y = ',num2str(q.a),'x^2 + ',num2str(q.b),'x + ',num2str(q.c),...
    '. ',strrep(datestr(now),':','-'),'.png'])

%Close figure
close(fig);

%%

%Randomize inputs (always integer factorized form)
function PUSHrandomquad_Callback(hObject, eventdata, handles)

%Get GUI data to find max value of factorized form integer
h = guidata(gcf);
q.N = h.q.N;
q.fix_axis = h.q.fix_axis;
q.show_title = h.q.show_title;
q.xmin = h.q.xmin;
q.xmax = h.q.xmax;
q.ymin = h.q.ymin;
q.ymax = h.q.ymax;

%Randomly generate a quadratic
[q.A,q.B,q.C,q.D,q.a,q.b,q.c,expanded_quad_str,fact_quad_str] = gen_quad(q.N);
q.alpha = q.a;
q.beta = q.b/(2*q.a);
q.gamma = q.c - (q.b^2)/(4*q.a);

%Compute vertex coordinates
q.vertex_x = -q.b/(2*q.a);
q.vertex_y = q.c - (q.b^2)/(4*q.a);

%Compute roots
q.xroot1 = (-q.b-sqrt( q.b^2 -4*q.a*q.c ))/(2*q.a);
q.xroot2 = (-q.b+sqrt( q.b^2 -4*q.a*q.c ))/(2*q.a);

%Update GUI
update_gui_from_quad(q);

%%

%Unhide or hide Expanded quadratic checkbox
function CHECKexpanded_Callback(hObject, eventdata, handles)
h = guidata(gcf);
if get(h.CHECKexpanded,'value')==1
    set(h.TEXThideexpanded,'visible','off');
else
    set(h.TEXThideexpanded,'visible','on');
end
guidata(gcf,h);

%%

%Unhide or hide Completed square quadratic checkbox
function CHECKsquare_Callback(hObject, eventdata, handles)
h = guidata(gcf);
if get(h.CHECKsquare,'value')==1
    set(h.TEXThidesquare,'visible','off');
else
    set(h.TEXThidesquare,'visible','on');
end
guidata(gcf,h);

%%

%Unhide or hide Factorized quadratic checkbox
function CHECKfactorized_Callback(hObject, eventdata, handles)
h = guidata(gcf);
if get(h.CHECKfactorized,'value')==1
    set(h.TEXThidefactorized,'visible','off');
else
    set(h.TEXThidefactorized,'visible','on');
end
guidata(gcf,h);

%%

%Toggle fix axes limits
function TOGGLEfixaxis_Callback(hObject, eventdata, handles)
h = guidata(gcf);
if get(h.TOGGLEfixaxis,'value')==1
    h.q.fix_axis = 1;
else
    h.q.fix_axis = 0;
end
guidata(gcf,h);

%%

%Toggle title
function CHECKtitle_Callback(hObject, eventdata, handles)
h = guidata(gcf);
axes(h.AXESquad);
if get(h.CHECKtitle,'value')==1
    set(h.q.title,'string',make_quad_title(h.q));
    h.q.show_title = 1;
else
    set(h.q.title,'string','');
    h.q.show_title = 0;
end
guidata(gcf,h);

%%

%Toggle plot
function CHECKplot_Callback(hObject, eventdata, handles)
h = guidata(gcf);
axes(h.AXESquad);
if get(h.CHECKplot,'value')==1
    set(h.TEXThideplot,'visible','off');
else
    set(h.TEXThideplot,'visible','on');
end
guidata(gcf,h);

%%

%Unhide or hide roots
function CHECKroots_Callback(hObject, eventdata, handles)
h = guidata(gcf);
axes(h.AXESquad);
if get(h.CHECKroots,'value')==1
    set(h.TEXThideroots,'visible','off');
else
    set(h.TEXThideroots,'visible','on');
end
guidata(gcf,h);

%%

%Unhide or hide vertex coordinates
function CHECKvertex_Callback(hObject, eventdata, handles)
h = guidata(gcf);
axes(h.AXESquad);
if get(h.CHECKvertex,'value')==1
    set(h.TEXThidevertex,'visible','off');
else
    set(h.TEXThidevertex,'visible','on');
end
guidata(gcf,h);

%%

%Update GUI from quadratic parameters in structured array q
function update_gui_from_quad(q)

%Get current GUI data
h = guidata(gcf);

%Update quadratic parameters in GUI

%Expanded form y = a*x^2 + b*x + c
set( h.EDITa,'string',num2str(n_dp(q.a,1)) );
set( h.EDITb,'string',signstr(n_dp(q.b,1)) );
set( h.EDITc,'string',signstr(n_dp(q.c,1)) );

%Completed square form y = alpha*( x + beta )^2 + gamma
set( h.EDITaa,'string',num2str(n_dp(q.alpha,1)) );
set( h.EDITbb,'string',signstr(n_dp(q.beta,1)) );
set( h.EDITcc,'string',signstr(n_dp(q.gamma,1)) );

%Factorized form y = (Ax+B)(Bx+C)
set(h.EDITU,'string',num2str(n_dp(q.A,1)) );
set(h.EDITV,'string',signstr(n_dp(q.B,1)) );
set(h.EDITQ,'string',num2str(n_dp(q.C,1)) );
set(h.EDITW,'string',signstr(n_dp(q.D,1)) );

%Compute vertex coordinates
set(h.TEXTvertex,'string',['( ',num2str(n_dp(q.vertex_x,2)),' , ',num2str(n_dp(q.vertex_y,2)),' )']);

%Compute roots
set(h.TEXTroot1,'string',['x = ',num2str(n_dp(q.xroot1,2))]);
set(h.TEXTroot2,'string',['x = ',num2str(n_dp(q.xroot2,2))]);

%Plot graph and update structure q in GUI data structure h
axes(h.AXESquad);
cla;
h.q = plot_quad(h.AXESquad,q);

%Update GUI data
guidata(gcf,h);

%%

%Update quadratic paremeters from GUI
function q = update_q_from_gui( quad_form )

%Get GUI data
h = guidata(gcf);
q.N = h.q.N;
q.fix_axis = h.q.fix_axis;
q.show_title = h.q.show_title;
q.xmin = h.q.xmin;
q.xmax = h.q.xmax;
q.ymin = h.q.ymin;
q.ymax = h.q.ymax;

%Update parameters based on quadratic form modified
if strcmp( quad_form, 'expanded' ) == 1
    %Expanded form y = a*x^2 + b*x + c
    q.a = str2num( get(h.EDITa,'string') );
    q.b = str2num( get(h.EDITb,'string') );
    q.c = str2num( get(h.EDITc,'string') );
    
    %Completed square form y = alpha*( x + beta )^2 + gamma
    q.alpha = q.a;
    q.beta = q.b/(2*q.a);
    q.gamma = q.c - q.b^2 / (4*q.a);
    
    %Compute vertex coordinates
    q.vertex_x = -q.b/(2*q.a);
    q.vertex_y = q.c - q.b^2 / (4*q.a);
    
    %Compute roots
    q.xroot1 = (-q.b-sqrt( q.b^2 -4*q.a*q.c ))/(2*q.a);
    q.xroot2 = (-q.b+sqrt( q.b^2 -4*q.a*q.c ))/(2*q.a);
    
    %Factorized form y = (Ax+B)(Bx+C) %This may not be the most elegant!%
    q.A = q.a;
    q.B = -q.A*q.xroot1;
    q.C = 1;
    q.D = -q.xroot2;
    
elseif strcmp( quad_form, 'completed square' ) == 1
    %Completed square form y = alpha*( x + beta )^2 + gamma
    q.alpha = str2num( get(h.EDITaa,'string') );
    q.beta = str2num( get(h.EDITbb,'string') );
    q.gamma = str2num( get(h.EDITcc,'string') );
    
    %Expanded form y = a*x^2 + b*x + c
    q.a = q.alpha;
    q.b = q.alpha * 2 * q.beta;
    q.c = q.alpha*q.beta^2 + q.gamma;
    
    %Compute vertex coordinates
    q.vertex_x = -q.b/(2*q.a);
    q.vertex_y = q.c - q.b^2 / (4*q.a);
    
    %Compute roots
    q.xroot1 = (-q.b-sqrt( q.b^2 -4*q.a*q.c ))/(2*q.a);
    q.xroot2 = (-q.b+sqrt( q.b^2 -4*q.a*q.c ))/(2*q.a);
    
    %Factorized form y = (Ax+B)(Cx+D)  %This may not be the most elegant!%
    q.A = q.a;
    q.B = -q.A*q.xroot1;
    q.C = 1;
    q.D = -q.xroot2;
    
elseif strcmp( quad_form, 'factorized' ) == 1
    %Factorized form y = (Ax+B)(Cx+D)
    q.A = str2num( get(h.EDITU,'string') );
    q.B = str2num( get(h.EDITV,'string') );
    q.C = str2num( get(h.EDITQ,'string') );
    q.D = str2num( get(h.EDITW,'string') );
    
    %Expanded form y = a*x^2 + b*x + c
    q.a = q.A*q.C;
    q.b = q.B*q.C + q.A*q.D;
    q.c = q.B*q.D;
    
    %Completed square form y = alpha*( x + beta )^2 + gamma
    q.alpha = q.a;
    q.beta = q.b/(2*q.a);
    q.gamma = q.c - q.b^2 / (4*q.a);
    
    %Compute vertex coordinates
    q.vertex_x = -q.b/(2*q.a);
    q.vertex_y = q.c - q.b^2 / (4*q.a);
    
    %Compute roots
    q.xroot1 = (-q.b-sqrt( q.b^2 -4*q.a*q.c ))/(2*q.a);
    q.xroot2 = (-q.b+sqrt( q.b^2 -4*q.a*q.c ))/(2*q.a);
end

%%

%% Generate random factorizable quadratic %%
function [A,B,C,D,a,b,c,expanded_quad_str,fact_quad_str] = gen_quad(N)

%Generate random signed integers for facorized quadratic expression
%y = (Ax+B)(Cx+D)
I = randi( [-N,N],[1,4] );

%Replace zeros by ones
I(I==0) = 1;

%Assign factorized quadratic coefficients
A = I(1);
B = I(2);
C = I(3);
D = I(4);

%Determine expanded form coefficients
a = A*C;
b = B*C+D*A;
c = B*D;

%Create string for factorized form
fact_quad_str = ['y = (',num2str(A),'x',signstr(B),')(',num2str(C),'x',signstr(D),')'];

%Create string for expanded form
expanded_quad_str = ['y = ',num2str(a),'x^2',signstr(b),'x',signstr(c)];

%%

%Create signed string from number x
function s = signstr(x)
if x<0
    s_sign = ' - ';
else
    s_sign = ' + ';
end
s = [s_sign,num2str(abs(x))];

%%

%Make quadratic graph title string
function s = make_quad_title(q)
s = {['y = ',num2str(q.a),'x^2 + ',num2str(q.b),'x + ',num2str(q.c)],...
    ['y = ',num2str(q.alpha),'( x',signstr(q.beta),' )^2',signstr(q.gamma)],...
    ['y = (',num2str(q.A),'x',signstr(q.B),')(',num2str(q.C),'x',signstr(q.D),')'] };

%%

%Compute limits of quadratic plot box such that all interesting features
%are contained. Quadratic is y = ax^2 + bx + c;
function [xmin,xmax,ymin,ymax] = quad_limits( vertex_x,vertex_y, xroot1, xroot2, a, c )

%Set x limits of y = x^2
M = 1;

%Smiles
if a > 0
    ymin = min( [c,vertex_y,0] );
    ymax = max( [c,vertex_y+M*M*a,0] );
else
    %Frowns
    ymin = min( [c,vertex_y-M*M*abs(a),0] );
    ymax = max( [c,vertex_y,0] );
end
xmin = min( [vertex_x-M, real(xroot1),real(xroot2),0] );
xmax = max( [vertex_x+M, real(xroot1),real(xroot2),0] );

%Expand plot box by 10% in each direction
d = xmax - xmin;
xmin = xmin - 0.1*d;
xmax = xmax + 0.1*d;
d = ymax - ymin;
ymin = ymin - 0.1*d;
ymax = ymax + 0.1*d;

%Round limits to one decimal place
xmin = n_dp(xmin,1);
xmax = n_dp(xmax,1);
ymin = n_dp(ymin,1);
ymax = n_dp(ymax,1);

%%

%Plot quadratic
function q = plot_quad(ax,q)

%Construct vector of data points to plot
N = 100;
[xmin,xmax,ymin,ymax] = quad_limits( q.vertex_x,q.vertex_y, q.xroot1, q.xroot2, q.a, q.c );
x = linspace(xmin,xmax,N);
y = q.a*(x.^2) + q.b*x + q.c;

%Plot quadratic curve
plot(x,y,'b','linewidth',1);
hold on

%Set axes limits
if q.fix_axis == 0
    set(gca,'xlim',[xmin,xmax])
    set(gca,'ylim',[ymin,ymax])
    q.xmin = xmin;
    q.xmax = xmax;
    q.ymin = ymin;
    q.ymax = ymax;
else
    set(gca,'xlim',[q.xmin,q.xmax])
    set(gca,'ylim',[q.ymin,q.ymax])
    xmin = q.xmin;
    xmax = q.xmax;
    ymin = q.ymin;
    ymax = q.ymax;
end

%Grid and tick marks
grid on
box on
xticks = xmin : n_dp((xmax - xmin)/5,1)  : xmax;
yticks = ymin : n_dp((ymax - ymin)/5,1)  : ymax;
set(ax,'xtick',xticks,'ytick',yticks);

%Annotations

%Set labels and title
xlabel('x')
ylabel('y')
q.title = title('','horizontalalignment','left','fontsize',14 );
if q.show_title == 1
    set(q.title,'string',make_quad_title(q));
end

%If abs(imag( root )) is less than tol we shall assume it to be zero!
tol = 1e-10;

%Text annotation x offset
d = (xmax-xmin)/50;

if abs(imag(q.xroot1))<tol
    plot( q.xroot1,0,'r*'); annotate_coordinate(q.xroot1,0,d)
    plot( q.xroot2,0,'m*'); annotate_coordinate(q.xroot2,0,d)
else
    plot( 0,q.c,'r*')
    plot( 0,q.c,'m*')
end
plot( 0,q.c,'g*'); annotate_coordinate(0,q.c,d)
plot( q.vertex_x, q.vertex_y, 'k*' ); annotate_coordinate(q.vertex_x,q.vertex_y,d)
legend('Quadratic','Root 1','Root 2', 'y intercept','Vertex',...
    'location','eastoutside')

%Plot x and y axis
plot([0,0],[ymin,ymax],'r','linewidth',1);
plot([xmin,xmax],[0,0],'r','linewidth',1);

%%

%Round to a fixed number of decimal places
function y = n_dp(x,N)
y = round(x*(10^N))/(10^N);

%%

%Annotate coordinates with (x,y) text
function annotate_coordinate(x,y,d)
text(x+d,y,['(',num2str( n_dp(x,1) ),',',num2str( n_dp(y,1) ),')' ],'fontsize',14);

%End of code