%shm
%
% Dynamic model of Forced Simple Harmonic Motion (SHM).
%
% LAST UPDATED by Andy French November 2013

function varargout = shm(varargin)

% Begin initialization code - DO NOT EDIT
gui_Singleton = 1;
gui_State = struct('gui_Name',       mfilename, ...
    'gui_Singleton',  gui_Singleton, ...
    'gui_OpeningFcn', @shm_OpeningFcn, ...
    'gui_OutputFcn',  @shm_OutputFcn, ...
    'gui_LayoutFcn',  [] , ...
    'gui_Callback',   []);
if nargin && ischar(varargin{1})
    gui_State.gui_Callback = str2func(varargin{1});
end

if nargout
    [varargout{1:nargout}] = gui_mainfcn(gui_State, varargin{:});
else
    gui_mainfcn(gui_State, varargin{:});
end
% End initialization code - DO NOT EDIT

%%

% GUI opening function
function shm_OpeningFcn(hObject, eventdata, handles, varargin)

% Choose default command line output for shm
handles.output = hObject;

% Update handles structure
guidata(hObject, handles);

%Centre figure
figcentre(gcf);

%Store GUI data in a global structure d
global d
d = handles;

%%% HARD CODED INPUTS %%%

%Set initial conditions
d.x0 = 1;
d.xdot0 = 0;

%Default values of parameters
d.tmin = 0;
d.tmax = 15;
d.xmin = -10;
d.xmax = 10;
d.wmin = 0;
d.wmax = 10;
d.Amin = 0;
d.Amax = 10;
d.gamma = 0.5;
d.omega0 = 5;
d.omega = 4.6;
d.A0 = 1;
d.x0 = 8;
d.xdot0 = 1;

%Fontsize of graph annotation
d.fsize = 12;

%Linewidth of graphs
d.linewidth = 2;

%Hard code the number of data points for both graphs
d.N = 1000;

%Hard code gamma, omega0, omega, A0, x0 and xdot0 increments
d.domega0 = 0.1;
d.domega = 0.1;
d.dgamma = 0.05;
d.dA0 = 0.1;
d.dx0 = 0.1;
d.dxdot0 = 0.1;

%%%%%%

%Set initial parameters
set(d.EDITtmin,'string', num2str(d.tmin) );
set(d.EDITtmax,'string', num2str(d.tmax));
set(d.EDITxmin,'string', num2str(d.xmin));
set(d.EDITxmax,'string', num2str(d.xmax));
set(d.EDITwmin,'string', num2str(d.wmin) );
set(d.EDITwmax,'string', num2str(d.wmax));
set(d.EDITAmin,'string', num2str(d.Amin));
set(d.EDITAmax,'string', num2str(d.Amax));
set(d.EDITgamma,'string',num2str(d.gamma) );
set(d.EDITomega0,'string', num2str(d.omega0));
set(d.EDITomega,'string', num2str(d.omega));
set(d.EDITA0,'string', num2str(d.A0) );
set(d.EDITx0,'string', num2str(d.x0) );
set(d.EDITv0,'string', num2str(d.xdot0) );

%Load SHM differential equation JPEG
axes(d.AXESeqa);
image(imread('SHM_eqa.jpg'));
axis image
axis off;

%Update the GUI
update_gui;

%%

%Modify maximum limit of x
function EDITxmax_Callback(hObject, eventdata, handles)
update_gui;

%%

%Modify minimum limit of x
function EDITxmin_Callback(hObject, eventdata, handles)
update_gui;

%%

%Modify maximum limit of t
function EDITtmax_Callback(hObject, eventdata, handles)
update_gui;

%%

%Modify minimum limit of t
function EDITtmin_Callback(hObject, eventdata, handles)
update_gui;

%%

%Modify maximum limit of A
function EDITAmax_Callback(hObject, eventdata, handles)
update_gui;

%%

%Modify minimum limit of A
function EDITAmin_Callback(hObject, eventdata, handles)
update_gui;

%%

%Modify maximum limit of omega
function EDITwmax_Callback(hObject, eventdata, handles)
update_gui;

%%

%Modify minimum limit of omega
function EDITwmin_Callback(hObject, eventdata, handles)
update_gui;

%%

%Auto scale axis toggle
function CHECKautoscale_Callback(hObject, eventdata, handles)
update_gui;

%%

%Edit gamma
function EDITgamma_Callback(hObject, eventdata, handles)
update_gui;

%%

%Edit omega0
function EDITomega0_Callback(hObject, eventdata, handles)
update_gui;

%%

%Edit omega
function EDITomega_Callback(hObject, eventdata, handles)
update_gui;

%%

%Edit A0
function EDITA0_Callback(hObject, eventdata, handles)
update_gui;

%%

%%

% Increment gamma
function PUSHgammaplus_Callback(hObject, eventdata, handles)
global d
d.gamma = d.gamma + d.dgamma;
set( d.EDITgamma,'string',num2str(d.gamma) );
update_gui;

%%

% Decrement gamma
function PUSHgammaminus_Callback(hObject, eventdata, handles)
global d
d.gamma = d.gamma - d.dgamma;
set( d.EDITgamma,'string',num2str(d.gamma) );
update_gui;

%%

% Increment omega0
function PUSHomega0plus_Callback(hObject, eventdata, handles)
global d
d.omega0 = d.omega0 + d.domega0;
set( d.EDITomega0,'string',num2str(d.omega0) );
update_gui;

%%

% Decrement omega0
function PUSHomega0minus_Callback(hObject, eventdata, handles)
global d
d.omega0 = d.omega0 - d.domega0;
set( d.EDITomega0,'string',num2str(d.omega0) );
update_gui;

%%

% Increment omega
function PUSHomegaplus_Callback(hObject, eventdata, handles)
global d
d.omega = d.omega + d.domega;
set( d.EDITomega,'string',num2str(d.omega) );
update_gui;

%%

% Decrement omega
function PUSHomegaminus_Callback(hObject, eventdata, handles)
global d
d.omega = d.omega - d.domega;
set( d.EDITomega,'string',num2str(d.omega) );
update_gui;

%%

% Increment A0
function PUSHA0plus_Callback(hObject, eventdata, handles)
global d
d.A0 = d.A0 + d.dA0;
set( d.EDITA0,'string',num2str(d.A0) );
update_gui;

%%

% Decrement A0
function PUSHA0minus_Callback(hObject, eventdata, handles)
global d
d.A0 = d.A0 - d.dA0;
set( d.EDITA0,'string',num2str(d.A0) );
update_gui;

%%

%Edit x0
function EDITx0_Callback(hObject, eventdata, handles)
update_gui;

%%

%Edit xdot0
function EDITv0_Callback(hObject, eventdata, handles)
update_gui;

%%

% Increment x0
function PUSHx0plus_Callback(hObject, eventdata, handles)
global d
d.x0 = d.x0 + d.dx0;
set( d.EDITx0,'string',num2str(d.x0) );
update_gui;

%%

% Decrement x0
function PUSHx0minus_Callback(hObject, eventdata, handles)
global d
d.x0 = d.x0 - d.dx0;
set( d.EDITx0,'string',num2str(d.x0) );
update_gui;

%%

% Increment xdot0
function PUSHv0plus_Callback(hObject, eventdata, handles)
global d
d.xdot0 = d.xdot0 + d.dxdot0;
set( d.EDITv0,'string',num2str(d.xdot0) );
update_gui;

%%

% Decrement xdot0
function PUSHv0minus_Callback(hObject, eventdata, handles)
global d
d.xdot0 = d.xdot0 - d.dxdot0;
set( d.EDITv0,'string',num2str(d.xdot0) );
update_gui;

%%

% GUI output function
function varargout = shm_OutputFcn(hObject, eventdata, handles)
varargout{1} = handles.output;

%%

% Update the GUI
function update_gui
global d

%Get parameters from GUI
d.gamma = str2num( get( d.EDITgamma,'string' ) );
d.omega0 = str2num( get( d.EDITomega0,'string' ) );
d.omega = str2num( get( d.EDITomega,'string' ) );
d.A0 = str2num( get( d.EDITA0,'string' ) );
d.x0 = str2num( get( d.EDITx0,'string' ) );
d.xdot0 = str2num( get( d.EDITv0,'string' ) );

%Get the values of the axis limits
d.tmin = str2num( get( d.EDITtmin, 'string' ) );
d.tmax = str2num( get( d.EDITtmax, 'string' ) );
d.xmin = str2num( get( d.EDITxmin, 'string' ) );
d.xmax = str2num( get( d.EDITxmax, 'string' ) );
d.wmin = str2num( get( d.EDITwmin, 'string' ) );
d.wmax = str2num( get( d.EDITwmax, 'string' ) );
d.Amin = str2num( get( d.EDITAmin, 'string' ) );
d.Amax = str2num( get( d.EDITAmax, 'string' ) );

%Determine time vector and omaga vector
t = linspace( d.tmin, d.tmax, d.N );
w = linspace( d.wmin, d.wmax, d.N );

%Determine A = |A(omega)| steady-state solution magntitude to enable
%resonance curve to be plotted
A_w = sqrt( (d.omega0^2 - w.^2 ).^2 + 4*w.*w*d.gamma^2 );
A_w = d.A0 *d.omega0^2* A_w.^-1;
A = sqrt( (d.omega0^2 - d.omega^2 ).^2 + 4*d.omega^2*d.gamma^2 );
A = d.A0 *d.omega0^2* A^-1;

%Compute phase /radians of steady state response x = Asin( w*t - phi )
phi_w = atan2( 2*d.gamma*w, d.omega0^2 -w.^2 );
phi = atan2( 2*d.gamma*d.omega, d.omega0^2 -d.omega^2 );

%Determine x(t)
if d.omega0 > d.gamma
    %Under-damped i.e. oscillatory transient solutions
        PHI = atan2( ( d.xdot0 + d.gamma*(d.x0 + A*sin(phi)) -A*d.omega*cos(phi) )/...
        ( (d.x0 + A*sin(phi) )*sqrt( d.omega0^2 - d.gamma^2 ) ), 1);
    AA = (d.x0 +A*sin(phi)) /cos(PHI);
    x = AA*exp(-d.gamma*t).*cos( t*sqrt( d.omega0^2 - d.gamma^2) - PHI ) +...
        A*sin( d.omega*t - phi );
elseif d.omega0 == d.gamma
    %Critically damped
    A1 = d.x0 + A*sin(phi);
    A2 = d.xdot0 + d.gamma*( d.x0 + A*sin(phi) ) - A*d.omega*cos(phi);
    x = exp( -d.gamma*t ).* ( A2*t + A1 ) + A*sin( d.omega*t - phi );
elseif d.omega0 < d.gamma
    %Over-damped
    q = sqrt( d.gamma^2 - d.omega0^2 );
    AA = 0.5*( d.x0 + A*sin(phi) );
    AAA = 0.5*( d.xdot0 + d.gamma*( d.x0 + A*sin(phi) ) -A*d.omega*cos(phi) )/q;
    A1 = AA + AAA;
    A2 = AA - AAA;
    x = exp( -d.gamma*t ).*( A1*exp( t*q ) + A2*exp(-t*q ) ) + A*sin( d.omega*t - phi );
end

%Plot t vs x
axes(d.AXEStvsx);
cla;
plot(t,x,'b','linewidth',d.linewidth);
xlabel('t','fontsize',d.fsize )
ylabel('x','fontsize',d.fsize )
set(d.AXEStvsx,'fontsize',d.fsize )
title('x vs t','fontsize',d.fsize )
set(d.AXEStvsx, 'xlim',[d.tmin,d.tmax],...
    'ylim',[d.xmin,d.xmax] )
grid on
set(d.AXEStvsx,'xtick',floor(d.tmin): (1/10)*floor( d.tmax-d.tmin  ): ceil( d.tmax),...
    'ytick',floor(d.xmin): (1/10)*floor( d.xmax-d.xmin ): ceil( d.xmax)  );
hold on
plot( [d.tmin,d.tmax],[0,0],'k','linewidth',d.linewidth );
plot( [0,0],[d.xmin,d.xmax],'k','linewidth',d.linewidth );
plot( t, d.x0*exp( -d.gamma*t ),'r--');
plot( t, -d.x0*exp( -d.gamma*t ),'r--');

%Plot resonance curve
axes(d.AXESwvsF);
cla;
plot(w,A_w,'r','linewidth',d.linewidth);
xlabel('\omega','fontsize',d.fsize )
ylabel('|x|','fontsize',d.fsize )
set(d.AXESwvsF,'fontsize',d.fsize )
title('Magnitude of steady state response','fontsize',d.fsize )
set(d.AXESwvsF, 'xlim',[d.wmin,d.wmax],'ylim',[d.Amin,d.Amax] )
grid on
set(d.AXESwvsF,'xtick',floor(d.wmin): (1/10)*floor(d.wmax-d.wmin): ceil( d.wmax),...
    'ytick',floor(d.Amin): (1/10)*floor( d.Amax-d.Amin): ceil( d.Amax)  );
hold on
plot( d.omega, A, 'b*')

%%

%figcenter
% Centres a figure with handle fig in the current display.
function figcentre(fig)

%Get screen dimensions in pixels.
set(0,'units','pixels') ;
screen_size=get(0,'Screensize');

%Get original figure units
orig_units = get(fig,'units');

%Get position vector for figure in pixels
set(fig,'units','pixels');
fig_position=get(gcf,'position') ;

%Define coordinates or figure bottom hand corner from screen bottom hand corner
%that will place the figure window in the center of the screen.
centre_x = 0.5*screen_size(3) - 0.5*fig_position(3);
centre_y = 0.5*screen_size(4) - 0.5*fig_position(4);

%Modify figure position to centre
set(fig,'position',[centre_x,centre_y,fig_position(3),fig_position(4)] );

%Reset figure units
set(fig,'units',orig_units);

%End of code
