function waves

A = 1;           %Amplitude
lambda = 0.5;    %Wavelength /m
c = 340;         %Wavespeed /ms^-1
f_unit = 'kHz';  %Frequency unit
t_unit = 'ms';   %Time unit
 
%Spatial sinusoid separation (fraction of wavelength)
dx = lambda/8;

fsize = 18;     %Fontsize
N = 1000;       %Number of data points 
Nwaves = 3;     %Number of waves to plot
linewidth = 2;  %Line width

%Determine period /s and frequency /Hz
f = c/lambda;
T = 1/f;
 
%Determine time delay associated with dx
dt = dx/c;
 
%Generate x,y and t,y data vectors
x = linspace( 0, Nwaves*lambda, N);
t = linspace( 0, Nwaves*T, N);

yx1 = A*sin( 2*pi*x/lambda );
yx2 = A*sin( 2*pi*( x- dx)/lambda );
yt = A*sin( 2*pi*t/T );

%Plot y vs x graph
figure('color',[1 1 1],'name','waves anatomy')
plot(x,yx1,'b-',x,yx2,'r-','linewidth',linewidth);
legend( {'t = 0','t = \Deltat'},'fontsize',fsize )
xlabel( 'x /m','fontsize',fsize )
title([' A = ',num2str(A),', \lambda = ',num2str(lambda),...
    ',m, c = ',num2str(c),'m/s'],'fontsize',fsize )
grid on;
set( gca,'fontsize',fsize )
axis tight
print( gcf,'wave y vs x.png','-dpng', '-r300' )
clf;
 
%Plot y vs t graph
plot(t/tuf( t_unit ),yt,'linewidth',linewidth);
xlabel( ['t /',t_unit],'fontsize',fsize )
title([' A = ',num2str(A),', \lambda = ',num2str(lambda),...
    ',m, c = ',num2str(c),'m/s, f = ',...
    num2str( f/fuf( f_unit),5),f_unit,', T = ',...
    num2str(T/tuf( t_unit ),5),t_unit],'fontsize',fsize )
grid on;
set( gca,'fontsize',fsize )
axis tight
print( gcf,'wave y vs t.png','-dpng', '-r300' )
close(gcf);
 
%%
 
%Frequency unit factor
function m = fuf( f_unit )
if strcmp( f_unit, 'THz' )==1
    m = 1e12;
elseif strcmp( f_unit, 'GHz' )==1
    m = 1e9;
elseif strcmp( f_unit, 'MHz' )==1
    m = 1e6;
elseif strcmp( f_unit, 'kHz' )==1
    m = 1e3;
else; m = 1; end
 
%
 
%Time unit factor
function m = tuf( t_unit )
if strcmp( t_unit, 'ps' )==1
    m = 1e-12;
elseif strcmp( t_unit, 'ns' )==1
    m = 1e-9;
elseif strcmp( t_unit, '\mus' )==1
    m = 1e-6;
elseif strcmp( t_unit, 'ms' )==1
    m = 1e-3;
else; m = 1; end
