%solve_sudoku
% Function which solves a sudoku x
%
% LAST UPDATED by Andy French 30th January 2010

function [x,C,iter] = solve_sudoku(x,na)

%Initialise candidates array C
C = cell(na,na);
for n=1:na*na
    C{n} = x(n);
end

%Check if Sudoku is bad in the first instance
if sudoku_isbad(x,C,na)
    disp('Sudoku badly formed and cannot be solved')
    return
end

%Iterate around deductive rules until either (i) Sudoku is solved or (ii)
%the following iteration results in no change to the Suduko (xchange=0)
x_old = x;
xchange = NaN;
iter = 0;
while (xchange~=0) && ~sudoku_issolved(x)

    %Obtain candidates array C and check for deductions.
    [C,x] = sudoku_candidates( x, C, na );

    %Run other logical tests and aim to reduce possible number of
    %candidates and make further deductions
    [C,x] = sudoku_samecandidates( x, C, na );
    [C,x] = sudoku_rowsandcols( x, C, na );
    [C,x] = sudoku_uniquelocation( x, C, na );

    %Has Sudoku changed in this iteration?
    xchange = ~isequal(x,x_old);
    x_old = x;
end

%Apply a bad Sudoku check
if sudoku_isbad(x,C,na)
    disp('Sudoku is incorrect. A bug is to be found in the code logic!')
    return
end

%If Sudoku is not solved, apply a recursive guessing method
if ~sudoku_issolved(x)
    [x,C,iter] = sudoku_recursive( x, C, na, iter );
end

%If Sudoku is still not solved (sudoku_recursive will terminate if a 'bad'
%solution is reached), admit defeat!
if ~sudoku_issolved(x)
    disp('Cannot solve this pesky Sudoku!')
    [C,x,bad] = sudoku_candidates( x, na );
else
    %Suduko is solved. Check that Sudoku is not bad as a final check.
%     for n=1:na*na
%         C{n} = x(n);
%     end
    bad_check = sudoku_isbad(x,C,na)
end

%End of code