%sudoku_candidates
% Function which computes a cell array C of candidate values for
% corresponding elements of a sudoku x of alphabet 1:na, where na is a
% positive integer. i.e. dimensions of sudoku must be [na,na].
% If any candidate value are 'naked singles' i.e. a
% unique value, the sudoku is updated with these values.
%
% LAST UPDATED by Andy French 30th January 2010

function [C,x] = sudoku_candidates( x, C, na )

%Check whether sudoku has been solved
if sudoku_issolved(x)
    return
end

%Compute H,V and S arrays of undeduced values
[H,V,S] = sudoku_compute_HVS(x,na);

%Compute 'possibilities' cell array C
for n=1:na
    for m=1:na

        %Element (n,m) is an unknown number in sudoku
        if x(n,m)==0

            %Determine square set index
            ns = sudoku_square_set_index( na, n,m );

            %Compute candidates for sudoku number (n,m)
            C{n,m} = intersect( S{ns},...
                intersect( H{n}, V{m} ) );

            %Check whether candidates C{n,m} is a unique value. If so,
            %update the sudoku element x(n,m). If C{n,m} is [] then set the
            %bad flag to be 1.
            if length(C{n,m})==1
                x(n,m) = C{n,m};
            end
        else
            %Element (n,m) is a defined number in sudoku.
            %There is only one possibility for this element!
            C{n,m} = x(n,m) ;
        end
    end
end

%End of code