%sudoku_recursive
% Trial & error 'last resort' to solving a sudoku. Based upon an algorithm
% by Cleve Moler ("Cleve's Corner: Solving Sudoku with MATLAB", Jan 2010)
%
% LAST UPDATED by Andy French 30th January 2010

function [x,C,iter] = sudoku_recursive( x, C, na, iter )

%Increment iteration
iter = iter+1;

%Compute array of candidates C for sudoku C, and update x with any single
%values candidate elements.
[C,x] = sudoku_candidates( x, C, na );

%Check whether Sudoku has been solved or if C contains an empty element,
%indicating a guessed Sudoku element is incorrect. If either of these is
%true, exit the function.
if sudoku_issolved(x) || sudoku_isbad(x,C,na)
    return
end

%If Sudoku is not solved, or 'bad', apply recursive backtracking; i.e. Make
%a guess for first unknown Sudoku element, re-call this function. If it
%results in a 'bad' Sudoku, step through possible candidates until the
%correct one is found.

%Store Sudoku as x_old just in case this choice results in a bad Sudoku.
x_old = x;

%Determine vector of values v corresponding to first unknown value of the
%Sudoku
unknowns = find( x==0 );
v = C{ unknowns(1) };

%Step through v and see if Suduko is solved or not, following guessing of
%candidates
for z=1:length(v)

    %Reset Sudoku prior to guessing
    x = x_old;

    %Make a guess and recursively call this function to see if this solves
    %the Sudoku
    x( unknowns(1) ) = v(z);
    [x,C,iter] = sudoku_recursive( x, C, na, iter );

    %Check whether Sudoku has been solved. If so, exit the function. If
    %not, continue to the next candidate value.
    if sudoku_issolved(x)
        return
    end
end

%End of code