%sudoku_uniquelocation
% Values in each H,V and S set are checked at all positions to see if a
% unique location can be proven. If this is the case then the sudoku value
% at this unique location is deduced.

% LAST UPDATED by Andy French 30th January 2010

function [C,x] = sudoku_uniquelocation( x, C, na )

%Check whether sudoku has been solved
if sudoku_issolved(x)
    return
end

%Get H,V & S sets
[H,V,S] = sudoku_compute_HVS(x,na);

%%%% H sets %%%%
for n=1:na
    if isempty(H{n})
        break;
    else
        %Step through elements of H{n} and try all locations. If only a single
        %location is allowed, assign this value to the Sudoku x.
        for m=1:length( H{n} )
            locations = find( x(n,:)==0 );
            if isempty(locations)
                break;
            else
                location_check = zeros(size(locations));
                for p=1:length(locations)
                    %Check whether H{n}(m) is a member of the Sqaure set at
                    %location x(n,locations(p))
                    location_check(p) = ~isempty( intersect( H{n}(m),...
                        S{sudoku_square_set_index( na, n,locations(p) )} ));
                end

                %Does binary vector location_check have a single unity
                %element? If so then this is the new deduction!
                unities = find(location_check==1);
                if length(unities)==1
                    x(n,locations(unities)) = H{n}(m);
                    C{n,locations(unities)} = H{n}(m);
                end
            end
        end
    end
end

%Get H,V & S sets
[H,V,S] = sudoku_compute_HVS(x,na);

%%%% V sets %%%%
for n=1:na
    if isempty(V{n})
        break;
    else
        %Step through elements of V{n} and try all locations. If only a single
        %location is allowed, assign this value to the Sudoku x.
        for m=1:length( V{n} )
            locations = find( x(:,n)==0 );
            if isempty(locations)
                break;
            else
                location_check = zeros(size(locations));
                for p=1:length(locations)
                    %Check whether V{n}(m) is a member of the Sqaure set at
                    %location x(locations(p),n)
                    location_check(p) = ~isempty( intersect( V{n}(m),...
                        S{sudoku_square_set_index( na, locations(p),n )} ));
                end

                %Does binary vector location_check have a single unity
                %element? If so then this is the new deduction!
                unities = find(location_check==1);
                if length(unities)==1
                    x(locations(unities),n) = V{n}(m);
                    C{locations(unities),n} = V{n}(m);
                end
            end
        end
    end
end

%Get H,V & S sets
[H,V,S] = sudoku_compute_HVS(x,na);

%%%% S sets %%%%
for n=1:na
    if isempty(S{n})
        break;
    else

        %Determine Sudoku indices corresponding to Square set S{n}
        [rows_s,cols_s] = sudoku_indices_from_S(n,na);

        %Step through elements of S{n} and try all locations. If only a single
        %location is allowed, assign this value to the Sudoku x.
        for m=1:length( S{n} )

            %Get locations in Square set which correspond to zero values
            locations=[];
            for z=1:na
                if x( rows_s(z),cols_s(z) )==0
                    locations = [locations,z];
                end
            end
            if isempty(locations)
                break;
            else
                location_check = zeros(size(locations));
                for p=1:length(locations)
                    %Check whether S{n}(m) is a member of the V set at
                    %location x(rows_s(locations(p)),cols_s(locations(p)))
                    %and the H set at location x(rows_s(locations(p)),cols_s(locations(p)))
                    location_check(p) = ~isempty( intersect( S{n}(m),...
                        V{ cols_s(locations(p)) } )) && ...
                        ~isempty( intersect( S{n}(m),...
                        H{ rows_s(locations(p)) } ));
                end

                %Does binary vector location_check have a single unity
                %element? If so then this is the new deduction!
                unities = find(location_check==1);
                if length(unities)==1
                    x( rows_s(locations(unities)),...
                        cols_s(locations(unities)) ) = S{n}(m);
                    C{rows_s(locations(unities)),...
                        cols_s(locations(unities)) } = S{n}(m);
                end
            end
        end
    end
end

%End of code