function sudokue(varargin)

%  SUDOKUE play sudoku with MATLAB (aids for solving included)
%
%   A guide on how sudoku works can be found e.g. on 
%   http://en.wikipedia.org/wiki/Sudoku
% 
%   This program has a history-function, i.e. you can undo and redo each
%   single step. You can also save and load a score including all settings
%   (execpt for the branching solver).
%
%   New games can be imported either as ASCII-string or from sdm- or
%   sdk-files resprectively. Puzzles in ASCII-Format or as sdm- or sdk-file
%   respectively can be found e.g. on http://www.sudocue.net/
%   
%   Aids for solving the puzzle can be found on the menu entry "solver": If
%   you enable tooltips, then all possible numbers for each fields are
%   shown as tooltip when mouse cursor is placed over the corresponding
%   field. There is a semiautomatic, which shows all fields whose solution
%   is unique regarding the logical constraints, but in opposite to the
%   automatic solver, the values are just taken if one clicks on it. In
%   addition to the automatik solver, which just evaluates the logical
%   constraints, there is an additional branching algorithm implemented,
%   who solves any arbitrary sudoku. This algorithm will not check, whether
%   this solution is unique, but it is guaranteed, that the solution is
%   valid.

%   (c) Matthias Schwaiger, 2007
%   <matthias.schwaiger@gmx.de>

sudoku_version='$Revision: 1.1 $';
sudoku_datum='$Date: 2007/01/15 07:19:00 $';

userdata.version=sudoku_version(12:length(sudoku_version)-2);
userdata.datum=sudoku_datum(8:length(sudoku_datum)-2);

if nargin > 0 && ~isempty(findobj('tag',mfilename))
    sudoku_cb(varargin{1});
    return
end

if ~isempty(findobj('tag',mfilename))
    figure(findobj('tag',mfilename));
    return
end

sudoku_scr=get(0,'screensize');
sudoku_fig_width=360;
sudoku_fig_height=300;
sudoku_fig=figure('menubar','none', 'numbertitle','off', ...
    'position',[(sudoku_scr(3)-sudoku_fig_width)/2 (sudoku_scr(4)-sudoku_fig_height)/2 sudoku_fig_width sudoku_fig_height],...
    'resize','off', ...
    'tag',mfilename,'CloseRequestFcn',[mfilename '(103);']);
eval('set(sudoku_fig,''DockControl'',''off'')','');

feldanteil=0.9;
statusleistenhoehe=0.07;
for k=1:9
    for m=1:9
        sudoku_feld((k-1)*9+m)=uicontrol('parent', sudoku_fig, 'units', 'normalized', 'style','edit',...
            'fontsize',12,'fontweight','bold','Horizontalalignment','center','Backgroundcolor',[0.7 0.7 0.7],...
            'position',[(1-feldanteil)/10+(m-1)*((feldanteil/9)+(1-feldanteil)/10) ...
            1-(feldanteil/9)*(1-statusleistenhoehe)-((1-feldanteil)/10+(k-1)*((feldanteil/9)*(1-statusleistenhoehe)+(1-feldanteil)/10)) ...
            (feldanteil/9) (feldanteil/9)*(1-statusleistenhoehe)]);%#ok
        if floor(((floor((k-1)/3)+1)+(floor((m-1)/3)+1))/2)==((floor((k-1)/3)+1)+(floor((m-1)/3)+1))/2
            set (sudoku_feld((k-1)*9+m),'Backgroundcolor',[0.8 0.8 0.8]);
        end
    end
end

sudoku_feld(99)=uicontrol('parent', sudoku_fig, 'units', 'normalized', 'style','text',...
    'Horizontalalignment','left', ...
    'fontsize',8,'position',[(1-feldanteil)/10 0 1-2*(1-feldanteil)/10 statusleistenhoehe-(1-feldanteil)/10]);

menu_datei=uimenu('label','&File');
sudoku_feld(104)=uimenu(menu_datei,'label','&New','accelerator','n');
sudoku_feld(105)=uimenu(menu_datei,'label','New from &Keyboard','accelerator','k');
sudoku_feld(106)=uimenu(menu_datei,'label','New from &file','accelerator','f');
sudoku_feld(100)=uimenu(menu_datei,'label','&Open','separator','on','accelerator','o');
sudoku_feld(101)=uimenu(menu_datei,'label','&Save','accelerator','s');
sudoku_feld(102)=uimenu(menu_datei,'label','Save &as...');
sudoku_feld(103)=uimenu(menu_datei,'label','Quit','separator','on');

menu_bearbeiten=uimenu('label','&Edit');
sudoku_feld(110)=uimenu(menu_bearbeiten,'label','&Undo','accelerator','z');
sudoku_feld(111)=uimenu(menu_bearbeiten,'label','&Redo','accelerator','y');

menu_loeser=uimenu('label','&Solver');
sudoku_feld(120)=uimenu(menu_loeser,'label','&Tooltips','accelerator','t');
sudoku_feld(121)=uimenu(menu_loeser,'label','&Semiautomatic','separator','on','accelerator','m');
sudoku_feld(122)=uimenu(menu_loeser,'label','&Automatic','accelerator','a');
sudoku_feld(123)=uimenu(menu_loeser,'label','Autom. w. &branching','accelerator','b');

menu_hilfe=uimenu('label','&Help');
sudoku_feld(130)=uimenu(menu_hilfe,'label','&Help');
sudoku_feld(131)=uimenu(menu_hilfe,'label','&About SUDOKU','separator','on');

for k=1:length(sudoku_feld)
    if sudoku_feld(k)~=0
        set(sudoku_feld(k),'callback',[mfilename '(' num2str(k) ')']);
    end
end

userdata.sudoku_feld=sudoku_feld;

set(sudoku_fig,'userdata',userdata);
sudoku_cb(0);

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
function sudoku_cb(choice)

sudoku_fig=findobj('tag',mfilename);
userdata=get(sudoku_fig,'userdata');

sudoku_feld=userdata.sudoku_feld;
set(sudoku_feld(99),'string',[char(169) ' Matthias Schwaiger, 2007'],'horizontalalignment','right','Foregroundcolor',[0 0 0],'enable','off');

if choice == 0
    userdata.datei='';
    userdata.gespeichert=1;
    userdata.tooltips=0;
    userdata.automatik=0;
    userdata.halbautomatik=0;
    userdata.baumsuche=0;
    userdata.suchfelder=[];
    userdata.schrittzaehler=1;
    userdata.maxschritt=1;
    userdata.sudokudaten=zeros(81,82);
    userdata.sudokubounds=ones(81,82)*(2^9-1);
end

if any(choice == 1:81)
    nummer=get(sudoku_feld(choice),'string');
    set(sudoku_feld(choice),'style','edit');
    if isempty(nummer) || userdata.sudokudaten(choice,userdata.schrittzaehler) ~= 0
        return
    end
    nummer=max(1,min(9,round(str2num(nummer(1)))));%#ok
    
    % berprfung, ob Zahl berhaupt gesetzt werden darf
    if ~bitget(userdata.sudokubounds(choice,userdata.schrittzaehler),nummer)
        set(sudoku_feld(choice),'foregroundcolor',[1 0 0]);
        set(sudoku_feld(99),'string','Clash: Number must not be set!', ...
            'foregroundcolor',[1 0 0],'horizontalalignment','left','enable','on');
        return
    end

    userdata.schrittzaehler=userdata.schrittzaehler+1;
    userdata.maxschritt=userdata.schrittzaehler;

    userdata.sudokudaten(:,userdata.schrittzaehler)=userdata.sudokudaten(:,userdata.schrittzaehler-1);
    userdata.sudokudaten(choice,userdata.schrittzaehler)=nummer;

    userdata.sudokubounds(:,userdata.schrittzaehler)=userdata.sudokubounds(:,userdata.schrittzaehler-1);
    zeile=floor((choice-1)/9)+1;
    spalte=choice-(floor((choice-1)/9)*9);
    for k=1:9
        % Ziffern in der Zeile drfen nicht mehr gesetzt werden
        userdata.sudokubounds((zeile-1)*9+k,userdata.schrittzaehler)= ...
            bitset(userdata.sudokubounds((zeile-1)*9+k,userdata.schrittzaehler),nummer,0);
        % Ziffern in der Spalte drfen nicht mehr gesetzt werden
        userdata.sudokubounds((k-1)*9+spalte,userdata.schrittzaehler)= ...
            bitset(userdata.sudokubounds((k-1)*9+spalte,userdata.schrittzaehler),nummer,0);
    end
    zeileblock=floor((zeile-1)/3)+1;
    spalteblock=floor((spalte-1)/3)+1;
    for k=1:3
        for m=1:3
            userdata.sudokubounds((((zeileblock-1)*3+(k-1))*9+((spalteblock-1)*3+m)),userdata.schrittzaehler)= ...
                bitset(userdata.sudokubounds((((zeileblock-1)*3+(k-1))*9+((spalteblock-1)*3+m)),userdata.schrittzaehler),nummer,0);
        end
    end
end

if choice==100 %Laden
    if ~userdata.gespeichert
        ButtonName = questdlg('Do you want to save the game?', ...
            'Quit Sudoku', ...
            'Yes', 'No', 'Cancel', 'Cancel');
        switch ButtonName,
            case 'Yes',
                eval([mfilename '(102);'],'');
            case 'Cancel',
                return;
        end
    end
    [filename, pathname] = uigetfile({'*.mat','MATLAB mat-files (*.mat)';'*.*','All files (*.*)'}, 'Load Sudoku-game');
    if ~isequal(filename,0) && ~isequal(pathname,0)
        clear sudokudata;
        load(fullfile(pathname, filename),'sudokudata');
    else
        return
    end
    try
        if ~strcmp(sudokudata.id,'SUDOKU - (c) Matthias Schwaiger, 2007')%#ok
            set(sudoku_feld(99),'string','The filen chosen is not a valid Sudoku-file.','horizontalalignment','left','enable','on');
            return
        end
    catch
        set(sudoku_feld(99),'string','The filen chosen is not a valid Sudoku-file.','horizontalalignment','left','enable','on');
        return
    end
    userdata.datei=fullfile(pathname, filename);
    userdata.gespeichert=1;
    userdata.baumsuche=0;
    userdata.tooltips=sudokudata.tooltips;
    userdata.automatik=sudokudata.automatik;
    userdata.halbautomatik=sudokudata.halbautomatik;
    userdata.suchfelder=eval('sudokudata.suchfelder','[]');
    userdata.schrittzaehler=sudokudata.schrittzaehler;
    userdata.maxschritt=sudokudata.maxschritt;
    userdata.sudokudaten=sudokudata.sudokudaten;
    userdata.sudokubounds=sudokudata.sudokubounds;
    set(sudoku_feld(99),'string','The game has been loaded.','horizontalalignment','left','enable','on');
end

if choice==102 %Speichern unter
    aktdir=pwd;
    if ~isempty(userdata.datei)
        temp=findstr(userdata.datei,filesep);
        cd(userdata.datei(1:temp(end)));
        [filename, pathname] = uiputfile({'*.mat','MATLAB mat-files (*.mat)';'*.*','All files (*.*)'}, 'Save Sudoku-Game',userdata.datei(temp(end)+1:end));
    else
        [filename, pathname] = uiputfile({'*.mat','MATLAB mat-files (*.mat)';'*.*','All files (*.*)'}, 'Save Sudoku-Game');
    end
    cd(aktdir);
    if ~isequal(filename,0) && ~isequal(pathname,0)
        userdata.datei=fullfile(pathname, filename);
    else
        return
    end
end

if any(choice==[101 102]) % Speichern ...
    if isempty(userdata.datei)
        return
    end
    sudokudata.id='SUDOKU - (c) Matthias Schwaiger, 2007';
    sudokudata.version=userdata.version;
    sudokudata.tooltips=userdata.tooltips;
    sudokudata.automatik=userdata.automatik;
    sudokudata.halbautomatik=userdata.halbautomatik;
    sudokudata.suchfelder=userdata.suchfelder;
    sudokudata.schrittzaehler=userdata.schrittzaehler;
    sudokudata.maxschritt=userdata.maxschritt;
    sudokudata.sudokudaten=userdata.sudokudaten;
    sudokudata.sudokubounds=userdata.sudokubounds;
    save(userdata.datei,'sudokudata');
    userdata.gespeichert=1;
    set(sudoku_feld(99),'string','The game has been saved..','horizontalalignment','left','enable','on');
end

if choice==103 %Ende
    if ~userdata.gespeichert
        ButtonName = questdlg('Do you want to save the game?', ...
            'Quit Sudoku', ...
            'Yes', 'No', 'Cancel', 'Cancel');
        switch ButtonName,
            case 'Yes',
                eval([mfilename '(102);'],'');
            case 'Cancel',
                return;
        end
    end

    delete(gcf);
    return
end

if any(choice==104:106)
    if ~userdata.gespeichert
        ButtonName = questdlg('Do you want to save the game?', ...
            'Quit Sudoku', ...
            'Yes', 'No', 'Cancel', 'Cancel');
        switch ButtonName,
            case 'Yes',
                eval([mfilename '(102);'],'');
            case 'Cancel',
                return;
        end
    end

    userdata.datei='';
    userdata.gespeichert=1;
    userdata.schrittzaehler=1;
    userdata.maxschritt=1;
    userdata.sudokudaten=zeros(81,82);
    userdata.sudokubounds=ones(81,82)*(2^9-1);
    userdata.baumsuche=0;
    set(sudoku_fig,'userdata',userdata);
    drawnow;
end

if any(choice==105)
    antwort=inputdlg({'Input Sudoku as ASCII-string:'},'New from keyboard',1,...
        {'620040080800900000001000502000001003000623000700400000305000200000006007090080035'});
    if isempty(antwort)
        return
    end
    asciizeile=antwort{1};
    if length(asciizeile)~=81
        set(sudoku_feld(99),'string','Input is faulty!', ...
            'foregroundcolor',[0 0 0],'horizontalalignment','left','enable','on');
        return
    end
    for k=1:length(asciizeile)
        if str2double(asciizeile(k)) < 0 || str2double(asciizeile(k)) > 9
            set(sudoku_feld(99),'string','Input is faulty!', ...
                'foregroundcolor',[0 0 0],'horizontalalignment','left','enable','on');
            return
        end
    end
    for k=1:length(asciizeile)
        if str2double(asciizeile(k))~=0
            set(sudoku_feld(k),'string',str2double(asciizeile(k)));
            eval([mfilename '(' num2str(k) ');'],'');
        end
    end
    set(sudoku_feld(99),'string','Have fun ...', ...
        'foregroundcolor',[0 0 0],'horizontalalignment','left','enable','on');
    return
end

if any(choice==106)
    [filename, pathname, filterindex] = uigetfile({'*.sdm','Sudocue SDM-files (*.sdm)';'*.sdk','Sudocue SDK-files (*.sdk)'}, 'Load new Sudoku-Puzzle');
    if ~isequal(filename,0) && ~isequal(pathname,0)
        sdmzeile=[];
        rfid=fopen(fullfile(pathname,filename),'rt');
        while 1
            temp=fgetl(rfid);
            if ~ischar(temp)
                break
            end
            if filterindex==1
                sdmzeile{length(sdmzeile)+1}=temp;
            elseif ~strncmp(temp,'#',1)
                sdmzeile=[sdmzeile temp];%#ok
            end
        end
        fclose(rfid);
    else
        return
    end
    
    randnummer=0;
    if filterindex==1
        rand('state',sum(100*clock));
        randnummer=ceil(rand(1)*length(sdmzeile));
        asciizeile=sdmzeile{randnummer};
    else
        asciizeile=sdmzeile;
        asciizeile(findstr(asciizeile,'.'))='0';
    end
    if length(asciizeile)~=81
        set(sudoku_feld(99),'string','The data loaded are faulty!', ...
            'foregroundcolor',[0 0 0],'horizontalalignment','left','enable','on');
        return
    end
    for k=1:length(asciizeile)
        if str2double(asciizeile(k)) < 0 || str2double(asciizeile(k)) > 9
            set(sudoku_feld(99),'string','The data loaded are faulty!', ...
                'foregroundcolor',[0 0 0],'horizontalalignment','left','enable','on');
            return
        end
    end
    for k=1:length(asciizeile)
        if str2double(asciizeile(k))~=0
            set(sudoku_feld(k),'string',str2double(asciizeile(k)));
            eval([mfilename '(' num2str(k) ');'],'');
        end
    end
    if filterindex==1
        set(sudoku_feld(99),'string',['The ' num2str(randnummer) '. record (out of ' num2str(length(sdmzeile)) ' records) has been loaded.'], ...
            'foregroundcolor',[0 0 0],'horizontalalignment','left','enable','on');
    else
        set(sudoku_feld(99),'string','The data have been loaded. Enjoy!', ...
            'foregroundcolor',[0 0 0],'horizontalalignment','left','enable','on');
    end
    return
end

if choice==110 %Rckgngig
    if userdata.schrittzaehler ~= 1
        userdata.schrittzaehler=userdata.schrittzaehler-1;
    end
end

if choice==111 %Wiederherstellen
    if userdata.schrittzaehler < userdata.maxschritt
        userdata.schrittzaehler=userdata.schrittzaehler+1;
    end
end

if choice==120 %Tooltips
    if userdata.tooltips
        userdata.tooltips=0;
    else
        userdata.tooltips=1;
    end
end

if choice==121 %Halbautomatik
    if userdata.halbautomatik
        userdata.halbautomatik=0;
    else
        userdata.halbautomatik=1;
    end
    userdata.automatik=0;
    userdata.baumsuche=0;
end

if choice==122 %Automatik
    if userdata.automatik && ~userdata.baumsuche
        userdata.automatik=0;
    else
        userdata.automatik=1;
    end
    userdata.halbautomatik=0;
    userdata.baumsuche=0;
end

if choice==123 % Baumsuche
    userdata.automatik=1;
    userdata.baumsuche=1;
    userdata.halbautomatik=0;
end

if any(choice==[100 104 110:111 121:123])
    for k=1:81
        set(sudoku_feld(k),'style','edit');
    end
end

if choice==130 %Hilfe
    uiwait(msgbox(help(mfilename),'Help on Sudoku','help','modal'));
end

if choice==131 %ber
    uiwait(msgbox(['SUDOKU' char(10) char(10) ...
        'Version ' userdata.version char(10) ...
        userdata.datum char(10) char(10) ...
        char(169) ' Matthias Schwaiger, 2007'],'About Sudoku','modal'));
end

% Aktionen, die eine Vernderung des Speicherstatus bewirken
if any(choice == [1:81 110:129])
    userdata.gespeichert=0;
end

% Aktualisieren der Zahlen in den Feldern (nach dem Laden, etc.)
if any(choice==[1:81 100 104 110 111 121:123])
    for k=1:81
        if userdata.sudokudaten(k,userdata.schrittzaehler) ~= 0
            set(sudoku_feld(k),'string',num2str(userdata.sudokudaten(k,userdata.schrittzaehler)),...
                'foregroundcolor',[0 0 0]);
        else
            set(sudoku_feld(k),'string','','foregroundcolor',[0 0 0]);
        end
    end
end

% Aktualisieren der tooltips in den Feldern (nach dem Laden, etc.)
if any(choice==[0 1:81 100 104 110 111 120])
    if userdata.tooltips
        for k=1:81
            if userdata.sudokudaten(k,userdata.schrittzaehler) ~= 0
                set(sudoku_feld(k),'tooltipstring','');
            else
                bounds=[];
                for m=1:9
                    if bitget(userdata.sudokubounds(k,userdata.schrittzaehler),m)
                        bounds=[bounds m];%#ok
                    end
                end
                set(sudoku_feld(k),'tooltipstring',num2str(bounds));
            end
        end
    else
        for k=1:81
            set(sudoku_feld(k),'tooltipstring','');
        end
    end
end

% Irrwege anzeigen und Automatikfunktionen realisieren
Automatikfeld=[];
Zahl=0;
if any(choice==[0 1:81 100 104 110 111 121:123])
    for k=1:81
        if userdata.sudokudaten(k,userdata.schrittzaehler) == 0
            bounds=0;
            for m=1:9
                if bitget(userdata.sudokubounds(k,userdata.schrittzaehler),m)
                    bounds=bounds+1;
                    Zahl=m;
                end
            end
            if bounds == 0
                set(sudoku_feld(k),'string','X','foregroundcolor',[1 0 0]);
                set(sudoku_feld(99),'string','Error: Some fields can not be set any more!', ...
                    'foregroundcolor',[1 0 0],'horizontalalignment','left','enable','on');
            end
            if bounds == 1
                if userdata.automatik
                    Automatikfeld=k;
                    Automatikzahl=Zahl;
                end
                if userdata.halbautomatik
                    set(sudoku_feld(k),'string',num2str(Zahl),'foregroundcolor',[0 1 0],'style','pushbutton');
                end
            end
        end
    end
    if isempty(Automatikfeld) && userdata.automatik && userdata.baumsuche && ...
            ~(all(userdata.sudokudaten(:,userdata.schrittzaehler) ~= 0))
        if isempty(userdata.suchfelder)
            [suchfelder,flag]=baumsuche(userdata.sudokudaten(:,userdata.schrittzaehler),...
                userdata.sudokubounds(:,userdata.schrittzaehler));
            if flag
                userdata.suchfelder=suchfelder;
            else
                set(sudoku_feld(99),'string','Error: The branching algorithm indicates that the puzzle is insolvable!', ...
                    'foregroundcolor',[1 0 0],'horizontalalignment','left','enable','on');
            end
        end
        if ~isempty(userdata.suchfelder)
            Automatikfeld=userdata.suchfelder(end,1);
            Automatikzahl=userdata.suchfelder(end,2);
            userdata.suchfelder(end,:)=[];
        end
    end
end


if any(choice==[1:81 100 104 110 111 120:123]) && all(userdata.sudokudaten(:,userdata.schrittzaehler) ~= 0)
    set(sudoku_feld(99),'string','Congratulations: The puzzle has been solved.','horizontalalignment','left','enable','on');
end

% Menaktionen einschrnken und Hkchen setzen
if isempty(userdata.datei) || userdata.gespeichert
    set(sudoku_feld(101),'enable','off');
else
    set(sudoku_feld(101),'enable','on');
end

if userdata.schrittzaehler==1
    set(sudoku_feld(110),'enable','off');
else
    set(sudoku_feld(110),'enable','on');
end

if userdata.schrittzaehler==userdata.maxschritt
    set(sudoku_feld(111),'enable','off');
else
    set(sudoku_feld(111),'enable','on');
end

if userdata.tooltips
    set(sudoku_feld(120),'checked','on');
else
    set(sudoku_feld(120),'checked','off');
end

if userdata.halbautomatik
    set(sudoku_feld(121),'checked','on');
else
    set(sudoku_feld(121),'checked','off');
end

if userdata.automatik
    if userdata.baumsuche
        set(sudoku_feld(122),'checked','off');
        set(sudoku_feld(123),'checked','on');
    else
        set(sudoku_feld(122),'checked','on');
        set(sudoku_feld(123),'checked','off');
    end
else
    set(sudoku_feld(122),'checked','off');
    set(sudoku_feld(123),'checked','off');
end

name='Sudoku';
if ~isempty(userdata.datei)
    temp=findstr(userdata.datei,filesep);
    name=[name ' - ' userdata.datei(temp(end)+1:end)];
    if ~userdata.gespeichert
        name=[name '*'];
    end
end

set(sudoku_fig,'name',name);
set(sudoku_fig,'userdata',userdata);
drawnow;

if any(choice==[1:81 122 123]) && userdata.automatik && ...
        ~(all(userdata.sudokudaten(:,userdata.schrittzaehler) ~= 0)) && ~isempty(Automatikfeld)
    set(sudoku_feld(Automatikfeld),'string',num2str(Automatikzahl));
    drawnow;
    eval([mfilename '(' num2str(Automatikfeld) ');'],'');
end

% %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
function [suchfelder,flag]=baumsuche(daten,bounds)

suchfelder=[];
flag=0;

zuweisung=1;
while zuweisung
    zuweisung=0;
    minbounds=10;
    minboundfeld=[];
    minboundzahlen=[];
    for k=1:81
        if daten(k) == 0
            numbounds=0;
            zahl=[];
            for m=1:9
                if bitget(bounds(k),m)
                    numbounds=numbounds+1;
                    zahl=[zahl m];%#ok
                end
            end
            if numbounds == 0
                return
            elseif numbounds == 1
                daten(k)=zahl;
                zeile=floor((k-1)/9)+1;
                spalte=k-(floor((k-1)/9)*9);
                for m=1:9
                    % Ziffern in der Zeile drfen nicht mehr gesetzt werden
                    bounds((zeile-1)*9+m)= bitset(bounds((zeile-1)*9+m),zahl,0);
                    % Ziffern in der Spalte drfen nicht mehr gesetzt werden
                    bounds((m-1)*9+spalte)= bitset(bounds((m-1)*9+spalte),zahl,0);
                end
                zeileblock=floor((zeile-1)/3)+1;
                spalteblock=floor((spalte-1)/3)+1;
                for n=1:3
                    for m=1:3
                        bounds((((zeileblock-1)*3+(n-1))*9+((spalteblock-1)*3+m)))= ...
                            bitset(bounds((((zeileblock-1)*3+(n-1))*9+((spalteblock-1)*3+m))),zahl,0);
                    end
                end
                zuweisung=1;
            else
                if numbounds < minbounds
                    minbounds=numbounds;
                    minboundfeld=k;
                    minboundzahlen=zahl;
                end
            end
        end
    end
end

% jetzt kommt die eigentliche Baumsuche
if ~all(daten ~= 0)
    for k=minboundzahlen
        bounds2=bounds;
        daten2=daten;
        daten2(minboundfeld)=k;
        zeile=floor((minboundfeld-1)/9)+1;
        spalte=minboundfeld-(floor((minboundfeld-1)/9)*9);
        for m=1:9
            % Ziffern in der Zeile drfen nicht mehr gesetzt werden
            bounds2((zeile-1)*9+m)= bitset(bounds2((zeile-1)*9+m),k,0);
            % Ziffern in der Spalte drfen nicht mehr gesetzt werden
            bounds2((m-1)*9+spalte)= bitset(bounds2((m-1)*9+spalte),k,0);
        end
        zeileblock=floor((zeile-1)/3)+1;
        spalteblock=floor((spalte-1)/3)+1;
        for n=1:3
            for m=1:3
                bounds2((((zeileblock-1)*3+(n-1))*9+((spalteblock-1)*3+m)))= ...
                    bitset(bounds2((((zeileblock-1)*3+(n-1))*9+((spalteblock-1)*3+m))),k,0);
            end
        end
        [suchfelder,flag]=baumsuche(daten2,bounds2);
        if flag
            suchfelder=[suchfelder; minboundfeld k];
            break
        end
    end
end

if all(daten ~= 0)
    flag=1;
end

