%sudoku
% Sudoku solver. A Sudoku is a number puzzle based upon a 9x9 grid, which
% is itself comprised of 3x3 subgrids of 3x3 numbers. All rows, columns and
% subgrids contain (in a solved Sudoku) the integers 1,2,3...9 without
% repeats. This app assists the manual solution of a Sudoku. Each time the
% SOLVE button is pressed, a list of possible integers for each unfilled
% square are determined and dispayed. Further pressing of the SOLVE button
% will use any deductions (i.e. single value possibles), detect any pairs
% etc and hence, by looping the logical process, enable the Sudoku to be
% solved (unless guesswork is required).
%
% A Sudoku can also be generated, as well as manually inputted.
%
% LAST UPDATED by Andy French. December 2024.

function varargout = sudoku(varargin)

%Auto-generated code from GUIDE. Don't edit!
gui_Singleton = 1;
gui_State = struct('gui_Name',       mfilename, ...
    'gui_Singleton',  gui_Singleton, ...
    'gui_OpeningFcn', @sudoku_OpeningFcn, ...
    'gui_OutputFcn',  @sudoku_OutputFcn, ...
    'gui_LayoutFcn',  [] , ...
    'gui_Callback',   []);
if nargin && ischar(varargin{1})
    gui_State.gui_Callback = str2func(varargin{1});
end
if nargout
    [varargout{1:nargout}] = gui_mainfcn(gui_State, varargin{:});
else
    gui_mainfcn(gui_State, varargin{:});
end

%%

% Executes just before sudoku is made visible.
function sudoku_OpeningFcn(hObject, eventdata, handles, varargin)

% Choose default command line output for sudoku
handles.output = hObject;

% Update handles structure
guidata(hObject, handles);

%Define a global variable d to store app data
global d; d.h = handles;

%Default Sudoku from https://sandiway.arizona.edu/sudoku/examples.html
d.S = [
    0,0,0,2,6,0,7,0,1;
    6,8,0,0,7,0,0,9,0;
    1,9,0,0,0,4,5,0,0;
    8,2,0,1,0,0,0,4,0;
    0,0,4,6,0,2,9,0,0;
    0,5,0,0,0,3,0,2,8;
    0,0,9,3,0,0,0,7,4;
    0,4,0,0,5,0,0,3,6;
    7,0,3,0,1,8,0,0,0
    ];

%Initialise array of possibles and deductions
d.P = cell(9,9); d.D = zeros(9,9); d.Solution = d.S; d.Sstart = d.S;

%Default Sudoku counter
d.default_num = 1;

%Update Sudoku GUI
update_sudoku

%Save Sudoku
S = d.S; P = d.P; D = d.D; Sstart = d.Sstart; Solution = d.Solution;
save 'sudoku_data' S P D Sstart Solution

%%

% Outputs from this function are returned to the command line.
function varargout = sudoku_OutputFcn(hObject, eventdata, handles)

% Get default command line output from handles structure
varargout{1} = handles.output;

%%

%Callback function for the PRINT (P) pushbutton
function PUSHprint_Callback(hObject, eventdata, handles)
global d;
plot_sudoku(d.S,'not solution')
plot_sudoku(d.Solution,'solution')

%%

%Callback function for the LOAD (L) pushbutton
function PUSHload_Callback(hObject, eventdata, handles)
global d;
[FILENAME, PATHNAME, FILTERINDEX] = uigetfile('.mat', 'Load Sudoku data');
if FILENAME~=0
    load([PATHNAME,'\',FILENAME]);
    d.S = S; d.Start = Sstart; d.Solution = Solution; d.P = P; d.D = D;
end
update_sudoku

%%

%Callback function for the SAVE (S) pushbutton
function PUSHsave_Callback(hObject, eventdata, handles)
global d;

%Save Sudoku
[FILENAME, PATHNAME, FILTERINDEX] = uiputfile('.mat', 'Save Sudoku data');
if FILENAME~=0
    S = d.S; P = d.P; D = d.D; Sstart = d.Sstart; Solution = d.Solution;
    save([PATHNAME,'\',FILENAME], 'S', 'P', 'D', 'Sstart', 'Solution')
end

%%

%Callback function for the CLEAR pushbutton
function PUSHclear_Callback(hObject, eventdata, handles)
global d;
%Set a blank Sudoku
d.S = zeros(9,9); d.Sstart = d.S; d.Solution = d.S; d.P = cell(9,9); d.D = zeros(9,9);

%Update Sudoku GUI
update_sudoku

%%

%Callback function for the DEFAULT pushbutton
function PUSHdefault_Callback(hObject, ~, handles)
global d;

%Default Sudokus from https://sandiway.arizona.edu/sudoku/examples.html
d.default_num = d.default_num+1;
if d.default_num>5; d.default_num=1; end
if d.default_num==1
    %Easiest
    d.S = [
        0,0,0,2,6,0,7,0,1;
        6,8,0,0,7,0,0,9,0;
        1,9,0,0,0,4,5,0,0;
        8,2,0,1,0,0,0,4,0;
        0,0,4,6,0,2,9,0,0;
        0,5,0,0,0,3,0,2,8;
        0,0,9,3,0,0,0,7,4;
        0,4,0,0,5,0,0,3,6;
        7,0,3,0,1,8,0,0,0
        ];
elseif d.default_num==2
    %Intermediate
    d.S = [
        0,2,0,6,0,8,0,0,0;
        5,8,0,0,0,9,7,0,0;
        0,0,0,0,4,0,0,0,0;
        3,7,0,0,0,0,5,0,0;
        6,0,0,0,0,0,0,0,4;
        0,0,8,0,0,0,0,1,3;
        0,0,0,0,2,0,0,0,0;
        0,0,9,8,0,0,0,3,6;
        0,0,0,3,0,6,0,9,0
        ];
elseif d.default_num==3
    %Difficult
    d.S = [
        0,0,0,6,0,0,4,0,0;
        7,0,0,0,0,3,6,0,0;
        0,0,0,0,9,1,0,8,0;
        0,0,0,0,0,0,0,0,0;
        0,5,0,1,8,0,0,0,3;
        0,0,0,3,0,6,0,4,5;
        0,4,0,2,0,0,0,6,0;
        9,0,3,0,0,0,0,0,0;
        0,2,0,0,0,0,1,0,0
        ];
elseif d.default_num==4
    %The Week 28 Dec 2024
    d.S = [
        7,0,0,0,2,6,0,0,0;
        0,4,0,3,0,0,0,6,0;
        0,8,0,4,0,0,5,0,0;
        0,0,0,0,0,0,0,1,9;
        1,0,0,0,0,0,0,0,6;
        5,7,0,0,0,0,0,0,0;
        0,0,9,0,0,2,0,8,0;
        0,3,0,0,0,1,0,2,0;
        0,0,0,5,7,0,0,0,3
        ];
else
    %Blank
    d.S = [
        0,0,0,0,0,0,0,0,0;
        0,0,0,0,0,0,0,0,0;
        0,0,0,0,0,0,0,0,0;
        0,0,0,0,0,0,0,0,0;
        0,0,0,0,0,0,0,0,0;
        0,0,0,0,0,0,0,0,0;
        0,0,0,0,0,0,0,0,0;
        0,0,0,0,0,0,0,0,0;
        0,0,0,0,0,0,0,0,0
        ];
end

%Initialise array of possibles and deductions
d.Solution = d.S; d.Sstart = d.S;
[d.P,d.SS,d.D] = possibles(d.S);

%Update Sudoku GUI
update_sudoku

%%

%Callback function for the RESET pushbutton
function PUSHreset_Callback(hObject, eventdata, handles)
global d;
%Reset Sudoku
d.S = d.Sstart; [d.P,d.SS,d.D] = possibles(d.S);

%Update Sudoku GUI
update_sudoku

%%

%Callback function for the GENERATE pushbutton
function PUSHgenerate_Callback(hObject, eventdata, handles)
global d;

%Set number of blanks, out of 81, (to be allocated randomly).
N = randi( [30,55] );

%Generate Sudoku (using Dario Fasino's quick way using groups)
d.S = gen_sudoku;

%%% MY REALLY SLOW WAY OF DOING IT! %%%

%Do this by randomly permuting the integers 1:9 for each
%row, and then checking the Sudoku has no errors

% d.S = zeros(9,9); errors = 1;
% for r=1:9
%     while errors == 1
%         d.S(r,:) = randperm(9);
%         errors = check_S_for_errors(d.S);
%     end
%     errors = 1;
%     set(d.h.TEXTstatus,'string',['Rows ',num2str(r), ' of 9 generated ...']);
%     drawnow;
% end

%Assign generated sudoku to be the solution
d.Solution = d.S; set(d.h.TEXTstatus,'string','New Sudoku generated.');

%Randomly set N elements of S to be 0
i = randperm(81); d.S(i(1:N)) = 0;  d.Sstart = d.S;

%Initialise array of possibles and deductions
[d.P,d.SS,d.D] = possibles(d.S);

%Update Sudoku GUI
update_sudoku;

%Save Sudoku
S = d.S; P = d.P; D = d.D; Sstart = d.Sstart; Solution = d.Solution;
save 'sudoku_data' S P D Sstart Solution

%%

%Callback function for the SOLUTION checkbox
function CHECKsolution_Callback(hObject, eventdata, handles)
global d

if get( d.h.CHECKsolution,'value')==1
    %Step through Sudoku elements and set formatting
    for r=1:9
        for c=1:9
            %Determine EDIT box tag string
            T = ['EDIT',num2str(r),num2str(c)];
            
            %Display Sudoku solution
            if d.Solution(r,c)==0
                eval( ['set(d.h.',T,',''BackgroundColor''',',[1,1,0] );'] );
                eval( ['set(d.h.',T,',''string''',','''' );'] );
            else
                eval( ['set(d.h.',T,',''BackgroundColor''',',[1,1,1] );'] );
                eval( ['set(d.h.',T,',''string''',',num2str( d.Solution(r,c) ) );'] );
                eval( ['set(d.h.',T,',''fontsize''',',18,''FontWeight'',''Bold'');'] );
                if d.Sstart(r,c)~=0
                    eval( ['set(d.h.',T,',''ForegroundColor''',',[1,0,0] );'] );
                else
                    eval( ['set(d.h.',T,',''ForegroundColor''',',[0,0,0] );'] );
                end
            end
            eval( ['set(d.h.',T,',''style''',',''edit'' );'] );
        end
    end
else
    update_sudoku;
end

%%

%Callback function for the SOLVE pushbutton
function PUSHsolve_Callback(hObject, eventdata, handles)
global d

%If Edit toggle is enabled, update Sudoku. But only if no errors.
if get(d.h.TOGGLEedit,'value')==1
    S = zeros(9,9);
    for r=1:9
        for c=1:9
            %Determine EDIT box tag string
            T = ['EDIT',num2str(r),num2str(c)];
            
            %Get updated value of Suduko
            eval( ['s = str2num( get(d.h.',T,',''string''));'] );
            if numel(s)~=1; S(r,c) = 0; else S(r,c) = s; end
            
            %Set cell to be inactive
            eval( ['set(d.h.',T,',''Enable''',',''inactive'' );'] );
        end
    end
    if check_S_for_errors(S)==0
        d.S = S;
        set(d.h.TEXTstatus,'string','Edit OK for now ...');
    else
        set(d.h.TEXTstatus,'string','Bad edit, resetting');
    end
    set(d.h.TOGGLEedit,'value',0);
end

%Compute cell array of possible integers for each element of S
[d.P,d.SS,d.D] = possibles(d.S);

%Update Sudoku with any deductions
d.S = d.SS;

%Update Sudoku GUI
update_sudoku

%Save Sudoku
S = d.S; P = d.P; D = d.D; Sstart = d.Sstart; Solution = d.Solution;
save 'sudoku_data' S P D Sstart Solution

%%

%Callback function for the EDIT toggle
function TOGGLEedit_Callback(hObject, eventdata, handles)
global d
for r=1:9
    for c=1:9
        %Determine EDIT box tag string
        T = ['EDIT',num2str(r),num2str(c)];
        
        %Allow cells to be editable
        if get( d.h.TOGGLEedit,'value' )==1
            eval( ['set(d.h.',T,',''style''',',''edit'' );'] );
            eval( ['set(d.h.',T,',''Enable''',',''on'' );'] );
        else
            eval( ['set(d.h.',T,',''Enable''',',''inactive'' );'] );
            if numel( d.P{r,c} ) > 3
                eval( ['set(d.h.',T,',''style''',',''text'' );'] );
            end
        end
    end
end

%%

%Callback function for the SHOW NOTES toggle
function TOGGLEnotes_Callback(hObject, eventdata, handles)
global d
[d.P,d.SS,d.D] = possibles(d.S); %Compute cell array of possible integers for each element of S
update_sudoku

%%

%Update GUI with Sudoku entries
function update_sudoku
global d

%Step through Sudoku elements and set formatting
for r=1:9
    for c=1:9
        %Determine EDIT box tag string
        T = ['EDIT',num2str(r),num2str(c)];
        
        %Set EDIT box to be a static text box (to allow for multi-line
        %possibilities) if number of possibilities is more than 3
        if ( numel( d.P{r,c} ) > 3 ) && ( get(d.h.TOGGLEedit,'value') == 0 ) && ( get( d.h.TOGGLEnotes,'value')==1 )
            eval( ['set(d.h.',T,',''style''',',''text'' );'] );
            eval( ['set(d.h.',T,',''ForegroundColor''',',[0,0,0] );'] );
        else
            eval( ['set(d.h.',T,',''style''',',''edit'' );'] );
        end
        
        %Display possibilities, or known Sudoku elements
        if ( d.S(r,c)==0 ) && ( get( d.h.TOGGLEnotes,'value')==1 )
            %Unknown Sudoku elements, so display possibles
            eval( ['set(d.h.',T,',''string''',',num2str( d.P{r,c} ) );'] );
            eval( ['set(d.h.',T,',''fontsize''',',10,''FontWeight'',''normal'');'] );
        elseif ( d.S(r,c)==0 ) && ( get( d.h.TOGGLEnotes,'value')==0 )
            %Unknown Sudoku elements, so display possibles
            eval( ['set(d.h.',T,',''string''',','''' );'] );
        elseif d.S(r,c)~=0
            %Display current state of Sudoku known element
            eval( ['set(d.h.',T,',''string''',',num2str( d.S(r,c) ) );'] );
            eval( ['set(d.h.',T,',''fontsize''',',18,''FontWeight'',''Bold'');'] );
        end
        
        %Set default Sudoku element background and foreground colour
        eval( ['set(d.h.',T,',''ForegroundColor''',',[0,0,0] );'] );
        eval( ['set(d.h.',T,',''BackgroundColor''',',[1,1,1] );'] );
        if d.S(r,c)==0
            eval( ['set(d.h.',T,',''BackgroundColor''',',[1,1,0] );'] );
        end
        if d.Sstart(r,c)~=0
            eval( ['set(d.h.',T,',''ForegroundColor''',',[1,0,0] );'] );
        end
        
        %Highlight any new deductions in green, otherwise set known Sudoku elements
        %to be white in background colour, and yellow if unknown. Set any
        %pairs to be blue, and triples to be magenta
        
        if get( d.h.TOGGLEnotes,'value')==1
            if d.D(r,c)==1
                eval( ['set(d.h.',T,',''BackgroundColor''',',[0,1,0] );'] );
            elseif d.D(r,c)==2
                eval( ['set(d.h.',T,',''BackgroundColor''',',[0,0,1] );'] );
                eval( ['set(d.h.',T,',''ForegroundColor''',',[1,1,1] );'] );
            elseif d.D(r,c)==3
                eval( ['set(d.h.',T,',''BackgroundColor''',',[1,0,1] );'] );
            end
        end
        
        %Don't allow cell to be editable if E toggle button is off
        if get( d.h.TOGGLEedit,'value')==0
            eval( ['set(d.h.',T,',''Enable''',',''inactive'' );'] );
        end
    end
end

%Check if Sudoku is solved. If so. update status box.
S = d.S; S(S==0) = []; %Remove zeros from S
if numel(S)==81;
    set( d.h.TEXTstatus,'string', 'Sudoku solved!')
else
    set( d.h.TEXTstatus,'string', '')
end

%%

%Determine a 9x9 cell array P of possible integers for each element of Sudoku
%S. The array SS is an updated Sudoku, incorporating any deductions. These
%correspond to ones in a binary array D, and correspond to single-element
%vectors of the elements of P. Twos in D correspond to any pairs in P, and
%threes any triples
function [P,SS,D] = possibles(S)
P = cell(9,9); SS = S; D = zeros(9,9);
for r=1:9
    for c=1:9
        
        %Get Horizontal, Vertical and 3x3 subgrid sets which intersect with
        %cell S(r,c)
        [H,V,U] = get_HVU(r,c,S);
        
        %Unless cell has an already known value, determine intersection of
        %the set of integers from {1...9} that are NOT already present in
        %the H,V and U sets. These are the possibles for cell (r,c)
        if S(r,c)~=0;
            %Sudoku element is known, so possibles element is Sudoku value
            P{r,c} = S(r,c);
        else
            %Sudoku element is not already known, so determine a vector of
            %possible values and store these in element r,c of cell array P
            P{r,c} = setdiff( setdiff( setdiff( 1:9,H ),V ), U );
        end
    end
end

%Tips from https://sudoku.com/sudoku-rules ...

%Check for obvious pairs. Check H,V,U sets for (only) two cells that contain the
%same pair of possibles. e.g. 1,2 appear in just two of the nine cells in a
%H,V or U set, and these cells ONLY contain 1,2. This means the pairs can't
%be possibles in the other cells in the set.
for r=1:9
    for c=1:9
        %Get Horizontal, Vertical and 3x3 subgrid sets which intersect with
        %cell P(r,c)
        [H,V,U] = get_HVU(r,c,P);
        
        %Is cell P{r,c} a pair of possibles?
        if numel( P{r,c} )==2
            
            %Check H set
            paircount = 0;
            for n=1:length(H)
                if isequal( sort( P{r,c} ), sort( H{n} ) )==1
                    paircount = paircount+1;
                end
            end
            
            %Remove possibles from cells that don't equal the pair
            if paircount==2
                for n=1:9
                    if isequal( sort( P{r,c} ), sort( H{n} ) )==0
                        k = P{r,n}; k(k==P{r,c}(1)) = []; k(k==P{r,c}(2)) = [];
                        P{r,n} = k;
                    end
                end
            end
            
            %Check V set
            paircount = 0;
            for n=1:length(V)
                if isequal( sort( P{r,c} ), sort( V{n} ) )==1
                    paircount = paircount+1;
                end
            end
            
            %Remove possibles from cells that don't equal the pair
            if paircount==2
                for n=1:9
                    if isequal( sort( P{r,c} ), sort( V{n} ) )==0
                        k = P{n,c}; k(k==P{r,c}(1)) = []; k(k==P{r,c}(2)) = [];
                        P{n,c} = k;
                    end
                end
            end
            
            %Check U set
            paircount = 0;
            for n=1:length(U)
                if isequal( sort( P{r,c} ), sort( U{n} ) )==1
                    paircount = paircount+1;
                end
            end
            
            %Determine 3x3 subgrid row and column
            rs = ceil(r/3); cs = ceil(c/3);
            
            %Determine subgrid indices
            subgridrows = 3*(rs-1) + 1 : 3*rs;
            subgridcols = 3*(cs-1) + 1 : 3*cs;
            
            %Remove possibles from cells that don't equal the pair
            if paircount==2
                for i=1:3
                    for j=1:3
                        if isequal( sort( P{r,c} ), sort( P{subgridrows(i),subgridcols(j)} ) )==0
                            k=P{subgridrows(i),subgridcols(j)}; k(k==P{r,c}(1)) = []; k(k==P{r,c}(2)) = [];
                            P{subgridrows(i),subgridcols(j)} = k;
                        end
                    end
                end
            end
        end
    end
end

%Check for obvious triples. Three cells in a H, V or U set are pairs and
%are from just three numbers. This means these numbers can't be possibles
%in the other elements of the sets.

%Check for hidden singles. This is where a possible appears just once in a
%H,V or U set. In this case we have a deduction!

%Check for hidden pairs. Within a H,V or U set, only two cells contain the
%same pair of possibles, which don't appear anywhere else. Remove any extra
%possibles from these cells.

%Check for hidden triples. Within a H,V or U set, only three cells contain the
%same trio of possibles, which don't appear anywhere else. Remove any extra
%possibles from these cells.

%Check for pointing pairs. This is where two possibles are in the same row
%or column of a U set, but nowhere else in this U set. If this is the case,
%remove possibles from U sets with the same row (or column).

%Check for pointing triples. This is where three possibles are in the same row
%or column of a U set, but nowhere else in this U set. If this is the case,
%remove possibles from U sets with the same row (or column).

%Determine any deductions
for r=1:9
    for c=1:9
        D(r,c) = numel( P{r,c} );
        if D(r,c)==1
            %If the number of possible values is one, then a deduction
            %is made! Update S accordingly.
            SS(r,c) = P{r,c};
        end
    end
end

%%

%Check whether sudoku array 9x9 array S has any mistakes. i.e. in all H,V
%and U sets, no integers (apart from 0) are repeated.
function errorcheck = check_S_for_errors(S)
errorcheck = 0; %Assume no mistakes
for r=1:9
    for c=1:9
        
        %Check that S(r,c) is not empty
        if isempty(S(r,c)); errorcheck = 1; return; end
        
        %Determine 1x9 row vectors of elements of Sudoku S that correspond to
        %Vertical (V) columns and Horizontal (H) rows which intersect at S element (r,c).
        %Note (1,1) is the top,left corner of the Sudoku S.
        H = S(r,:); V = S(:,c).';
        
        %Determine 3x3 subgrid row and column
        rs = ceil(r/3); cs = ceil(c/3);
        
        %Determine subgrid that intersects with (r,c)
        U = S( 3*(rs-1) + 1 : 3*rs, 3*(cs-1) + 1 : 3*cs );
        U = reshape(U,[1,9]);
        
        %Remove zeros from H,V,U sets
        H(H==0)=[]; V(V==0) = []; U(U==0) = [];
        
        %Check for duplication in H,V and U sets
        if numel(H)> numel( unique(H) ); errorcheck = 1; return; end
        if numel(V)> numel( unique(V) ); errorcheck = 1; return; end
        if numel(U)> numel( unique(U) ); errorcheck = 1; return; end
    end
end

%%

%gen_sudoku
% Uses a group-theoretic approach to generate Sudoku matrices. Not all
% Sudoku matrices can be generated in this way,but it's a quite large
% number. Based on code by Dario Fasino Dec 2005.
% http://www.dimi.uniud.it/fasino
% https://uk.mathworks.com/matlabcentral/fileexchange/9462-sudoku_lite
function sol = gen_sudoku

S = [1 2 3 ; 4 5 6 ; 7 8 9];
C = [7 8 9 1 2 3 4 5 6];
R = [3 1 2 6 4 5 9 7 8];
P = [1 2 3 4 5 6 7 8 9];
if rand < .5
    P(1) = 2; P(2) = 1;
    if rand < .5, P(3) = 6; P(6) = 3; end
    if rand < .5, P(8) = 9; P(9) = 8; end
    if rand < .5, P(4) = 7; P(7) = 4; end
else
    P(1) = 4; P(4) = 1;
    if rand < .5, P(9) = 6; P(6) = 9; end
    if rand < .5, P(8) = 7; P(7) = 8; end
    if rand < .5, P(2) = 5; P(5) = 2; end
end
C = P(C(P));
R = P(R(P));
sol = [S C(S) C(C(S))];
sol = [sol ; R(sol) ; R(R(sol))];
sol = sol([randperm(3) randperm(3)+3 randperm(3)+6], ...
    [randperm(3) randperm(3)+3 randperm(3)+6]);
per = randperm(9);
sol = per(sol);

%%

%Get H,V,U subsets of a sudoku
function [H,V,U] = get_HVU(r,c,S)
%Determine 1x9 row vectors of elements of Sudoku S that correspond to
%Vertical (V) columns and Horizontal (H) rows which intersect at S element (r,c).
%Note (1,1) is the top,left corner of the Sudoku S.
H = S(r,:); V = S(:,c).';

%Determine 3x3 subgrid row and column
rs = ceil(r/3); cs = ceil(c/3);

%Determine subgrid that intersects with (r,c)
U = S( 3*(rs-1) + 1 : 3*rs, 3*(cs-1) + 1 : 3*cs );
U = reshape(U,[1,9]);

%%

%Create new figure and axes
function plot_sudoku(S,is_solution)
fig = figure; ax = axes('nextplot','add');
xlim([0,10]); ylim([0,10]); axis off; axis equal;

%Add Sudoku text, enclosed by a box
for r=1:9
    for c=1:9
        xc = c; yc = 10-r;
        if S(r,c)~=0
            text( xc, yc, num2str( S(r,c) ),...
                'fontsize',24,'fontweight','bold','horizontalalignment','center' );
        end
        plot( [xc-0.5,xc+0.5,xc+0.5,xc-0.5,xc-0.5],...
            [yc-0.5,yc-0.5,yc+0.5,yc+0.5,yc-0.5],'linewidth',1,'color',0.3*[1,1,1]);
    end
end

%Plot 3x3 squares
for r=1:3
    for c=1:3
        xc = 2+3*(c-1); yc = 2+3*(r-1);
        plot( [xc-1.5,xc+1.5,xc+1.5,xc-1.5,xc-1.5],...
            [yc-1.5,yc-1.5,yc+1.5,yc+1.5,yc-1.5],'linewidth',3,'color',[0,0,0]);
    end
end

%Print and close
if strcmp( is_solution, 'solution' )
    print(gcf,['sudoku SOLUTION ',strrep( datestr(now),':','-' ),'.png'],'-dpng','-r300');
else
    print(gcf,['sudoku ',strrep( datestr(now),':','-' ),'.png'],'-dpng','-r300');
end
close(fig);

%End of code

