%factorizer
% GUI which randomly generates polynomials that can be factorized.
%
% LAST UPDATED by Andy French May 2017.

function varargout = factorizer(varargin)

%% GUI initialization code %%
gui_Singleton = 1;
gui_State = struct('gui_Name',       mfilename, ...
    'gui_Singleton',  gui_Singleton, ...
    'gui_OpeningFcn', @factorizer_OpeningFcn, ...
    'gui_OutputFcn',  @factorizer_OutputFcn, ...
    'gui_LayoutFcn',  [] , ...
    'gui_Callback',   []);
if nargin && ischar(varargin{1})
    gui_State.gui_Callback = str2func(varargin{1});
end

if nargout
    [varargout{1:nargout}] = gui_mainfcn(gui_State, varargin{:});
else
    gui_mainfcn(gui_State, varargin{:});
end

%% GUI opensing function %%
function factorizer_OpeningFcn(hObject, eventdata, handles, varargin)

% Choose default command line output for slope
handles.output = hObject;

%Turn off warnings (e.g. 'divide by zero')
warning off

%Set up global variable d which can be accessed by all functions
global d
d.h = handles;

%Centre figure
figcentre(gcf);

%Polynomial menu
d.poly_menu = {
    'Quadratic',...
    'Cubic'
    };
set( d.h.MENUpoly,'string',d.poly_menu,'value',1 );
d.poly = d.poly_menu{1};

%Create random polynomial and store this in the GUI data
[d.expanded_str,d.factorized_str,d.pf,d.pe] = gen_poly(d.poly);

%Update the text boxes
d.expanded_text_box = annotation(gcf,'textbox', [0.024 0.494 0.948 0.173],'string',d.expanded_str,...
    'fitboxtotext','off','FontSize',30,'edgecolor','none','HorizontalAlignment','center','color',[1,0,0]);
d.factorized_text_box = annotation(gcf,'textbox', [0.024 0.166 0.948 0.173],'string','y = ...',...
    'fitboxtotext','off','FontSize',30,'edgecolor','none','HorizontalAlignment','center','color',[0,0,1]);

%%

%% GUI output function %%
function varargout = factorizer_OutputFcn(hObject, eventdata, handles)

%%

%% Callback for generate new quadratic pushbutton %%
function PUSHnewpoly_Callback(hObject, eventdata, handles)
global d

%Create random polynomial and store this in the GUI data
[d.expanded_str,d.factorized_str,d.pf,d.pe,d.factor] = gen_poly(d.poly);

%Update the text boxes
set( d.expanded_text_box,'string',d.expanded_str );

%If Cubic option set, provide one of the factors
if strcmp( d.poly, 'Cubic' )
    set( d.factorized_text_box,'string',['y = ',d.factor] );
else
    set( d.factorized_text_box,'string','y = ...' );
end

%Plot polynomial if check-box ticked
if get( d.h.CHECKplotgraph, 'value' ) == 1
    plot_poly;
end

%%

%% Factorize current quadratic pushbutton %%
function PUSHfactorize_Callback(hObject, eventdata, handles)
global d

%Update the factorized polynomial text box
set( d.factorized_text_box,'string',d.factorized_str );

%%

%% Plot polynomial checkbox
function CHECKplotgraph_Callback(hObject, ~, handles)
global d

%Plot polynomial if check-box ticked
if get( d.h.CHECKplotgraph, 'value' ) == 1
    plot_poly;
end

%%

%% Choose polynomial menu
function MENUpoly_Callback(hObject, eventdata, handles)
global d

%Update d.poly from menu
d.poly = d.poly_menu{ get(d.h.MENUpoly,'value') };

%Create random polynomial and store this in the GUI data
[d.expanded_str,d.factorized_str,d.pf,d.pe,d.factor_str] = gen_poly(d.poly);

%Update the text boxes
set( d.expanded_text_box,'string',d.expanded_str );

%If Cubic option set, provide one of the factors
if strcmp( d.poly, 'Cubic' )
    set( d.factorized_text_box,'string',['y = ',d.factor_str]);
else
    set( d.factorized_text_box,'string','y = ...' );
end

%Plot polynomial if check-box ticked
if get( d.h.CHECKplotgraph, 'value' ) == 1
    plot_poly;
end

%%

%% Generate random factorizable polynomial %
function [expanded_str,factorized_str,poly_fact,poly_exp,factor_str] = gen_poly(poly)

%Choose coefficients of factors e.g. a,b in (ax+b) from the following prime
%numbers, and unity
coeffs = [1,2,3,5,7];

%Determine polynomial in factorized form
if strcmp(poly,'Quadratic')==1
    
    % y = S(Ax + B)(Cx + D)  (S is 1 or -1)
    
    %Choose four random elements from coeffs
    I = randi(numel(coeffs),1,4);
    
    %Choose signs
    s = randi(2,1,3);
    s(s==2) = -1;
    
    %Assign factorized quadratic coefficients
    A = coeffs(I(1));
    B = s(2)*coeffs(I(2));
    C = coeffs(I(3));
    D = s(3)*coeffs(I(4));
    poly_fact = [s(1),A,B,C,D];
    
    %Determine expanded form coefficients
    a = s(1)*A*C;
    b = s(1)*(B*C+D*A);
    c = s(1)*B*D;
    poly_exp = [a,b,c];
    
    %Create string for factorized form
    if s(1)==-1
        factorized_str = ['y = -(',num2str(A),'x',signstr(B),')(',num2str(C),'x',signstr(D),')'];
    else
        factorized_str = ['y = (',num2str(A),'x',signstr(B),')(',num2str(C),'x',signstr(D),')'];
    end
    factor_str = '';
    
    %Create string for expanded form
    expanded_str = ['y = ',num2str(a),'x^2',signstr(b),'x',signstr(c)];
    
elseif strcmp(poly,'Cubic')==1
    
    % y = S(Ax + B)(Cx + D)(Ex + F)  (S is 1 or -1)
    
    %Choose six random elements from coeffs
    I = randi(numel(coeffs),1,6);
    
    %Choose signs
    s = randi(2,1,4);
    s(s==2) = -1;
    
    %Assign factorized quadratic coefficients
    A = coeffs(I(1));
    B = s(2)*coeffs(I(2));
    C = coeffs(I(3));
    D = s(3)*coeffs(I(4));
    E = coeffs(I(5));
    F = s(4)*coeffs(I(6));
    poly_fact = [s(1),A,B,C,D,E,F];
    
    %Determine expanded form coefficients
    a = s(1)*A*C*E;
    b = s(1)*( B*C*E + A*(D*E+C*F) );
    c = s(1)*( B*(D*E+C*F)+A*D*F );
    d = s(1)*B*D*F;
    poly_exp = [a,b,c,d];
    
    %Create string for factorized form
    if s(1)==-1
        factorized_str = ['y = -(',num2str(A),'x',signstr(B),')(',num2str(C),'x',signstr(D),')(',num2str(E),'x',signstr(F),')'];
    else
        factorized_str = ['y = (',num2str(A),'x',signstr(B),')(',num2str(C),'x',signstr(D),')(',num2str(E),'x',signstr(F),')'];
    end
    factor_str = ['(',num2str(A),'x',signstr(B),')'];
    
    %Create string for expanded form
    expanded_str = ['y = ',num2str(a),'x^3',signstr(b),'x^2',signstr(c),'x',signstr(d)]; 
end

%Replace ' 1x' with 'x'
%Need the space to avoid replacing 21x with 2x !!
factorized_str = strrep(factorized_str,' 1x','x');
factor_str = strrep(factor_str,' 1x','x');
expanded_str = strrep(expanded_str,' 1x','x');

%Replace '(1x' with '(x'
factorized_str = strrep(factorized_str,'(1x','(x');
factor_str = strrep(factor_str,'(1x','(x');
expanded_str = strrep(expanded_str,'(1x','(x');

%Replace '-1x' with '-x'
factorized_str = strrep(factorized_str,'-1x','-x');
factor_str = strrep(factor_str,'-1x','-x');
expanded_str = strrep(expanded_str,'-1x','-x');

%Remove any + 0x or - 0x etc
factorized_str = strrep(factorized_str,'+ 0x','');
expanded_str = strrep(expanded_str,'+ 0x','');
factorized_str = strrep(factorized_str,'- 0x','');
expanded_str = strrep(expanded_str,'- 0x','');
factorized_str = strrep(factorized_str,'+ 0x^2','');
expanded_str = strrep(expanded_str,'+ 0x^2','');
factorized_str = strrep(factorized_str,'- 0x^2','');
expanded_str = strrep(expanded_str,'- 0x^2','');

%%

%Create signed string from number x
function s = signstr(x)
if x<0
    s_sign = ' - ';
else
    s_sign = ' + ';
end
s = [s_sign,num2str(abs(x))];

%%

function plot_poly
global d
x = linspace( -8,8,500 );
if strcmp( d.poly, 'Quadratic' )
    F = d.pf(1)*(d.pf(2)*x + d.pf(3)).*(d.pf(4)*x + d.pf(5));
    E = d.pe(1)*x.^2 + d.pe(2)*x + d.pe(3);
elseif strcmp( d.poly, 'Cubic' )    
    F = d.pf(1)*(d.pf(2)*x + d.pf(3)).*(d.pf(4)*x + d.pf(5)).*(d.pf(6)*x + d.pf(7));
    E = d.pe(1)*x.^3 + d.pe(2)*x.^2 + d.pe(3)*x + d.pe(4);
else
    F = NaN*x;
end
fsize = 18;
figure('color',[1 1 1],'name','Plot polynomial');
plot(x,F,x,E);
xlim([-8,8])
xlabel('x','fontsize',fsize )
ylabel('y','fontsize',fsize )
title({[d.expanded_str],[d.factorized_str]},'fontsize',fsize )
legend({'Expanded','Factorized'},'fontsize',fsize );
set(gca,'fontsize',fsize );
grid on

%%

%figcenter
% Centres a figure with handle fig in the current display.
function figcentre(fig)

%Get screen dimensions in pixels.
set(0,'units','pixels') ;
screen_size=get(0,'Screensize');

%Get original figure units
orig_units = get(fig,'units');

%Get position vector for figure in pixels
set(fig,'units','pixels');
fig_position=get(gcf,'position') ;

%Define coordinates or figure bottom hand corner from screen bottom hand corner
%that will place the figure window in the center of the screen.
centre_x = 0.5*screen_size(3) - 0.5*fig_position(3);
centre_y = 0.5*screen_size(4) - 0.5*fig_position(4);

%Modify figure position to centre
set(fig,'position',[centre_x,centre_y,fig_position(3),fig_position(4)] );

%Reset figure units
set(fig,'units',orig_units);

%%

%End of code